/**
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * AppletMathGraph32.java
 *
 * Created on 26 juillet 2004, 14:30
 */

/**
 *
 * @author  Yves
 * @version 
 */
package Applet;

  
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.imageio.ImageIO;
import javax.swing.*;

import netscape.javascript.JSObject;
import org.scilab.forge.jlatexmath.TeXFormula;
import dialogs.*;
import entreesSorties.*;
import glob.Kernel;
import objets.*;
import outils.*;
import types.*;


@SuppressWarnings("serial")
public class AppletMathGraph32 extends JApplet implements MouseListener, KeyListener,
    ActionListener, Runnable {
	public String mtgOnInitParam; // Nouveau version 4.6 permet au javaScript de
	// connatre via mtgOnInit le nom de l'applet qui a appel mtgOnInit()
	public SpringLayout springl; // Ajout version 4.5.2
	public static final Insets insetsNul = new Insets(0,0,0,0);
  private boolean isConstruction; // True si l'applet sert  faire un exercice de construction
  private boolean toolsAvailable;
  private boolean allowTools; // Ajout version 4.7.2.6 . True par dfaut si false, la barre d'outil de gauche ne sera pas permise
	public static String messageCopyright = "Applet MathGraph32 V4.7.7 Copyright (C) Yves Biton 2006 -> Under Gnu GPL3 License";
	Locale locale; // Pour la gestion de la langue
	// Ajout version 4.6.2. isReady sert  savoir si l'applet a bien t affiche
	// Sera mis  true dans paint de MtgFigurePanel quand premierAffichage = true et aussi aprs appel  setScript()
	public boolean isReady;
	public static ResourceBundle messages; // Pour la gestion des messages suivant la langue
  // Variable de classe servant  connatre le nombre d'instances de l'applet
  private static int nbInstances;
  private static AppletManager appletManager;
  // Dialogue servant  l'historique. Sera affich par l'outil outilHistorique
  public HistoriqueDlg historiqueDlg;
  // Variable servant  savoir si le dialogue d'historique est en cours d'utilisation
  public boolean historiqueEnCours = false;
  // Pour les figures de construction, une variable boolean qui contiendra true
  // si les objets dsigns avec l'outil de validation outilPropSol sont bons
  public boolean constructionExacte;
  // Ajout version 3.2.6 : Mode trace pouvant tre activ par une macro
  public boolean modeTrace = false;
  // Pour la version permettant des constructions, utilisation d'un gestionnaire
  // d'dition permettant d'annuler-refaire
  public GestionnaireEdition gestionnaire = new GestionnaireEdition(this);
  // A partir de la version 2.0, l'applet maintient un objet du type CPointBase
  // qui est charg de suivre les mouvements de la souris pour les aides visuelles ?
  // la cration d'objet
  public CPointBase pointSouris;
  // Une ventuelle bote de dialogue servant  modifier les variables pour lesquelles
  // on a coch Dialogue associ en crant la figure
  public PaneAffichageVariables paneAffichageVariables = null;
  // curseurs pointeurs de souris
  public static Cursor handCursor = new Cursor(Cursor.HAND_CURSOR);
  public static Cursor crossHairCursor = new Cursor(Cursor.CROSSHAIR_CURSOR);
  public static Cursor defaultCursor = new Cursor(Cursor.DEFAULT_CURSOR);
	public static Cursor supCursor; // Curseur poubelle
	public static Cursor crayonCursor; // Curseur pour point libre
	public static Cursor modifCursor; // Curseur de modification d'objet graphique
	
	// Une liste contenant les styles de trait
	MtgListeStyleTrait listeStyleTrait;
	
	// Ajout version 4.7.4. Les icones deviennent statiques
	public static ImageIcon iconAnnuler = null;
	public static ImageIcon iconRefaire = null;
	public static ImageIcon iconSupprimerObjetGraphique = null;
	public static ImageIcon iconHistorique = null;
	public static ImageIcon iconNommer = null;
	public static ImageIcon iconCapture = null;
	public static ImageIcon iconCreerPointLibre = null;
	public static ImageIcon iconCreerPointLie = null;
	public static ImageIcon iconCreerMilieu = null;
	public static ImageIcon iconCreerPointParAbscisse = null;
	public static ImageIcon iconCreerPointParCoordonnees = null;
	public static ImageIcon iconModifierObjetGraphique = null;
	public static ImageIcon iconModifierFormule = null;
	public static ImageIcon iconCreerSegment = null;
	public static ImageIcon iconCreerDroiteAB = null;
	public static ImageIcon iconCreerDroiteParallele = null;
	public static ImageIcon iconCreerDroitePerpendiculaire = null;
	public static ImageIcon iconCreerDroiteMediatrice = null;
	public static ImageIcon iconCreerDroiteBissectrice = null;
	public static ImageIcon iconCreerDemiDroite = null;
	public static ImageIcon iconCreerCercleOA = null;
	public static ImageIcon iconCreerCercleOR = null;
	public static ImageIcon iconCreerPolygone = null;
	public static ImageIcon iconCreerIntersection = null;
	public static ImageIcon iconCreerImageParTranslation = null;
	public static ImageIcon iconCreerImageParSymetrieAxiale = null;
	public static ImageIcon iconCreerImageParSymetrieCentrale = null;
	public static ImageIcon iconCreerImageParRotation = null;
	public static ImageIcon iconCreerImageParHomothetie = null;
	public static ImageIcon iconRapporteur = null;
	public static ImageIcon iconCreerCalcul = null;
	public static ImageIcon iconMesureLongueur = null;
	public static ImageIcon iconMesureAngleNonOriente = null;
	public static ImageIcon iconMesureAngleOriente = null;
	public static ImageIcon iconMesureAbscisse = null;


  // Les outils de pointage
  public OutilPointageCapt outilPointageCapture = new OutilPointageCapt(this);
  public OutilPointageClic outilPointageClic = new OutilPointageClic(this);
  public OutilPointageCre outilPointageCre = new OutilPointageCre(this);
  public OutilPointageSup outilPointageSup = new OutilPointageSup(this);
  public OutilPointageRapporteur outilPointageRapporteur = new OutilPointageRapporteur(this);
  public OutilPointageObjetClignotant outilPointageObjetClignotant =
          new OutilPointageObjetClignotant(this);
  public OutilPointagePropSol outilPointagePropSol = new OutilPointagePropSol(this);
  public OutilPointagePourNommer outilPointagePourNommer = new OutilPointagePourNommer(this);
  public OutilPointageHistorique outilPointageHistorique = new OutilPointageHistorique(this);
  public OutilPointageIntersection outilPointageIntersection = new OutilPointageIntersection(this); // Ajout version 3.0
  public OutilPointageModification outilPointageModification = new OutilPointageModification(this);
  // Pointeur sur l'outil de pointage actif qui est l'outil de capture au d?part
  public OutilPointage outilPointageActif = outilPointageCapture;
  // Pointeur sur l'outil de cration ou modification de la figure actif
  // Sera null si on est en mode capture d'objet.
  public Outil outilActif = null;
  // Un JColorChooser qui servira  choisir la couleur active. Il appartient  l'applet
  // car il mmorise les couleurs utilises
  JColorChooser colorChooser;
  // Le nombre de boutons  partir duquel on passe  3 colonnes d'icnes
  private static int nbBoutonsMiniPour3Col = 15;
  private static int nbBoutonsMiniPour4Col = 28;
  // Les boutons de la barre d'outils
  MtgButton buttonCreerCalcul;
  MtgButton buttonAnnuler;
  MtgButton buttonRefaire;
  MtgButton buttonCapt;
  MtgButton buttonPtLib;
  MtgButton buttonMil;
  MtgButton buttonSeg;
  MtgButton buttonDteAB;
  MtgButton buttonDtePer;
  MtgButton buttonDtePar;
  MtgButton buttonDteBis;
  MtgButton buttonDteMed;
  MtgButton buttonDemiDt;
  MtgButton buttonCerOA;
  MtgButton buttonCerOR;
  MtgButton buttonInt;
  MtgButton buttonRapp;
  MtgButton buttonTrans;
  MtgButton buttonSymAxiale;
  MtgButton buttonSymCentrale;
  MtgButton buttonRot;
  MtgButton buttonSup;
  MtgButton buttonPtLie;
  MtgButton buttonPropSol;
  MtgButton buttonPtParAbs;
  MtgButton buttonHom;
  MtgButton buttonMesLong;
  MtgButton buttonMesAngNor;
  MtgButton buttonMesAngOr;
  MtgButton buttonMesAbs;
  MtgButton buttonPolygone;
  MtgButton buttonHistorique;
  MtgButton buttonNommer;
  MtgButton buttonPointParCoordonnees;
  MtgButton buttonModifierObjetGraphique;
  MtgButton buttonModifierFormule;
  MtgColorButton colorButton;
  // Un groupe de boutons englobant tous ces boutons
  ButtonGroup btngroup;
  // Une zne de texte de deux lignes en bas de la fentre pour les indications
  JTextArea indicationArea = new JTextArea(1, 60);
  
  // Les barres devant contenir les icnes. Il n'y en aura deux, trois ou quatre
  // suivant le nombre d'icnes
  JToolBar[] barIcones = new JToolBar[4];
  // Le nombre de colonnes contenant les icones
  int nbColonnesIcones;
  public JPanel paneLow = new JPanel();
  
  // Les panneaux contenant lers composants et la figure
  JPanel panePrincipal = new JPanel(); // Le panneau principal.
  JPanel paneLeftHigh = new JPanel();
  JPanel paneLeftLow = new JPanel();
  JPanel paneLeft = new JPanel();
  JPanel paneSouth = new JPanel();
  JLayeredPane paneCenter = new JLayeredPane();
  public MtgPaneFigure paneFigure = new MtgPaneFigure(this);
  int decalageBordGauche = 0; // Dcalage de la fentre contenant la figure par rapport
    // au bord gauche de la fentre
  int decalageBordBas = 0;
  // Outils de cration et modification
  OutilAnnuler outilAnnuler;
  OutilRefaire outilRefaire;
  OutilCapture outilCapture;
  OutilPtLib outilPtLib;
  OutilMilieu outilMilieu;
  OutilDtAB outilDtAB;
  OutilDtPar outilDtPar;
  OutilDtPer outilDtPer;
  OutilSeg outilSeg;
  OutilDemiDt outilDemiDt;
  OutilCerOA outilCerOA;
  OutilBis outilBis;
  OutilMed outilMed;
  public OutilInt outilInt;
  OutilTranslation outilTrans;
  OutilSymAxiale outilSymAxiale;
  OutilSymCentrale outilSymCentrale;
  public OutilRapporteur outilRapporteur;
  OutilRot outilRot;
  OutilCerOR outilCerOR;
  OutilSup outilSup;
  OutilPtLie outilPtLie;
  OutilCalcul outilCalcul;
  OutilPtParAbs outilPtParAbs;
  OutilHom outilHom;
  OutilMesLong outilMesLong;
  OutilMesAngNor outilMesAngnor;
  OutilMesAngOr outilMesAngOr;
  OutilMesAbs outilMesAbs;
  OutilPolygone outilPolygone;
  OutilPointParCoordonnees outilPointParCoordonnees;
  public OutilModifierBarycentre outilModifierBarycentre;
  OutilModifierObjetGraphique outilModifierObjetGraphique;
  OutilModifierFormule outilModifierFormule;
  // Outil servant  faire apparatre le dialogue d'historique
  public OutilHistorique outilHistorique;
  // Outil servnt  nommer un point ou une droite
  OutilEntrerNom outilEntreeNom;
  // Outil servant  cliquer sur les objets qu'on a demand de construire
  OutilPropSol outilPropSol;
  
  public String action; // Pour le gestionnaire d'applets pour savoir si l'applet doit
  // sauver via le gestionnaire d'applet les valeurs de res1, ..., res10 auquel cas action
  // doit contenir "Sauvegarde" ou si ces valeurs doivent ?tre restitu?es aux variables
  // res1, ..., res10 auquel cas action doit contenir "Restitution"
  public CMathGrafDoc document ;
  // Rajout pour la version 1.8 : possibilit de recharger une autre figure dans la mme applet
  // Le membre adresseFigure du  type string sera mis  null lors du dmarrage de l'applet.
  /// Lorsqu'on demandera lengthchargement d'une autre figure via ChargeFigure, ce membre
  // sera alors initialis et le thread de chargement relanc
  String adresseFigure = null;
  // statique car charg une seule fois par la premire instance
  FileInputStream file;
  public DimensionFenetre dimf;
  public boolean infRand;
  public InfoProx infoProx;
  // elementProcheCurseur appartient maintenant  l'outil de pointage actif
  // CElementGraphique ElementProcheCurseur;
  public CListeObjets listePr; // Pointeur sur document.listePrincipale
  CListeObjets listeClone; // Utilise comme sauvegarde de la figure dans le cas d'une macro par d?faut
  CListeObjets listeSauve; // Utilise comme sauvegarde de la figure en cas d'appui sur la touche chappement
  public CListeObjets listeConstruction; // Liste utilise pour stocker tous les lments construits par
                                  // l'utilisateur
  public CListeObjets listeCalculsAutorises;	// Spcial applet version 4.6. Si l'exercice est un exercice de validation interne, le commentaire
  private CMacroApparition macroPourCons;
  public boolean isExercise = false;  // Spcial Applet : true si l'applet contient via la macro ci-dessus une liste de calculs autoris
  public CMacroPause macroPourCalculs;	// Spcial applet : Pour un exercice de validation interne, ventuelle macro dont le commentaire contiendra les noms
  																				// des calculs ou variables autoriss  l'utilisateur
  																				// Pour un tel exercice, aucune cration d'objet n'est permise mais on peut y mettre des champs d'dition
  																				// Un calcul nomm reponse doit valoir 1 si l'exercice a t rsolu et 0 sinon.
  public CSousListeObjets listeAConstruire; // Pointe sur une liste forme des lments
  // que l'utilisateur doit construire avant de valider sa constriction.
  // null s'il ne s'agit pas d'un exercice de construction
  // La liste d'objets que l'utilisateur dsignera comme solutions
  public CListeObjets listeProposee;
  public CListeObjets listeDependantsDansPrincipale;
  // Nouvelle liste rajoute pour grer les macros jouant un fichier sonore, de faon
  //  lancer un thread chargeant tous les fichiers sonores
  CListeObjets listeMacrosJouantSon;
  // Idem pour la version 1.8 : liste contenant toutes les images
  CListeObjets listeImages;
  // Pour la version 2.0, liste clignotante lors des crations d'objets
  public CListeObjets listeClignotante;
  // Pour la version 2.0, liste forme des objets  ne pas dsigner  la souris
  public CListeObjets listeExclusionDesignation;
  // Pour la version 2.0, liste des objets d'aide  la cration visuelle
  public CListeObjets listeObjetsVisuels;
  public String chaineIndicationPourCons;
  boolean premierAffichage;
  // Ajout version 3.4.3 : Si aRecalculer est true, la figure sera recalcule et redessine
  // la prochaine fois que MtgPaneFigure excutera paint();
  boolean aRecalculerEtRedessiner = false;
  // Ajout version 3.4.3 : Si aRedessiner est true, la figure sera retrace dans le gbuffer
  // la prochaine fois que MtgPaneFigure excutera paint();
  boolean aRedessiner = false;
  // naturePoint appartient maintenant l'outil de dsignation actif
  // naturePointCapture contient la nature du point qui a t captur  la souris
  // long naturePointCapture;
  // deplacementPointEnCours appartient maitenant  l'outil de capture
  // deplacementPointEnCours contient true si le point captur peut tre dplac
  // boolean deplacementPointEnCours;
  // pointabs et pointres appartiennent maintenant  l'outil de pointage de capture
  // Le champ x de pointabs contient la nouvelle abscisse d'un point
  // li qui doit tre dplac
  // private static Point2D.Double pointabs = new Point2D.Double();
  // pointres continet les nouvelles coordonnes d'un point libre qui doit tre dplac
  // private static Point2D.Double pointres = new Point2D.Double();
  // L'ventuelle image de fond
  Image imageFond = null;
  // macroEnCours sera null lorsqu'aucune macro est en cours d'ex?cution et sinon pointera sur la
  // macro en cours d'ex?cution.
  public CMacro macroEnCours;
  // Si la macro comporte une macro par d?faut qui est une macro de clignotement,
  // macroAnimation contiendra son adresse et l'animation
  // sera continue via un timer appel? animationTimer
  public CMacroAnimationPointLie macroAnimation;
  public CMacro macroParDefaut;
  public java.util.Timer animationTimer;
  // Thread utilis? pour certaines macros
  public Thread runner;
  // Thread utilis pour le chargement de la figure. Pendant ce temps (jusqu'au chargement total)
  // un message de copyright est affich dans la fentre de l'applet. Pour cela une variable
  // boolenne chargementEnCours est utilise
  // Thread utilis pour le chargement des fichiers sonores des macros jouant un fichier sonore
  // La couleur de fond doit tre chang^e dans paint() donc mmorise puisque c'est celle de la figure
  // en cours qui doit  tre choisie
  public Color couleurFond;
  // Variable mise  true si la couleur de fond a t spcifie par le paramtre BackGround
  // Si c'est la cas, elle a priorit sur la couleur du document
  boolean couleurFondParParam;
	// Version 4.5.1 : Possibilit de choisir l'antialiasing active ou non via le paramtre antialiasing
	public boolean pref_antialiasing;
  
  public  boolean chargementEnCours = true;

  // MediaTracker pour le chargement d'une ventuelle image de fond
  MediaTracker track;

  public MathGraphTimer timerTask;
  public MathGraphTimer timerTaskAnimation;
  public java.util.Timer timer;
  // Nombre de boutons d'action sur la figure
  private int nombreBoutons;
  // Utilis pour ajouter les boutons aux barres
  private int nbBoutons;

  // Un commentaire destin  tre affich au voisinage du pointeur souris
  public CCommentaire commentaireDesignation; 


  /** Creates new AppletMathGraph32 */
  public AppletMathGraph32() {
		// On met le dlai des info-bulles  1/4 s
		ToolTipManager.sharedInstance().setInitialDelay(250);
    for (int i = 0; i < 4; i++) barIcones[i] = new JToolBar(JToolBar.VERTICAL);
    for (int i = 0; i < 4; i++) {
    	barIcones[i].setMargin(insetsNul);
      barIcones[i].setFloatable(false);
    }
  	
		URL adr = AppletMathGraph32.class.getResource("images/CurseurSup.gif");
		ImageIcon icon = new ImageIcon(adr);
		Image img = icon.getImage();
		supCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 0), "");  	
		adr = AppletMathGraph32.class.getResource("images/CurseurCrayon.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		crayonCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 31), "");
		adr = AppletMathGraph32.class.getResource("images/CurseurModif.gif");
		icon = new ImageIcon(adr);
		img = icon.getImage();
		modifCursor = Toolkit.getDefaultToolkit().createCustomCursor(img,
				new Point(0, 0), "");
    if (nbInstances == 0) {
      appletManager = new AppletManager();
	    // Pour viter un bug de nullPointerException rcursif : 
	    TeXFormula.registerFonts(false);
			// On cre une formule latex pour initialiser tous les fichiers associs
			// et on en profite pour crer un alias de la commande \jlmDynamic qui sert aux insertions dynamiques
	    /* Abandonn version 4.7.4
			@SuppressWarnings("unused")
			TeXFormula formula = new TeXFormula("\\newcommand{\\Val}[1]{\\jlmDynamic{\0Val\0#1}}");
			formula = new TeXFormula("\\newcommand{\\For}[1]{\\jlmDynamic{\0For\0#1}}");// Ajout version 4.5.2
			formula = new TeXFormula("\\newcommand{\\ForSimp}[1]{\\jlmDynamic{\0ForSimp\0#1}}");// Ajout version 4.6
			formula = new TeXFormula("\\newcommand{\\Calc}[1]{\\jlmDynamic{\0Calc\0#1}}");// Ajout version 4.6
			formula = new TeXFormula("\\newcommand{\\CalcSimp}[1]{\\jlmDynamic{\0CalcSimp\0#1}}");// Ajout version 4.6
			formula = new TeXFormula("\\newcommand{\\FracRed}[1]{\\jlmDynamic{\0FracRed\0#1}}");
			formula = new TeXFormula("\\newcommand{\\If}[3]{\\jlmDynamic{\0If\0#1\0#2\0#3}}");
			*/
    }
    nbInstances++;
  }
  
  public static String getString(String key) {
  	try {
  		return messages.getString(key);
  	}
  	catch (Exception e) {
			return "!" + key + "!";
		}
  }
  
  /**
   * Ajout s'il n'est pas nul du bouton  une des deux barres d'outils
   * en changeant de barres alternativement
   * @param button
   */
  private void ajouteBouton(MtgButton button) {
    if (button != null) {
    	for (int col = 0; col < nbColonnesIcones; col ++) {
    		if ((nbBoutons % (nbColonnesIcones)) == col) barIcones[col].add(button);
    	}
      nbBoutons ++;
      btngroup.add(button);
    }
  }

  public void init() {
    loadToolsIcons(); // Ajout version 4.7.4
    prepareOutils(); // Ajout version 4.7.4
		isReady = false; // Ajout version 4.6.1. Sera mis  true dans paint de MtgFigurePanel lors du premier affichage et aprs appel  setScript().
    // Construction initialise comme fausse
    constructionExacte = false;
    chargementEnCours = true;
    /* Abandonn car provoque des nullPointerException lors de Init
    try
    {
        if (UIManager.getSystemLookAndFeelClassName().indexOf("Windows")!=-1
             || UIManager.getSystemLookAndFeelClassName().indexOf("apple")!=-1)
        {
            UIManager.setLookAndFeel( UIManager.getSystemLookAndFeelClassName());
        }


    } catch (Exception e)
    {
        //Si pb, garder le look par defaut
    }
    */
    document = new CMathGrafDoc(this);
    // Pour gagner de la rapidit et de la place :
    listePr = document.listePrincipale;
    listeClone = new CListeObjets();
    listeSauve = new CListeObjets();
    // Priorit  la langue si elle est impose dans les paramtres de l'applet
    /* Modifi version 4.3
    locale = Locale.getDefault();
    String language = getParameter("Language");
  	if (language == null) {
  		// language = locale.getDisplayLanguage(); Modifi version 4.3
  		language = locale.getDisplayLanguage(new Locale("fr"));  		
	  	// Si la langue n'est pas une des langues supportes, on utilise l'anglais
	  	//if (!language.equals("fr") && !language.equals("es") && !language.equals("en")) language = "en";
	  	if (!language.startsWith("fr") && !language.startsWith("es") && !language.startsWith("en")) language = "en";  		  		
  	}
  	else {
	  	if (language.startsWith("fr") || language.startsWith("es")|| language.startsWith("en"))
	  		locale = new Locale(language);
	  		else locale = new Locale("en");
  	}
  	*/
    // Ajout version 4.5.1. Un aparmtre permet de savoir si on utilise ou non le mode antialiasing
    String antialiasing = getParameter("antialiasing");
    if (antialiasing == null) pref_antialiasing = true;
    else pref_antialiasing = !antialiasing.equalsIgnoreCase("false");
		// Nouveau versio 4.6.2 : Un paramtre qui permet au javascript dans
		// l'ventuelle fonction mtgOnInit de la page
		// de connatre le nom de l'applet qui a appel mtgOninit
		mtgOnInitParam = getParameter("onInitParam");
		if (mtgOnInitParam == null) mtgOnInitParam = "";
    
		String figureData = getParameter("figureData");    
    locale = Locale.getDefault();
    String language = getParameter("langage");
  	if (language == null) {
  		language = locale.getDisplayLanguage(new Locale("fr"));
	  	// Si la langue n'est pas une des langues supportes, on utilise l'anglais
	  	if (!language.startsWith("fr") && !language.startsWith("es") && !language.startsWith("an")) language = "ang";
  	}
  	else if (language.startsWith("en")) language = "ang";
  	if (language.startsWith("fr" )) {
  		locale = new Locale("fr");
  	}
  	else
  	if (language.startsWith("es" )) {
   		locale = new Locale("es");
  	}
  	else
  	if (language.startsWith("an" )) {
  		locale = new Locale("en");
  	}    
  	// locale = new Locale(language);
  	messages = ResourceBundle.getBundle("textes.messages", locale);
    // Pour savoir si cette applet doit sauver les valeurs de res1, res2, ..., res10
    // ou au contraire les restituer ? var1, var2, ...., var10
    action = getParameter("Action");
    if (action == null) action = "";
    // On regarde si les outils de construction sont demands.
    String tools = getParameter("Tools");
    if (tools == null) setToolsAvailable(false);
    else setToolsAvailable(tools.equalsIgnoreCase("true"));
    // Ajout version 4.7.2.6
    String challowTools = getParameter("allowTools");
    if (challowTools == null) allowTools = true;
    else allowTools = !challowTools.equalsIgnoreCase("false");

    
    String coulFond = getParameter("Background");
    couleurFondParParam = (coulFond != null);
    if (couleurFondParParam) couleurFond = Color.decode(coulFond);
    else couleurFond = Color.white;
    Couleur.setCouleurFond(couleurFond);
		if (figureData != null) loadFigureFromDataBase64(figureData);
		else chargeApplet();
    listeConstruction = new CListeObjets();
    listeCalculsAutorises = new CListeObjets();
    // On ajoute  la liste des objets construits le calcul constant pi
    listeConstruction.add(listePr.get(0));
    listeClignotante = new CListeObjets(listePr.uniteAngle, listePr.pointeurLongueurUnite);
    listeObjetsVisuels = new CListeObjets(listePr.uniteAngle, listePr.pointeurLongueurUnite);
    listeExclusionDesignation = new CListeObjets(listePr.uniteAngle, listePr.pointeurLongueurUnite);
    listeProposee = new CListeObjets(listePr.uniteAngle, listePr.pointeurLongueurUnite);
    // On regarde s'il s'agit d'un exercice de construction.
    // Si non, le pointeur suivant rendra null
    macroPourCons = macroPourConstruction();
    if (macroPourCons != null) {
    	setConstruction(true);
      listeAConstruire = macroPourCons.listeAssociee;
      chaineIndicationPourCons = macroPourCons.commentaireMacro;
    }
    else setConstruction(false);
		macroPourCalculs = macroCalculsAutorises();
		isExercise = (macroPourCalculs != null);
		// if (isConstruction || isExercise) etablitListeObjetsConstruits(); // Pour permettre d'utiliser certains calculs
		if (isConstruction()) etablitListeObjetsConstruits(); // Appelle etablitListeCalculsAutorises
		else {
			if (isExercise) etablitListeCalculsAutorises(); // Modifi version 4.7		
		}
    // Si on est dans le cas d'un exercice de construction
    // if (! isExercise && (isConstruction() || isToolsAvailable())) {  // Modifi version 4.7
		if (isConstruction() || isToolsAvailable()) {
      // Prparation de la zone de texte d'indications
      //indicationArea.setEnabled(false);
      indicationArea.setEditable(false);
      indicationArea.setForeground(Color.BLUE);
      indicationArea.setSelectionColor(Color.WHITE);
      indicationArea.setSelectedTextColor(Color.BLUE);
      // Cration du groupe de boutons
      btngroup = new ButtonGroup();
      // On compte le nombre rel de boutons
      nombreBoutons = 0;
      buttonAnnuler = outilAnnuler.boutonAssocie();
      nombreBoutons++;
      buttonRefaire = outilRefaire.boutonAssocie();
      nombreBoutons++;
      buttonSup = outilSup.boutonAssocie();
      nombreBoutons++;     
      buttonHistorique = outilHistorique.boutonAssocie();
      nombreBoutons++;           
      buttonNommer = outilEntreeNom.boutonAssocie();
      nombreBoutons++;
      buttonCapt = outilCapture.boutonAssocie();
      nombreBoutons++;           
      buttonPtLib = outilPtLib.boutonAssocie();
      if (buttonPtLib != null) nombreBoutons++;
      buttonPtLie = outilPtLie.boutonAssocie();
      if (buttonPtLie != null) nombreBoutons++;
      buttonMil = outilMilieu.boutonAssocie();
      if (buttonMil!= null) nombreBoutons++;
      buttonPtParAbs = outilPtParAbs.boutonAssocie();
      if (buttonPtParAbs != null) nombreBoutons++;
      buttonPointParCoordonnees = outilPointParCoordonnees.boutonAssocie();
      if (buttonPointParCoordonnees != null) nombreBoutons++;
      buttonModifierObjetGraphique = outilModifierObjetGraphique.boutonAssocie();
      if (buttonModifierObjetGraphique != null) nombreBoutons++;
      buttonModifierFormule = outilModifierFormule.boutonAssocie();
      if (buttonModifierFormule != null) nombreBoutons++;
      
      buttonSeg = outilSeg.boutonAssocie();
      if (buttonSeg != null) nombreBoutons++;
      buttonDteAB = outilDtAB.boutonAssocie();
      if (buttonDteAB != null) nombreBoutons++;
      buttonDtePar = outilDtPar.boutonAssocie();
      if (buttonDtePar != null) nombreBoutons++;
      buttonDtePer = outilDtPer.boutonAssocie();
      if (buttonDtePer != null) nombreBoutons++;
      buttonDteMed = outilMed.boutonAssocie();
      if (buttonDteMed != null) nombreBoutons++;
      buttonDteBis = outilBis.boutonAssocie();
      if (buttonDteBis != null) nombreBoutons++;
      buttonDemiDt = outilDemiDt.boutonAssocie();
      if (buttonDemiDt != null) nombreBoutons++;
      buttonCerOA = outilCerOA.boutonAssocie();
      if (buttonCerOA != null) nombreBoutons++;
      buttonCerOR = outilCerOR.boutonAssocie();
      if (buttonCerOR != null) nombreBoutons++;
      buttonPolygone = outilPolygone.boutonAssocie();
      if (buttonPolygone != null) nombreBoutons++;
      buttonInt = outilInt.boutonAssocie();
      if (buttonInt != null) nombreBoutons++;
      buttonTrans = outilTrans.boutonAssocie();
      if (buttonTrans != null) nombreBoutons++;
      buttonSymAxiale = outilSymAxiale.boutonAssocie();
      if (buttonSymAxiale != null) nombreBoutons++;
      buttonSymCentrale = outilSymCentrale.boutonAssocie();
      if (buttonSymCentrale != null) nombreBoutons++;
      buttonRot = outilRot.boutonAssocie();
      if (buttonRot != null) nombreBoutons++;
      buttonRapp = outilRapporteur.boutonAssocie();
      if (buttonRapp != null) nombreBoutons++;
      buttonCreerCalcul = outilCalcul.boutonAssocie();
      if (buttonCreerCalcul != null) nombreBoutons++;
      buttonHom = outilHom.boutonAssocie();
      if (buttonHom != null) nombreBoutons++;
      buttonMesLong = outilMesLong.boutonAssocie();
      if (buttonMesLong != null) nombreBoutons++;
      buttonMesAngNor = outilMesAngnor.boutonAssocie();
      if (buttonMesAngNor != null) nombreBoutons++;
      buttonMesAngOr = outilMesAngOr.boutonAssocie();
      if (buttonMesAngOr != null) nombreBoutons++;
      buttonMesAbs = outilMesAbs.boutonAssocie();
      if (buttonMesAbs != null) nombreBoutons++;
      buttonPropSol = outilPropSol.boutonAssocie();

      // Un JPane pour contenir le bouton de proposition de solution et le bouton
      // de choix de couleur
      paneLow = new JPanel();
      paneLow.setLayout(new GridBagLayout());
      if (nombreBoutons < nbBoutonsMiniPour3Col) {
      	nbColonnesIcones = 2;
      	paneLeftHigh.setLayout(new GridLayout(1,2));
      }
      else {
      	if (nombreBoutons < nbBoutonsMiniPour4Col) {
      		nbColonnesIcones = 3;
      		paneLeftHigh.setLayout(new GridLayout(1,3));
      	}
      	else {
      		nbColonnesIcones = 4;
      		paneLeftHigh.setLayout(new GridLayout(1,4));
      	}
      }
      for (int i = 0; i < nbColonnesIcones; i++) paneLeftHigh.add(barIcones[i]);
       
      nbBoutons = 0;
      ajouteBouton(buttonAnnuler);
      ajouteBouton(buttonRefaire);
      ajouteBouton(buttonHistorique);
      ajouteBouton(buttonCapt);
      ajouteBouton(buttonModifierObjetGraphique);
      ajouteBouton(buttonModifierFormule);    
      ajouteBouton(buttonSup);
      ajouteBouton(buttonNommer);
      ajouteBouton(buttonPtLib);
      ajouteBouton(buttonMil);
      ajouteBouton(buttonPtLie);
      ajouteBouton(buttonPtParAbs);
      ajouteBouton(buttonPointParCoordonnees);
      ajouteBouton(buttonDteAB);
      ajouteBouton(buttonSeg);
      ajouteBouton(buttonDemiDt);
      ajouteBouton(buttonDtePar);
      ajouteBouton(buttonDtePer);
      ajouteBouton(buttonDteMed);
      ajouteBouton(buttonDteBis);
      ajouteBouton(buttonCerOA);
      if (listePr.pointeurLongueurUnite != null) ajouteBouton(buttonCerOR);
      ajouteBouton(buttonPolygone);
      ajouteBouton(buttonInt);
      if (listePr.uniteAngle == UniteAngle.Degre) ajouteBouton(buttonRapp);
      ajouteBouton(buttonTrans);
      ajouteBouton(buttonSymAxiale);
      ajouteBouton(buttonSymCentrale);
      ajouteBouton(buttonRot);
      ajouteBouton(buttonHom);
      ajouteBouton(buttonCreerCalcul);
      if (listePr.pointeurLongueurUnite != null) ajouteBouton(buttonMesLong);
      ajouteBouton(buttonMesAngNor);
      ajouteBouton(buttonMesAngOr);
      ajouteBouton(buttonMesAbs);
      btngroup.add(buttonPropSol);
      
      GridBagConstraints constraints = new GridBagConstraints();
      constraints.insets.bottom = 0;
      constraints.insets.top = 0;
      constraints.insets.left = 0;
      constraints.insets.right = 0;
      colorButton = new MtgColorButton(this);
  		listeStyleTrait = new MtgListeStyleTrait(this);
  		JScrollPane scrollPaneStyleTrait = new JScrollPane(listeStyleTrait,
  				JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
  				JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
  		scrollPaneStyleTrait.setBorder(BorderFactory.createEtchedBorder());		
      MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 100, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
      paneLow.add(buttonPropSol, constraints);
  		
      
      MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
      paneLow.add(colorButton, constraints);
      
      MtgDlg.setConstraints(constraints, 0, 2, 1, 1, 0, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
      paneLow.add(scrollPaneStyleTrait, constraints);
      paneLeftLow.add(paneLow);
      // On cre le dialogue de choix de couleurs
      colorChooser = new JColorChooser();
      colorChooser.setPreviewPanel(new JPanel()); // Pour qu'il n'y ait pas de preview
      colorChooser.setColor(colorButton.couleur);

      paneLeft.setLayout(new BorderLayout());
      paneLeft.add(paneLeftHigh, BorderLayout.NORTH);
      paneLeft.add(paneLeftLow, BorderLayout.CENTER);
      paneSouth.setLayout(new GridLayout(1,1));
      paneSouth.add(indicationArea);
            
      panePrincipal.setLayout(new BorderLayout());
      panePrincipal.add(paneLeft, BorderLayout.WEST);
      panePrincipal.add(paneCenter, BorderLayout.CENTER);
      panePrincipal.add(paneSouth, BorderLayout.SOUTH);
      // panePrincipal.add(("South"), informationArea);
      setContentPane(panePrincipal);
      Rectangle rect1 = paneLeft.getBounds();
      decalageBordGauche = rect1.width;
      rect1 = paneSouth.getBounds();
      decalageBordBas = rect1.height;
    }
    else { // Cas d'un exercice qui n'est pas un exercice de construction
      setContentPane(paneCenter);
      outilActif = outilCapture;
    }
    creePaneAffichageVariables();
    
    springl = new SpringLayout();
    paneCenter.setLayout(springl);
    paneCenter.add(paneFigure, JLayeredPane.DEFAULT_LAYER);
    if (paneAffichageVariables != null) {
    	paneCenter.add(paneAffichageVariables, JLayeredPane.DRAG_LAYER);
	    springl.putConstraint(SpringLayout.SOUTH, paneAffichageVariables, 0, SpringLayout.SOUTH, paneCenter);
	    springl.putConstraint(SpringLayout.EAST, paneAffichageVariables, 0, SpringLayout.EAST, paneCenter);
    }
    springl.putConstraint(SpringLayout.NORTH, paneFigure, 0, SpringLayout.NORTH, paneCenter);
    springl.putConstraint(SpringLayout.WEST, paneFigure, 0, SpringLayout.WEST, paneCenter);
    
    setCursor(crossHairCursor);

    // A revoir

    infRand = false;

    //dimf = new DimensionFenetre(getWidth(), getHeight());
    // Les dimensions horizontales de la figure ne sont pas celles de l'applet
    // pour cette version avec barre d'outils de construction
    dimf = new DimensionFenetre(getWidth()-decalageBordGauche, getHeight()-decalageBordBas);
    infoProx = new InfoProx();
    // Le point qui sert ? suivre les mouvements de la souris pour la cr?ation visuelle
    pointSouris = new CPointBase(listeExclusionDesignation, null, false);
    // Commentaire li au point prcdent servant  l'affichage des dsignations au
    // voisinage du pointeur souris
		// Ligne suivante modifie version 4.5.2
		commentaireDesignation = new CCommentaire(listePr, null, false,
				Couleur.TraduitCouleur(4), 0d, 0d, 10d, 12d, false, pointSouris, 2,
				StyleEncadrement.Sans, true, couleurFond, "",
				CAffichageLieAPoint.alignementHorizontalCentre, CAffichageLieAPoint.alignementVerticalHaut);
    track = null;
    premierAffichage = true;
    addKeyListener(this);
    addMouseListener(this);
    setFocusable(true);
    requestFocus(); 
    chargementEnCours = false;
    historiqueDlg = new HistoriqueDlg(this);
    // activeOutilCapture();
    start();
		// On regarde si la page web qui contient l'applet contient une fonction nomme mtgOnInit()
		// Si elle est prsente on l'exccute.
		// En gnral cette fonction devra appeler setXML(chaine) o chaine est une chaine de caractres contenant un document
		// Nouveau version 4.6 : un paramtre est pass qui est en fait le nom de
		// l'applet choisi par le paramtre onInitParam
		try {
			JSObject jso = JSObject.getWindow(this);
			Object[] arg = new Object[1];
			arg[0] = mtgOnInitParam;
			jso.call("mtgOnInit", arg);
		} catch (Exception e) {
			// TODO: handle exception
		}
    // repaint(); // Supprim version 4.6.2
  }
 
/**
 * @return Fonction renvoyant null si la figure ne contient pas un exercice de construction.
 * Dans le cas contraire, la figure doit contenir une macro d'apparition d'objet
 * nomme SolutionConstruction dont la liste d'objets  faire apparatre
 * sera en fait la liste des objets que l'utilisateur de l'applet doit construire.
 * le commentaire de cette macro contiendra des informations que l'on affichera
 * dans la ligne d'indication pour dire quel est le type d'objet que l'utilisateur
 * doit cliquer. Chacun commencera par un caractre du type #1: suivi de l'indication
 *  afficher
 */
  private CMacroApparition macroPourConstruction() {
    ListIterator<CElementBase> li = listePr.listIterator();
    while (li.hasNext()) {
      CElementBase elb = (CElementBase) li.next();
      if (elb.getClass() == CMacroApparition.class) {
        CMacroApparition mac = (CMacroApparition) elb;
        if ((mac.intitule.compareToIgnoreCase(getString("solutionConstruction")) == 0)
        		|| (mac.intitule.compareToIgnoreCase("SolutionConstruction") == 0)) return mac;
      }
    }
    return null;
  }
  // Il faut redfinir la mthode update car par dfaut elle efface le contenu de 
  // la fentre avec la couleur de fond avant d'appeler paint(), ce qui provoque un effet de flash
  public void update(Graphics graphics) {
    paint(graphics);
  }
  
  
  /**  Par dfaut, lors d'un repaint, la figure n'est pas recalcule
   *   Appeler cette fonction pour recalculer la figure puis la
   *   rafficher en particulier lors de termineAction() des macros
  **/
  public void repaintFigureAvecCalcul() {
		paneFigure.effacegBuffer();
    positionne();
    paneFigure.afficheToutDansgbuffer();
    paneFigure.repaint();
  }

  /**  Par d?faut, lors d'un repaint, la figure n'est pas recalcul?e
   *   Appeler cette fonction pour recalculer la figure puis la
   *   r?afficher en particulier lors de termineAction() des macros
  **/
  public void repaintFigureSansCalcul() {
		paneFigure.effacegBuffer();
    paneFigure.afficheToutDansgbuffer();
    paneFigure.repaint();
  }
  
  
public void repaintFigure() {
  paneFigure.repaint();
}

public void dessinegbuffer() {
  paneFigure.dessinegbuffer((Graphics2D) paneFigure.getGraphics());
}
  /** La mthode paint() a t profondment remanie dans la version 2.0
   *  L'ide principale est toujours la mme : dessiner dans la fentre
   *  principale exclusivement avec paint()
   *  La figure n'est dessine dans le gbuffer dans paint() que lors du premier
   *  affichage, except lors de la capture d'objets.
   *  Il faut donc dessiner ce qu'il faut dans le gbuffer avant d'appeler repaintFigure()
   *  Par exemple, pour le clignotement des objets dsigns lors de la cration
   *  dans la mthode run() de MathGraphTimerClig, on dessine ou efface la liste
   *  clignotante dans le gbuffer avant appel  repaintFigure()
   */
  
  
  public void start() {
    timerTaskAnimation = new MathGraphTimer(this);
    if (macroEnCours != null) {
      if (macroEnCours.executionEnCours) {
        macroEnCours.macroEnCours().initialiseInterruptions(this);
      }
    }
    // Avec le chargement de la figure par un thread, start() est appel avant le chargement
    // et ainsi on ne peut pas initialiser le timer  cet endroit au premier chargement.
    // Cela va tre fait dans paint()
    if (macroAnimation != null) {
      macroAnimation.actionAvantExecution(this);
      if (animationTimer == null) {
        animationTimer = new java.util.Timer();
        animationTimer.schedule(timerTaskAnimation, 0l, macroAnimation.frequenceAnimation());
      }
    }
  }

  // Si la page internet qui contient l'applet n'est plua active, il faut arr^ter tout timer oui thread
  // dclench par l'utilisation d'une macro et lengthrecharger ensuite dans start quand la page
  // est ractive.
  public void stop() {
    if (timer != null) {
      timer.cancel();
      timer = null;
    }
    if (animationTimer != null) {
      animationTimer.cancel();
      animationTimer = null;
    }
    timerTask = null;
    timerTaskAnimation = null;
    runner = null;
    //if (runner != null) runner = null;(
    //if (timer != null) timer = null;
    //if (animationTimer != null) {
    //  timerTaskAnimation = null;
    //  animationTimer.cancel();
    //  animationTimer = null;
    //}

  }

  public void destroy() {
  }
  
  // Activation de l'outil de capture
  public final void activeOutilCapture() {
    if (outilActif != null) outilActif.deselect();
    outilActif = outilCapture;
    if (buttonCapt != null) buttonCapt.setSelected(true); // Pour une applet de visualisation sans construction
    outilActif.select();
  }

  // Dans la version 20, il faut ragir aux clics sur les boutons
  public void actionPerformed(ActionEvent e) {
    if (chargementEnCours) return;
    Object ob = e.getSource();
    if (ob.getClass() == MtgButton.class) {
      MtgButton jb = (MtgButton) ob;
      Outil outilChoisi = ((MtgButton) ob).outil;
      // On regarde si l'outil choisi peut tre activ
      if (outilChoisi.activationValide()) {
        outilActif.deselect();
        outilActif = outilChoisi;
        outilChoisi.select();
        jb.requestFocus();
        
      }
      else {
        dialogueAvertissement(getString("ch15"));
        outilChoisi.deselect();
         // On ractive l'outil de capture si l'outil ne peut pas tre activ
        activeOutilCapture();
      }   
    }
  }
  
  public void mousePressed(java.awt.event.MouseEvent mouseEvent) {
  }

  public void mouseReleased(java.awt.event.MouseEvent mouseEvent) {
  }
  
  public void mouseClicked(java.awt.event.MouseEvent mouseEvent) {
  }
  
  public void mouseExited(java.awt.event.MouseEvent mouseEvent) {
    Object ob = mouseEvent.getSource();
    showStatus("");
    if (ob == paneFigure) setCursor(defaultCursor);
  }
  
  public void mouseEntered(java.awt.event.MouseEvent mouseEvent) {
    showStatus(messageCopyright);
  }
  
  public void run() {
    Thread thisThread = Thread.currentThread();
    while( runner == thisThread) {
      if (macroEnCours != null) {
        macroEnCours.macroEnCours().executePourThread(this);
      }
    }
  }
  
  public void keyTyped(KeyEvent evt) {
    if ((evt.getKeyChar() == KeyEvent.VK_ESCAPE)) {
      if (macroEnCours != null) {
        macroEnCours.termineAction(this);
        /*
        timerTask = null;
        if (timer != null){
          timer.cancel();
          timer = null;
        }
        if (runner != null) runner = null;
        */
				try {
					Thread.sleep(100); // Pour laisser le temps aux boucles en cours
															// d'excution de se terminer
				} catch (InterruptedException e) {
				}        
        macroEnCours = null;
      }
      //premierAffichage = true;
      listePr.positionne(infRand, dimf);
      listeSauve.positionne(infRand, dimf);
      listePr.setClone(listeSauve); // Doit tre appel aprs positionne()
      paneFigure.effacegBuffer();
      paneFigure.afficheToutDansgbuffer();
      paneFigure.repaint();
    }     
  }

  public void keyReleased(KeyEvent evt) {
  }
  
  public void keyPressed(KeyEvent evt) {
  }
  
    

  /** Fonction renvoyant le chemin d'accs  la figure sans le nom */
  String cheminAcces() {
    String ch = getParameter("Path");
    if (ch == null) return "";
    int i = ch.lastIndexOf('/');
    if (i == -1) return "";
    else return ch.substring(0,i+1);
  }
  
  /** Mthode chargeant l'applet et appele depuis un thread */
  void chargeApplet() {
    try
    {
      String chAcces = cheminAcces();
      // On regarde si un chargement d'une nouvelle figure a t demand via
      // chargeFigure
      String adFig;
      if (adresseFigure == null) adFig = getParameter("Path");
      else {
        adFig = chAcces + adresseFigure;
        paneFigure.effacegBuffer();
      }
      URL adresse = new URL(getDocumentBase(),adFig);
      BufferedInputStream buff = new BufferedInputStream(adresse.openStream());
      MathGraphDataInputStream dataStream = new MathGraphDataInputStream(buff);
      document.chargeDepuisFlux(dataStream);
      modeTrace = document.modeTraceActive;
      // On regarde si une couleur de fond est spcifie dans la figure.
      if (!couleurFondParParam) {
        couleurFond = document.couleurFond;
      }
      listePr.setCopy(listeClone);
      listePr.setCopy(listeSauve);
      listeDependantsDansPrincipale = new CListeObjets(listePr);
      listeMacrosJouantSon = new CListeObjets(listePr);
      listeMacrosJouantSon.ajouteMacrosJouantSon(listePr);
      listeImages = new CListeObjets(listePr);
      listeImages.ajouteImagesOldVersion(listePr);
      String imfond = new String();
      if (document.pathImageFond.length() == 0) {
        String s = getParameter("Image");
        if (s != null) imfond = s;
      }
      else {
        imfond = document.pathImageFond;
      }
      // Modification version 4.7.7 : Si l'image de fond est incorpore dans la figure, ne fonctionnait plus
      if (document.imageFond == null) {
	      boolean imageFondPresente = imfond.length() != 0;
	      imfond = chAcces + imfond;
	      // On attend le chargement de toutes les images 
	      if ((listeImages.size() != 0) || (imageFondPresente)) {
	        Toolkit kit = Toolkit.getDefaultToolkit();
	        track = new MediaTracker(this);
	        int indiceIm = 0;
	        if (imageFondPresente) {          
	          URL adr = new URL(getDocumentBase(),imfond);
	          imageFond = kit.getImage(adr);
	          track.addImage(imageFond, indiceIm);
	          indiceIm++;
	        }
	        CListeObjets list = listeImages;
	        ListIterator<CElementBase> li = list.listIterator();
	        while(li.hasNext()) {
	          CImage im = (CImage) (li.next());
	          URL adr = new URL(getDocumentBase(),chAcces + im.imagePath);
	          im.image = ImageIO.read(adr);
	          track.addImage(im.image, indiceIm);
	          indiceIm++;
	        }
	      }
      }
      else imageFond = document.imageFond; // Rajout version 4.7.7
      // On attend que l'ventuelle image de fond soit charge
      if (track != null) {
        try {
          track.waitForAll();
        }
        catch (InterruptedException e) { }
      }
      // Si ncessaire on charge les fichiers sonores des macros jouant des sons
      if (listeMacrosJouantSon.size() != 0) {
        CListeObjets list = listeMacrosJouantSon;
        ListIterator<CElementBase> li = list.listIterator();
        while(li.hasNext()) {
          CMacroJouantSon mac = (CMacroJouantSon) (li.next());
          mac.clip = getAudioClip(getDocumentBase(), chAcces + mac.wavePath);
        } 
      }
      macroParDefaut = listePr.macroParDefaut;
      if (macroParDefaut != null) {
        if (macroParDefaut instanceof CMacroAnimationPointLie)  {
          macroAnimation = (CMacroAnimationPointLie) macroParDefaut;
          macroParDefaut = null;
          long freq = macroAnimation.frequenceAnimationMacro;
          if (freq < 10) macroAnimation.frequenceAnimationMacro = 10;
          //macroAnimation.actionAvantExecution(this);
        }
      }
      // Une fois l'applet entirement charge, et si son paramtre Action
      // contenait "Restitution", on affecte le contenu des calculs res1, res2, ..., res10
      // de la figure prcdemment sauvegardes via l'AppletManger
      // aux variables var1, var2, ..., var10 de faon  pouvoir "reconstituer"
      // la figure de l'applet prcdente
      if (action.equals("Restitution")) {
        appletManager.restitueValeurs(this);
      }
    }
    catch (Exception e) {
      System.out.println("Erreur pendant le chargement");
      System.out.println(e.getMessage() + e.toString());
      System.exit(-1);
    }
  }
  
	/**
	 * Pour les macros devant laisser une trace, initialise le device context de
	 * traces en l'effaant et en affichant l'image de fond s'il y en a une
	 * Modifi version 4.7.7. Mme si le mode trace est actif on l'excute.
	 */
  public void initialisetBuffer()
  {
  	// if (!modeTrace) {
	  	paneFigure.effacetBuffer();
			if ((imageFond != null) && document.imageFondVisible)
				paneFigure.tbuffer.drawImage(imageFond, 0, 0, this);
  	//}
    
  }
  
	/**
	 * Fonction appele pour initialiser le buffer de traces lors de sa cration
	 * Ajout version 4.4.4 pour corriger un bug qui faisait que, quand on ouvrait une image
	 * avec le mode trace activ, la couleur de fond n'tait pas la bonne et l'image de fond pas affiche s'il y en avait une
	 */
	public void effacetBuffer() {
		paneFigure.effacetBuffer();
		Image imageFond = document.imageFond;
		if ((imageFond != null) && document.imageFondVisible)
			paneFigure.tbuffer.drawImage(imageFond, 0, 0, this);
	}  
   
  

  
  /** Version 1.8 : Mthode publique destine  recharger une autre figure via JavaScript */
  public final void chargeFigure(String nomFigure) {
  	// Impossible de charger une autre figure si la figure en cours est un exercice de construction
    if (isConstruction()) return;  	
    stop();
    adresseFigure = nomFigure;
    chargementEnCours = true;
    timerTaskAnimation = null;
    macroEnCours = null;
    macroAnimation = null;
    animationTimer = null;
    imageFond = null;
    // On d?truit le document actuel
    document = new CMathGrafDoc(this);
    listePr.clear();
    listePr = document.listePrincipale;
    listeClone.clear();
    listeSauve.clear();
    listeConstruction.clear();
    listeDependantsDansPrincipale.clear();
    listeMacrosJouantSon.clear();
    System.gc();
    premierAffichage = true;
    chargeApplet();
    repaint();
    chargementEnCours = false;
    start();
  }
  
  /** Version 1.8 : M?
   * thode publique destine  connaitre la valeur d'un calcul
   * Renvoie -1 si le calcul n'est pas d?fini ou n'existe pas. Sinon renvoie
   * sa valeur
   * @param nomCalcul
   * @return
   */
  public final double valeurDe(String nomCalcul) {
    // Si l'exercice est une exercice de construction et si on demande
    // la valeur d'un calcul nomm reponse qui en fait n'existe pas on revoie true
    // si les objets dsigns avec l'outil outilPropSol sont les bons
    if (isConstruction() && nomCalcul.equalsIgnoreCase("reponse")) {
      if (constructionExacte) return 1; else return 0;
    }
    CCalculAncetre calcul = listePr.pointeurCalculReelNomme(nomCalcul, -1);
    if (calcul == null) return -1;
    // Pas de retirage alatoire
    else {
    	// Modifi version 4.6
      // if ((!calcul.existe) || calcul.getClass() != CCalcul.class) return -1;
      if (!calcul.existe) return -1;
      else return calcul.rendValeur(false);
   }
  }
  
  public final double valueOf(String nameCalculus) {
  	return valeurDe(nameCalculus);
  }
  
  /** Version rectifie le 18/3/09
   * Le troisime paramtre n'est plus utilis et n'est gard
   * que pour compatibilit avec les versions antrieures
   * Version 1.8 : Mthode publique donnant la valeur correspond au contenu de
   * la chane chCalcul au calcul ou  la fonction de nom nomCalcul.
   * Renvoie 1 si tout s'est bien pass.
   * Renvoie 0 si mne  un calcul n'existant pas
   * Renvoie -1 s'il y a une faute de syntaxe
   * Renvoie -2 si le calcul ou la fonction n'existe pas.
   * La figure n'est pas repositionnes et raffiche aprs l'affectation
   * de la valeur  la variable.
   * Il faut pour cela appeler redessine()
   * @param nomCalcul
   * @param chCalcul
   * @param variableFormelle
   * @return
   */
  
  public final int donneValeurA(String nomCalcul, String chCalcul, String variableFormelle) {
    @SuppressWarnings("unused")
		double v;
    CCalculBase calc;
    // On remplace les virgules par des points dcimaux
    chCalcul = chCalcul.replace(',', '.');
    CCalcul valeur = listePr.pointeurCalculOuFonction(nomCalcul, -1);
    if (valeur == null) return -2;
    else {
      String[] tabvarf = null;
      boolean bFonction = false;
      if (valeur.getNatureCalcul() == NatureCalcul.NFonction) {
      	bFonction = true;
      	tabvarf = new String[1];
	      tabvarf[0] = ((CFonc) valeur).chaineVariable;
      }
      else {
        if (valeur.estDeNatureCalcul(NatureCalcul.NFonction2Var | NatureCalcul.NFonction3Var)) {
        	bFonction = true;
        	CFoncNVar foncnvar = (CFoncNVar) valeur;
        	int nbvar = foncnvar.nombreVariables();
        	tabvarf = new String[nbvar];
        	for (int k = 0; k < nbvar; k++) tabvarf[k] = foncnvar.chaineVariable[k];
        }
      }
      Int indErr = new Int();
      if (!CCalculBase.verifieSyntaxe(listePr, chCalcul, indErr, listePr.indexOf(valeur)-1, tabvarf))
        return -1;
      if (bFonction) {
        calc = CCalculBase.creeCalculBase
          (chCalcul ,listePr, 0, chCalcul.length()-1, tabvarf);        
        valeur.chaineCalcul = chCalcul;
        valeur.calcul = calc;
      	listePr.recalculeDeriveesDependantDe(valeur);
        return 1;
      }
      else {
        calc = CCalculBase.creeCalculBase
          (chCalcul ,listePr, 0, chCalcul.length()-1, tabvarf);
        try {
          v = calc.resultat(false);
        }
        catch (ErreurCalculException e) {
          return 0;
        }
        valeur.chaineCalcul = chCalcul;
        valeur.calcul = calc;
        return 1;        
      }
    }
  }
  
  public final int donneValeurA(String nomCalcul, String chCalcul) {
  	return donneValeurA(nomCalcul, chCalcul, "");
  }
  
  //Ajout version 4.6
  /**
   * Fonction publique renvoyant la chaine du calcul ou de la fonction de nom nomCalcul
   * @param nomCalcul
   * @return
   */
  public final String formulaOf(String nomCalcul) {
    CCalcul valeur = listePr.pointeurCalculOuFonction(nomCalcul, -1);
    if (valeur == null) return "";
    else return valeur.chaineCalcul;	
  }
  
  /**
   * public method giving formula in chCalcul to object named chCalcul
   * This object may be a real calculus or a real function of one too three variables
   * @param nomCalcul
   * @param chCalcul
   * @return
   * 1 if no error
   * Renvoie 0 if leads to a calculus non existing
   * Renvoie -1 if there is a syntax error
   * Renvoie -2 if there is non object named nomCalcul
   */
  public final int giveFormulaTo(String nomCalcul, String chCalcul) {
  	return donneValeurA(nomCalcul, chCalcul);
  }
  
  /**
   * public method giving formula in chCalcul to object named chCalcul
   * This object may be a complex calculus or a complex function of one too three variables
   * @param nomCalcul
   * @param chCalcul
   * @return
   * 1 if no error
   * Renvoie 0 if leads to a calculus non existing
   * Renvoie -1 if there is a syntax error
   * Renvoie -2 if there is non object named nomCalcul
   */
  public final int giveComplexFormulaTo(String nomCalcul, String chCalcul) {
    @SuppressWarnings("unused")
		double v;
    CCalculBase calc;
    // On remplace les virgules par des points dcimaux
    chCalcul = chCalcul.replace(',', '.');
    CCalcul valeur = listePr.pointeurCalculOuFonctionComplexe(nomCalcul, -1);
    if (valeur == null) return -2;
    else {
      String[] tabvarf = null;
      boolean bFonction = false;
      if (valeur.getNatureCalcul() == NatureCalcul.NFonctionComplexe) {
      	bFonction = true;
      	tabvarf = new String[1];
	      tabvarf[0] = ((CFoncComplexe) valeur).chaineVariable;
      }
      else {
        if (valeur.estDeNatureCalcul(NatureCalcul.NFonctionComplexe2Var | NatureCalcul.NFonctionComplexe3Var)) {
        	bFonction = true;
        	CFoncComplexeNVar foncnvar = (CFoncComplexeNVar) valeur;
        	int nbvar = foncnvar.nombreVariables();
        	tabvarf = new String[nbvar];
        	for (int k = 0; k < nbvar; k++) tabvarf[k] = foncnvar.chaineVariable[k];
        }
      }
      Int indErr = new Int();
      if (!CCalculBase.verifieSyntaxeComplexe(listePr, chCalcul, indErr, listePr.indexOf(valeur)-1, tabvarf))
        return -1;
      if (bFonction) {
        calc = CCalculBase.creeCalculBaseComplexe
          (chCalcul ,listePr, 0, chCalcul.length()-1, tabvarf);        
        valeur.chaineCalcul = chCalcul;
        valeur.calcul = calc;
      	listePr.recalculeDeriveesDependantDe(valeur);        
        return 1;
      }
      else {
        calc = CCalculBase.creeCalculBaseComplexe
          (chCalcul ,listePr, 0, chCalcul.length()-1, tabvarf);
        try {
          v = calc.resultat(false);
        }
        catch (ErreurCalculException e) {
          return 0;
        }
        valeur.chaineCalcul = chCalcul;
        valeur.calcul = calc;
        return 1;        
      }
    }
  }
  
  /** Mthode publique destine  excuter une macro via JavaScript.
   * Prend comme paramtre l'initul de la mthode (doit respecter la casse et les espaces)
   * Renvoie true si l'excution a public avoir lieu et false sinon
   * @param nameMacro
   * @return
   */
  public final boolean executeMacro(String nameMacro) {
    // Refus si la macro possde une macro par dfaut qui est une macro d'animation
  	// Modifi version 4.6 : Refus aussi si une autre marco est en cours d'excution.
    // if (macroAnimation != null) return false;
  	if ((macroAnimation != null) || (macroEnCours != null)) return false;
    // On cherche si NomMacro contient bien un intitul de macro valide
    CMacro macroAExecuter = listePr.pointeurMacro(nameMacro);
    if (macroAExecuter == null) return false;
    /* Supprim version 4.6
    if (macroEnCours != null) { // Si aucune macro n'est d?j? en cours d'ex?cution
                              // on lance l'ex?cution de la macro cliqu?e si possible
      CMacro mac = macroEnCours.macroEnCours();
      mac.termineAction(this);
      macroEnCours.termineAction(this);
    }
    */
    if (macroAExecuter.executionPossible()) {
      macroEnCours = macroAExecuter;
      macroEnCours.initialise(this);
      macroEnCours.macroEnCours().actionAvantExecution(this);
      // Si la macro utilise un thread, il faut le cr?er.
      // ? revoir
      macroEnCours.macroEnCours().initialiseInterruptions(this);

      if (macroEnCours.macroEnCours().typeAnimation() == CMacro.sansAnimation) repaint();
      return true;
    }
    else return false;
  }   
  
  /** Version 1.8 : Mthode publique destine  savoir si une macro est en cours d'excution */
  public final boolean macroEnCours() {
    return (macroEnCours != null);
  }
  
  public final boolean executingMacro() {
  	return macroEnCours();
  }
  
  // Version 1.8 : Mthode publique destine  savoir si la figure est bien charg?e
  public final boolean figureChargee() {
    return !chargementEnCours;
  }
  
  public final boolean figureLoaded() {
  	return figureChargee();
  }
  
  /**
   * Version 1.8 : mthode publique servant  recalculer et rafficher compltement la figure
   * en effaant tout, recalculant et affichant tout.
   * Sauvegarde les donnes pour rtroaction
   */
  public final void redessine() {
    positionne();
    if (action.equals("Sauvegarde")) {
      appletManager.sauveValeurs(this);
    }
    /* Modifi version 3.4.3 Le dessin de la figure doit se faire dans paint();
    paneFigure.effacegBuffer();
    paneFigure.afficheToutDansgbuffer();
    paneFigure.repaint();
    */
    aRedessiner = true;
    paneFigure.repaint();
  }
  
  /**
   * Public method recalculating and updating the figure without updating calls to rand
   */
  /* Abandonn
  public final void update() {
  	redessine();
  }
  */
  
  /**
   * Deprecated. Kept for backward compatibility
   */
  public final void updateWithRandom() {
  	infRand = true;
  	positionne();
  	infRand = false;
    paneFigure.effacegBuffer();
    paneFigure.afficheToutDansgbuffer();
    paneFigure.repaint();
  }
  
  /**
   * Ajout pour la version 2.5.4 pour sauvegarder les valeurs
   * dans une aplet de validation interne.
   */
  public final void sauvegarde() {
    if (action.equals("Sauvegarde")) {
      appletManager.sauveValeurs(this);
    }    
  }
  
  /**
   * public method saving values or formulas of calculuses or functions named z1 to z20
   * and calculuses named res1 to res2
   */
  public final void saveData() {
  	appletManager.sauveValeurs(this);
  }
  
  /**
   * public method retrieving values or formulas of calculuses or functions named z1 to z20
   * and calculuses named res1 to res2
   * The values or formula retrieved must have been saved before with saveData() method
   */
  public final void retrieveData() {
  	appletManager.restitueValeurs(this);
  }
  
  /**
   * Version 1.8 : mthode publique servant  recalculer et rafficher compltement la figure
   * en effaant tout et raffichant tout sans recalculer tout et sans sauvegarde pour rtroaction.
   * Sauvegarde les donnes pour rtroaction
   */  
  public final void redessineSansCalcul() {
  	/*
  	if (!premierAffichage) {
	    paneFigure.effacegBuffer();
	    paneFigure.afficheToutDansgbuffer();
	    paneFigure.repaint();
  	}
  	*/
  	aRedessiner = true;
  	paneFigure.repaint();
  }
  
  /**
   * public method redrawing the figure without recalculating it
   */
  public final void redraw() {
  	redessineSansCalcul();
  }
  
  /**
   * public method recalculating the figure.
   * @param updateRandom : if true, calls to rand function will be updated
   */
  public final void recalculateAndRedraw(boolean updateRandom) {
  	// infRand = true; // Dfaut corrig version 4.6
  	infRand = updateRandom;
  	aRecalculerEtRedessiner = true;
  	paneFigure.repaint();
  }
  
  // Ajout version 4.6
  /**
   * Fonction publique renvoyant 0 si un des champs d'dition de la figure a une faute
   * soit de syntaxe soit donnant un calcul non existant
   * Renvoie 1 sinon ou s'il n'y a pas de champ d'dition
   * sinon affiche un message d'erreur et si focus est  true redonne le focus au champ d'entre
   * @return
   */
  public final int fieldsValidation() {
  	ListIterator<CElementBase> li = listePr.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (elb.getNature() == NatureObjet.NEditeurFormule) {
  			CEditeurFormule ed = (CEditeurFormule) elb;
  			MtgFormulaEditor mtgfe = ed.mtgFormulaEditor;
  			// A noter :validation recalcule la figure mais ne la raffiche pas
  			if ((mtgfe.getText().isEmpty()) || !mtgfe.validation()) return 0;
  		}
  	}
  	// Si la rponse est bonne on recalcule tout et on raffiche la figure
  	recalculateAndRedraw(false);
  	return 1;
  }
  
  // Ajout version 4.6
  /**
   * Fonction regardant si un ou plusieurs champs d'dition sont associs  un
   * diteur de formule associ  un calcul ou une fonction nomm nomCalcul
   * Si oui, renvoie 1 si les rponses sont valides et 0 sinon ou s'il n'y a pas
   * de champ associ  ce calcul ou  cette fonction
   */
  public final int fieldValidation(String nomCalcul) {
  	boolean trouve = false;
  	ListIterator<CElementBase> li = listePr.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (elb.getNature() == NatureObjet.NEditeurFormule) {
  			CEditeurFormule ed = (CEditeurFormule) elb;
  			if (ed.calculAssocie.nomCalcul.equals(nomCalcul)) {
  				trouve = true;
	  			MtgFormulaEditor mtgfe = ed.mtgFormulaEditor;
	  			// A noter :validation recalcule la figure mais ne la raffiche pas
	  			if ((mtgfe.getText().isEmpty()) || !mtgfe.validation()) return 0;
  			}
  		}
  	}
  	// Si la rponse est bonne on recalcule tout et on raffiche la figure
  	recalculateAndRedraw(false);
  	if (trouve) return 1; else return 0;
  }
  
  // Ajout version 4.6
  /**
   * Fonction publique donnant le focus au premier champ d'dition invalide s'il en existe.
   */
  public final void activateFirstInvalidField() {
  	ListIterator<CElementBase> li = listePr.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (elb.getNature() == NatureObjet.NEditeurFormule) {
  			CEditeurFormule ed = (CEditeurFormule) elb;
  			MtgFormulaEditor mtgfe = ed.mtgFormulaEditor;
  			if (mtgfe.getText().isEmpty() || !mtgfe.validation()) {
  				ed.mtgFormulaEditor.requestFocus();
  				break;
  			}
  		}
  	}
  }

  // Ajout version 4.6
  /**
   * Fonction publique donnant le focus au premier champ d'dition invalide
   * associ au calcul nomm nomCalcul s'il en existe.
   */
  public final void activateField(String nomCalcul) {
  	ListIterator<CElementBase> li = listePr.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (elb.getNature() == NatureObjet.NEditeurFormule) {
  			CEditeurFormule ed = (CEditeurFormule) elb;
  			if (ed.calculAssocie.nomCalcul.equals(nomCalcul)) {
	  			MtgFormulaEditor mtgfe = ed.mtgFormulaEditor;
	  			if (mtgfe.getText().isEmpty() || !mtgfe.validation()) {
	  				ed.mtgFormulaEditor.requestFocus();
	  				break;
	  			}
  			}
  		}
  	}
  }
  
  /**
   * Marque le champ d'dition associ au calcul nomm nomCalcul comme invalide
   * en l'entourat de rouge.
   * @param nomCalcul
   */
  public void setInvalidField(String nomCalcul) {
  	ListIterator<CElementBase> li = listePr.listIterator();
  	while (li.hasNext()) {
  		CElementBase elb = li.next();
  		if (elb.getNature() == NatureObjet.NEditeurFormule) {
  			CEditeurFormule ed = (CEditeurFormule) elb;
  			if (ed.calculAssocie.nomCalcul.equals(nomCalcul)) {
	  			ed.mtgFormulaEditor.marquePourErreur();
	  			ed.mtgFormulaEditor.validate();
	  			break;
  			}
  		}
  	}  	
  }
  

  /** Lors du positionnement de la liste principale de l'applet, si le
   * paramtre Action de celle-ci contenait "Sauvegarde", il faut sauvegarder
   * via l'AppletManager les valeurs de res1,res2, ..., res10 (si elles existent)
   */
  public final void positionne() {
    listePr.positionne(infRand, dimf);
  }
  
  /** Fonction positionnant la liste pour la cration visuelle d'objets */
  public void positionneListeVisuelle() {
    listeObjetsVisuels.positionne(infRand, dimf);
  }
  
	/** Fonction affichant un dialogue d'avertissement */
	public void dialogueAvertissement(String texte) {
		JOptionPane.showMessageDialog(this, texte, getString("avertissement"),
				JOptionPane.WARNING_MESSAGE);
	}

	/** Fonction affichant un dialogue d'information */
	public void dialogueInformation(String texte) {
		JOptionPane.showMessageDialog(this, texte, getString("information"),
				JOptionPane.WARNING_MESSAGE);
	}  
	/**
	 * Fonction affichant un dialogue demandant une rponse oui-non et renvoyant
	 * true si la rponse est oui
	 */
	public boolean dialogueConfirmation_YES_NO(String texte) {
		return (JOptionPane.showConfirmDialog(this, texte, getString("confirmation"),
				JOptionPane.YES_NO_OPTION))== JOptionPane.YES_OPTION;
	}
  
  /** Fonction rajoutant un objet  la liste principale
   * Il faut pour cela mettre  zro la liste clone et la cloner  nouveau
   */
  public void ajouteElement(CElementBase pt) {
  	pt.positionne(false, dimf);
    CElementBase ptClone = pt.getClone(listePr, listeClone);
    listeClone.add(ptClone);
    listePr.add(pt);
    listeConstruction.add(pt);
    listeClone.setClone(listePr);
  }
  
	/**
	 * Fonction ajoutant l'lment pt avant l'lment el et dcale pt
	 * et les suivants
	 * 
	 * @param pt
	 * @param indice
	 */
	public void insereElement(CElementBase pt, CElementBase el) {
  	pt.positionne(false, dimf);
  	int indice1 = listePr.indexOf(el);
    CElementBase ptClone = pt.getClone(listePr, listeClone);
		listeClone.add(indice1, ptClone);
		listePr.add(indice1, pt);
		int indice2 = listeConstruction.indexOf(el);
		listeConstruction.add(indice2, pt);
    listeClone.setClone(listePr);
	}
  
  
  
  /** Fonction rajoutant un objet ? la liste des objets visuels */
  public void ajouteObjetVisuel(CElementBase pt) {
    listeObjetsVisuels.add(pt);
  }
  
  /** Fonction rajoutant un objet  la liste principale
   * Il faut pour cela mettre  zro la liste clone et la cloner  nouveau
   * Renvoie true si l'lment n'a pas dj t cr
   */
  /*
  public boolean ajouteElementAvecVerification(CElementBase pt) {
    if (!listeConstruction.existeDeja(pt)) {
      ajouteElement(pt);
      // positionne();
      return true;
    }
    else {
      dialogueAvertissement(getString("ch7);
      return false;
    }
  }
  */
	public boolean ajouteElementAvecVerification(CElementBase pt) {
		if (!listeConstruction.existeDeja(pt)) {
			if ((pt.getNature() & NatureObjet.NToutObjet) != 0) {
				if (pt.antecedentDirect() != pt)
					pt.antecedentDirect().positionne(false, dimf);
				pt.positionne(false, dimf);
				if (!((CElementGraphique) pt).dansFenetre(dimf)) {
					if (!dialogueConfirmation_YES_NO(getString("ch94")))
						return false;
				}
			}
			ajouteElement(pt);
			positionne();
			return true;
		} else {
			dialogueAvertissement(getString("ch7"));
			return false;
		}
	}
	
	/**
	 * Fonction rajoutant un objet image par transformation  la liste principale
	 * de la fentre active Renvoie true si l'lment n'a pas dj t cr
	 * 
	 * @param nbEltsAjoutes
	 *          reprsente le nombre d'lments qui ont t ventuellement 	 *          ajouts
	 * @param trans
	 *          est la transformation lors de la cration de l'objet (par exemple
	 *          pour un segment image les points images des extrmits)
	 */
	public boolean ajouteElementsImageAvecVerification(CElementBase pt,
			int nbEltsAjoutes, CTransformation trans) {
		if (!listeConstruction.existeDeja(pt)) {
			// On commence par positionner la tansformation qui est le premier des
			// objets ajouts  la liste
			// avant les objets ajouts ventueks
			trans.positionne(false, dimf);
			pt.antecedentDirect().positionne(false, dimf);
			// Il faut positionner les ventuels lments ajouts
			for (int i = nbEltsAjoutes; i >= 1; i--) {
				CElementBase el = listePr.get(listePr.size() - i);
				el.positionne(false, dimf);
			}
			// On positionne aussi l'image  rajouter
			pt.positionne(false, dimf);
			if (!((CElementGraphique) pt).dansFenetre(dimf)) {
				if (!dialogueConfirmation_YES_NO(getString("ch94")))
					return false;
			}
			ajouteElement(pt);
			positionne();
			return true;
		} else {
			if (nbEltsAjoutes != 0)
				detruitNDerniersElements(nbEltsAjoutes);
			dialogueAvertissement(getString("ch7"));
			return false;
		}
	}  	
	
	/* Fonction dtruisant le dernier objet de la liste ListePr et de sa liste
   * clone listeClone */
  public void detruitDernierElement() {
    listeClone.detruitDernierObjet();
    listePr.detruitDernierObjet();
    listeConstruction.detruitDernierObjet();
    listeProposee.detruitDernierObjet();
  }
  // Idem avec destruction des n derniers lments
  public void detruitNDerniersElements(int n) {
    for (int i = 0; i < n; i++) {
      detruitDernierElement();
    }
  }
  
  
  // Affichage d'une chaine d'indication dans la zone JTextArea en bas de l'?cran
  /**
   * Affichage d'une chaine d'indication dans la zone JTextArea en bas de l'?cran
   */
  public void indication(String ch) {
    indicationArea.setText(ch);
    indicationArea.repaint();
  }
  
  public void retireDependants(CElementBase pt) {
    int indice = listePr.indexOf(pt);
    listePr.retireDependants(pt);
    annuleSolutionProposee();
    etablitListeObjetsConstruits();
    CElementBase ptclone = (CElementBase) listeClone.get(indice);
    listeClone.retireDependants(ptclone);
  }
  
  public int nombreObjetsInitiaux() {
    return listeSauve.size();
  }
  
  public void etablitListeObjetsConstruits() {
		int nbo = nombreObjetsInitiaux();
		listeConstruction.retireTout();
		// On ajoute  la liste des objets construits le calcul constant pi de la liste principale
		// de faon  pouvoir l'utiliser dans les calculs
		listeConstruction.add(listePr.get(0));
		// Nouveau version 4.6 : On ajoute les lments de type calcul qui sont autoriss via le commentaire de la macro solutionConstruction
		if (isExercise) {
			etablitListeCalculsAutorises();
			for (int i = 0; i < listeCalculsAutorises.size(); i++) {
				listeConstruction.add(listeCalculsAutorises.get(i));
			}
		}
		for (int i = nbo; i < listePr.size(); i++) {
			listeConstruction.add(listePr.get(i));
		}
  }
  /**
   * Fonction renvoyant une chane permettant  l'utilisateur de savoir quel est
   * l'objet ni qu'il doit dsigner quand il dsigne les objets qu'on lui a demand de coNStruire
   * @param i
   * @return
   */
  public String indicationSol(int i)
  {
    String ret = "";
    String ch = "#" + glob.Kernel.chaineNombre(i, 0) +":";
    int j = chaineIndicationPourCons.indexOf(ch);
    if (j != -1)
    {
      int k = chaineIndicationPourCons.indexOf("#", j + ch.length());
      if (k != -1)
      {
        ret = chaineIndicationPourCons.substring(j + ch.length(), k);
      }
      else ret = chaineIndicationPourCons.substring(j + ch.length());
      // Recherche de retour chariot et saut
      int m = ret.indexOf("\r\n");
      if (m != -1) ret = ret.substring(0,m);
    // On ne prend que les 80 premiers caractres
    if (ret.length() > 80) ret = ret.substring(0, 81);
    }
    return ret;
  }
  /**
   * 
   * @return true si les objets qui ont t cliqus et sont contenus dans listeProposee
   * sont confondus avec ceux qu'ils faut construire contenus dans listeAConstruire
   */
  private boolean reponseValideUnePasse() {
    boolean res = true;
    int nbo = listeAConstruire.size();
    for (int i = 0; (i < nbo) && res; i++) {
      CElementBase elb1 = (CElementBase) listeAConstruire.get(i);
      boolean res2 = false;
      for (int j = 0; (j< nbo) && !res2; j++) {   
        CElementBase elb2 = (CElementBase) listeProposee.get(j);
        res2 = res2 || elb1.coincideAvec(elb2);
      }
      res = res && res2;
    }
    return res;
  }
  public boolean reponseValide() {
    // On regarde d'abord si les objets proposs coincident bien avec ceux 
    // construire
  	// listeClone.setClone(listePr);
    boolean res = reponseValideUnePasse();
    if (!res) return false;
    // La liste listeClone contient des clones des objets dans l'tat o ils sont actuellement
    // On remet les lments du texte de la figure dans l'tat o ils taient au chargement
    // de la figure.
    // C'est ncessaire car la mthode de vrification va dplacer lgrement
    // les points mobiles pour tester la validit de la rponse et ils faut que la
    // construction reste possible lors de ces dplacements
    listePr.setClone(listeSauve);  // Le clonage n'est fait que pour les lments initiaux de la figure
    listePr.positionne(infRand, dimf);
    CListeObjets listePtsLibres = new CListeObjets();
    CListeObjets listePtsLies = new CListeObjets();
    int nbo = listeSauve.size();
    for (int j = 0; j < nbo; j++) {
      CElementBase elb = (CElementBase) listePr.get(j);
      if ((elb.getNature() == NatureObjet.NPointBase) && (listeAConstruire.dependDe(elb)))
        listePtsLibres.add(elb);
      if ((elb.getNature() == NatureObjet.NPointLie) && (listeAConstruire.dependDe(elb)))
        listePtsLies.add(elb);
    }
    // On va 3 fois de suite dplacer un peu les points libres et lis dont dpendent
    // les objets  construire et qui font partie de la figure initiale et regarder
    // si la solution reste valide. Si oui elle sera accepte sinon refuse
    // listeClone.setClone(listePr);
    for (int i = 0; (i < 3) && res; i++) {
      // On dplace un peu les points libres et lis
      ListIterator<CElementBase> li1 = listePtsLibres.listIterator();
      while (li1.hasNext()) {
        CPointBase ptb = (CPointBase) li1.next();
        epsilon(ptb);
      }
      ListIterator<CElementBase> li2 = listePtsLies.listIterator();
      while (li2.hasNext()) {
        CPointLie ptl = (CPointLie) li2.next();
        epsilon(ptl);
      }
      listePr.positionne(infRand, dimf);
      res = res && reponseValideUnePasse();
      listePr.setClone(listeSauve);
      listePr.positionne(infRand, dimf);
    }
    // On remet les objets dans la position o les a mis l'utilisateur
    listePr.setClone(listeClone);
    return res;
  }
  
  /**
   * Fonction repositionnant lgrement (maximum 5 piles en + ou en - dans
   * chaque direction le point libre ptb
   * Utilis lors de la validation de la rponse
   * @param ptb
   */
  
  // Modifi version 3.4.2.1
  /*
  private void epsilon(CPointBase ptb) {
    double x1 = 0;
    double y1 = 0;
    double a = Math.floor(Math.random()*5) + 1;
    double b = Math.floor(Math.random()*5) + 1;
    double s1 = Math.floor(Math.random()*2);
    double s2 = Math.floor(Math.random()*2);
    if (s1 == 0) x1 = a; else x1 = -a;
    if (s2 == 0) y1 = b; else y1 = -b;
    ptb.placeEn(ptb.x + x1, ptb.y + y1);
  }
  */
  private void epsilon(CPointBase ptb) {
    double x1 = 0;
    double y1 = 0;
    double a = (Math.floor(Math.random()*5) + 1)*0.001;
    double b = (Math.floor(Math.random()*5) + 1)*0.001;
    double s1 = Math.floor(Math.random()*2);
    double s2 = Math.floor(Math.random()*2);
    if (s1 == 0) x1 = a; else x1 = -a;
    if (s2 == 0) y1 = b; else y1 = -b;
    ptb.placeEn(ptb.x + x1, ptb.y + y1);
  }
  
  /**
   * Fonction donnant  l'abscisse du point li une nouvelle valeur s'cartant au maximum
   * de 5 % vers la haut ou la bas par rapport  l'abscisse initiale.
   * @param ptl
   */
  // Modifi version 3.4.2.1
  /*
  private void epsilon(CPointLie ptl) {
    double absmin = ptl.abscisseMinimale();
    double absmax = ptl.abscisseMaximale();
    // on dcale l'abscisse au maximum de 5% vers la haut ou vers la bas au hasard
    double a = Math.floor(Math.random()*5) + 1;
    double s1 = Math.floor(Math.random()*2);
    if (s1 == 0) a  = -a;
    double nabs = ptl.abscisse + a*(absmax - absmin)/100;
    if (nabs > absmax) nabs = absmax;
    else if (nabs < absmin) nabs = absmin;
    ptl.donneAbscisse(nabs);
  }
  */
  private void epsilon(CPointLie ptl) {
    double absmin = ptl.abscisseMinimale();
    double absmax = ptl.abscisseMaximale();
    // on dcale l'abscisse au maximum de 5% vers la haut ou vers la bas au hasard
    double a = Math.floor(Math.random()*5) + 1;
    double s1 = Math.floor(Math.random()*2);
    if (s1 == 0) a  = -a;
    double nabs = ptl.abscisse + a*(absmax - absmin)/10000;
    // if (nabs > absmax) nabs = absmax;
    // else if (nabs < absmin) nabs = absmin;
    if ((nabs > absmin) && (nabs < absmax)) ptl.donneAbscisse(nabs);
    ptl.donneAbscisse(nabs);
  }
  /**
   * Fonction renvoyant true lorsque l'exercice n'est pas un exercice de construction
   * ou lorsque l'exercice est unn exercice de construction et que l'utilisateur
   * a bien dsign tous les objets qu'il pense tre ceux qu'on lui demandait de construire
   * @return
   */
  public boolean solutionProposee() {
    if (!isConstruction()) return true;
    return (listeAConstruire.size() == listeProposee.size());
  }
  
  /**
   * Renvoie la couleur active qui est donne par le bouton colorButton
   * @return
   */
  public Color couleurActive() {
    return colorButton.couleur;
  }
  /**
   * Renvoie la nature des objets dont on peut crer l'image par une transformation
   * Les points sont autoriss.
   * Ne sert que pour un exercice de construction
   * Les droites sont autorises si la macro macroPourCons  un commentaire qui
   * contient la squence de caractres ImageDroite. Idem pour les demi-droites
   * cercles et arcs de cercles
   * @return
   */
  public long natureObjetsPourImages() {
  	if (!isConstruction()) return NatureObjet.NToutPoint | NatureObjet.NDroite | NatureObjet.NDemiDroite | NatureObjet.NCercle | NatureObjet.NArc;
    long res = NatureObjet.NToutPoint;
    if ((chaineIndicationPourCons.indexOf(getString("imageDroite")) != -1)
    	|| (chaineIndicationPourCons.indexOf("imageDroite") != -1)) res |= NatureObjet.NDroite;
    if ((chaineIndicationPourCons.indexOf(getString("imageDemiDroite")) != -1)
    	|| (chaineIndicationPourCons.indexOf("imageDemiDroite") != -1))res |= NatureObjet.NDemiDroite;
    if ((chaineIndicationPourCons.indexOf(getString("imageCercle")) != -1) 
    	|| (chaineIndicationPourCons.indexOf("imageCercle") != -1))res |= NatureObjet.NCercle;
    if ((chaineIndicationPourCons.indexOf(getString("imageArc")) != -1) 
    	|| (chaineIndicationPourCons.indexOf("imageArc") != -1))res |= NatureObjet.NArc;
    return res;
  }
  /**
   * Fonction mettant  chane vide la chaine utilise pour la dsignation d'objets
   */
  public void videDesignation() {
    commentaireDesignation.donneChaine("");
  }
  
  /**
   * Fonction retirant tous les objets de la liste utilise pour proposer une solution
   * 
   */
  public void annuleSolutionProposee() {
    listeProposee.retireTout();
  }
  
  // Version 2.0 : mthode publique servant  remettre la figure dans l'tat o
  // elle tait  l'issu de son premier affichage
  public final void reset()
  {
    if (isConstruction()) {
      if (macroEnCours != null) {
        macroEnCours.termineAction(this);
        timerTask = null;
        if (timer != null){
          timer.cancel();
          timer = null;
        }
        if (runner != null) runner = null;
        macroEnCours.termineAction(this);
        macroEnCours = null;
      }
      listeConstruction.retireTout();
      listePr.retireTout();
      listeSauve.setCopy(listePr);
      listePr.setClone(listeSauve);
      //premierAffichage = true;
      activeOutilCapture();
      paneFigure.effacegBuffer();
      repaintFigureAvecCalcul();
      repaint();
    }
  }
  
  public Frame getParentFrame() {
    Container c = getParent();
    while (c != null) {
      if (c instanceof Frame)return (Frame) c;
      c = c.getParent();
    }
    return null;
  }
    
  
	/**
	 * Cration de la fentre associe aux variables pour lesquelles
	 * un dialogue associ a t demand
	 */
	public void creePaneAffichageVariables() {
		if ((paneAffichageVariables == null) && (listePr.nombreVariablesAvecDialogueAssocie() != 0)) {
			paneAffichageVariables = new PaneAffichageVariables(this);
			paneAffichageVariables.setVisible(true);
		}
	}
	
	public void metAJourPaneAffichageVariables(CVariableBornee var) {
		if (paneAffichageVariables != null) paneAffichageVariables.metAJourAffichageDe(var);
	}	

	public void setConstruction(boolean isConstruction) {
		this.isConstruction = isConstruction;
	}

	public boolean isConstruction() {
		// Modifi version 4.7.2.6
		// return isConstruction;
		return allowTools && isConstruction;
	}

	public void setToolsAvailable(boolean toolsAvailable) {
		this.toolsAvailable = toolsAvailable;
	}

	public boolean isToolsAvailable() {
		return toolsAvailable;
	}
	
	/**
	 * Fonction renvoyant le syle de trait actif
	 * 
	 * @return
	 */
	public int styleTraitActif() {
		return listeStyleTrait.styleTrait();
	}	
	
  private void loadFigureFromDataBase64(String data) {
    ByteArrayInputStream ba = new ByteArrayInputStream(Kernel.base64Decode(data));
		try {
			MathGraphDataInputStream inps = new MathGraphDataInputStream(ba);
			document.chargeDepuisFlux(inps);
			inps.close();
    	String chAcces = cheminAcces();			
      modeTrace = document.modeTraceActive;
      // On regarde si une couleur de fond est spcifie dans la figure.
      if (!couleurFondParParam) {
        couleurFond = document.couleurFond;
      }
      listePr.setCopy(listeClone);
      listePr.setCopy(listeSauve);
      listeDependantsDansPrincipale = new CListeObjets(listePr);
      listeMacrosJouantSon = new CListeObjets(listePr);
      listeMacrosJouantSon.ajouteMacrosJouantSon(listePr);
      // Si ncessaire on charge les fichiers sonores des macros jouant des sons
      if (listeMacrosJouantSon.size() != 0) {
        CListeObjets list = listeMacrosJouantSon;
        ListIterator<CElementBase> li = list.listIterator();
        while(li.hasNext()) {
          CMacroJouantSon mac = (CMacroJouantSon) (li.next());
          mac.clip = getAudioClip(getDocumentBase(), chAcces + mac.wavePath);
        } 
      }
      macroParDefaut = listePr.macroParDefaut;
      if (macroParDefaut != null) {
        if (macroParDefaut instanceof CMacroAnimationPointLie)  {
          macroAnimation = (CMacroAnimationPointLie) macroParDefaut;
          macroParDefaut = null;
          long freq = macroAnimation.frequenceAnimationMacro;
          if (freq < 10) macroAnimation.frequenceAnimationMacro = 10;
          //macroAnimation.actionAvantExecution(this);
        }
      }
      // Une fois l'applet entirement charge, et si son paramtre Action
      // contenait "Restitution", on affecte le contenu des calculs res1, res2, ..., res10
      // de la figure prcdemment sauvegardes via l'AppletManger
      // aux variables var1, var2, ..., var10 de faon  pouvoir "reconstituer"
      // la figure de l'applet prcdente
      if (action.equals("Restitution")) {
        appletManager.restitueValeurs(this);
      }
    }
    catch (Exception e) {
      System.out.println("Erreur pendant le chargement");
      System.out.println(e.getMessage() + e.toString());
      System.exit(-1);
    }  
  }
  
	/*
	 * Spcial Applet version 4.6 : Renvoie si elle existe un pointeur sur une macro de pause
	 * dont le commentaire contiendra les noms des calculs autoriss  l'utilisateur
	 */
	private CMacroPause macroCalculsAutorises() {
		ListIterator<CElementBase> li = listePr.listIterator();
		while (li.hasNext()) {
			CElementBase elb = (CElementBase) li.next();
			if (elb.getClass() == CMacroPause.class) {
				CMacroPause mac = (CMacroPause) elb;
				// Modifi version 4.7.2 . Si une macro de pause nomme calculs est prsente, elle fournit dans son commentaire
				// la liste des calculs que l'lve peut utiliser contenus dans des accolades
				/*
				if ((mac.intitule.compareToIgnoreCase("exercice") == 0)
						|| (mac.intitule.compareToIgnoreCase(getString("exercice")) == 0)) return mac;
				*/
				if ((mac.intitule.compareToIgnoreCase("calcul") == 0)
						|| (mac.intitule.compareToIgnoreCase(getString("calcul")) == 0)) return mac;
			}
		}
		return null;
	}

	public void etablitListeCalculsAutorises() {
		if (isExercise) {
			listeCalculsAutorises.retireTout();
			// Modifi version 4.7
			// CMacro mac;
			// if (cadreActif.isConstruction) mac = cadreActif.macroPourCons;
			// else  mac = cadreActif.macroPomacurCalculs;
			CMacro mac = macroPourCalculs;
			ListIterator<CElementBase> li = listePr.listIterator();
			while (li.hasNext()) {
				CElementBase elb = li.next();
				if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculNommeSaufConst)) {
					CCalculAncetre calc = (CCalculAncetre) elb;
					if (mac.commentaireMacro.indexOf("{"+calc.nomCalcul+"}") != -1) {
						listeCalculsAutorises.add(calc);
						// cadreActif.listeConstruction.add(calc); Modifi version 4.7
					}
				}
			}
		}
	}

	/**
	 * Spcial version applet version 4.6:
	 * Renvoie un pointeur sur la liste utilise pour les constructions suivant que l'applet sert ou non
	 *  faire un exercice de construction
	 * @return
	 */
	public CListeObjets listePourConstruction() {
		if (isConstruction() || isExercise) return listeConstruction;
		else return listePr;
	}

  /**
   * Ajout version 4.6.
   * Met  vide touys les champs d'dition
   */
  public void setEditorsEmpty() {
    ListIterator<CElementBase> li = listePr.listIterator();
    while (li.hasNext()) {
      CElementBase elb = (CElementBase) li.next();
      if (elb.getNature() == NatureObjet.NEditeurFormule) {
      	CEditeurFormule ed = (CEditeurFormule) elb;
      	ed.mtgFormulaEditor.setText("");
				ed.formula = null; // Ajout version 4.7.2
     }
    }
  }

	public final boolean isReady() {
		return isReady;
	}
	
  /**
   * Fonction retirant les composants crs sur la figure : Eventuel paneAffichageVariable
   * et champs d'dition de formule
   */
  public void retireComposants() {
  	if (paneAffichageVariables != null) {
			paneCenter.remove(paneAffichageVariables);
			paneAffichageVariables = null;
  	}
    java.util.ListIterator <CElementBase> li = listePr.listIterator();
    while (li.hasNext()) {
    	CElementBase elb = li.next();
    	elb.retireComposant(this);
    }
		paneCenter.validate();
  }
	
	// Fonctions spcifiques version Applet
	public final String getScript() {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(2000);
		MathGraphDataOutputStream oups = new MathGraphDataOutputStream(baos);
		// On enregistre la liste d'objets principale dans l'archive
		try {
			document.write(oups);
			oups.flush();
			oups.close();
			// ch = baos.toString();
			baos.flush();
			baos.close();
			byte[] ba = baos.toByteArray();
			// return Kernel.base64Encode(ba);
			return Kernel.base64Encode(ba);
		} catch (IOException e) {
			return "";
		}
	}

	/**
	 * Fonction chargeant une icne dans le dossier images de mathgraph32
	 * @param iconName
	 * @return
	 */
	private ImageIcon loadIcon(String iconName) {
    URL adr = AppletMathGraph32.class.getResource("images/" + iconName + ".png");
    return new ImageIcon(adr);
	}

	private void loadToolsIcons() {
		if(iconAnnuler == null) iconAnnuler = loadIcon("Undo24");
		if(iconRefaire == null) iconRefaire = loadIcon("Redo24");
		if(iconSupprimerObjetGraphique == null) iconSupprimerObjetGraphique = loadIcon("Delete24");
		if(iconHistorique == null) iconHistorique = loadIcon("Historique24");
		if(iconNommer == null) iconNommer = loadIcon("Nommer");
		if(iconCapture == null) iconCapture = loadIcon("Main24");
		if(iconCreerPointLibre == null) iconCreerPointLibre = loadIcon("PointLibre");
		if(iconCreerPointLie == null) iconCreerPointLie = loadIcon("PointLie");
		if(iconCreerMilieu == null) iconCreerMilieu = loadIcon("Milieu");
		if(iconCreerPointParAbscisse == null) iconCreerPointParAbscisse = loadIcon("PointParAbs");
		if(iconCreerPointParCoordonnees == null) iconCreerPointParCoordonnees = loadIcon("PointParCoordonnees");
		if(iconModifierObjetGraphique == null) iconModifierObjetGraphique = loadIcon("ModificationObjetGraphique");
		if(iconModifierFormule == null) iconModifierFormule = loadIcon("ModificationObjetNumerique");
		if(iconCreerSegment == null) iconCreerSegment = loadIcon("Segment");
		if(iconCreerDroiteAB == null) iconCreerDroiteAB = loadIcon("DteAB");
		if(iconCreerDroiteParallele == null) iconCreerDroiteParallele = loadIcon("DtePar");
		if(iconCreerDroitePerpendiculaire == null) iconCreerDroitePerpendiculaire = loadIcon("DtePer");
		if(iconCreerDroiteMediatrice == null) iconCreerDroiteMediatrice = loadIcon("DteMed");
		if(iconCreerDroiteBissectrice == null) iconCreerDroiteBissectrice = loadIcon("DteBis");
		if(iconCreerDemiDroite == null) iconCreerDemiDroite = loadIcon("DemiDt");
		if(iconCreerCercleOA == null) iconCreerCercleOA = loadIcon("CercOA");
		if(iconCreerCercleOR == null) iconCreerCercleOR = loadIcon("CercOR");
		if(iconCreerPolygone == null) iconCreerPolygone = loadIcon("Polygone");
		if(iconCreerIntersection == null) iconCreerIntersection = loadIcon("Int");
		if(iconCreerImageParTranslation == null) iconCreerImageParTranslation = loadIcon("Translation");
		if(iconCreerImageParSymetrieAxiale == null) iconCreerImageParSymetrieAxiale = loadIcon("SymAxiale");
		if(iconCreerImageParSymetrieCentrale == null) iconCreerImageParSymetrieCentrale = loadIcon("SymCentrale");
		if(iconCreerImageParRotation == null) iconCreerImageParRotation = loadIcon("Rotation");
		if(iconCreerImageParHomothetie == null) iconCreerImageParHomothetie = loadIcon("Homothetie");
		if(iconRapporteur == null) iconRapporteur = loadIcon("Rapporteur");
		if(iconCreerCalcul == null) iconCreerCalcul = loadIcon("Calcul");
		if(iconMesureLongueur == null) iconMesureLongueur = loadIcon("MesLong");
		if(iconMesureAngleNonOriente == null) iconMesureAngleNonOriente = loadIcon("MesAngNor");
		if(iconMesureAngleOriente == null) iconMesureAngleOriente = loadIcon("MesAngOr");
		if(iconMesureAbscisse == null) iconMesureAbscisse = loadIcon("MesAbs");
	}
  
	// Ajout version 4.7.4
	private void prepareOutils() {
	  outilAnnuler = new OutilAnnuler(this);
	  outilRefaire = new OutilRefaire(this);
	  outilCapture = new OutilCapture(this);
	  outilPtLib = new OutilPtLib(this);
	  outilMilieu = new OutilMilieu(this);
	  outilDtAB = new OutilDtAB(this);
	  outilDtPar = new OutilDtPar(this);
	  outilDtPer = new OutilDtPer(this);
	  outilSeg = new OutilSeg(this);
	  outilDemiDt = new OutilDemiDt(this);
	  outilCerOA = new OutilCerOA(this);
	  outilBis = new OutilBis(this);
	  outilMed= new OutilMed(this);
	  outilInt = new OutilInt(this);
	  outilTrans = new OutilTranslation(this);
	  outilSymAxiale = new OutilSymAxiale(this);
	  outilSymCentrale = new OutilSymCentrale(this);
	  outilRapporteur = new OutilRapporteur(this);
	  outilRot = new OutilRot(this);
	  outilCerOR = new OutilCerOR(this);
	  outilSup = new OutilSup(this);
	  outilPtLie = new OutilPtLie(this);
	  outilCalcul = new OutilCalcul(this);
	  outilPtParAbs = new OutilPtParAbs(this);
	  outilHom = new OutilHom(this);
	  outilMesLong = new OutilMesLong(this);
	  outilMesAngnor = new OutilMesAngNor(this);
	  outilMesAngOr = new OutilMesAngOr(this);
	  outilMesAbs = new OutilMesAbs(this);
	  outilPolygone = new OutilPolygone(this);
	  outilPointParCoordonnees = new OutilPointParCoordonnees(this);
	  outilModifierBarycentre = new OutilModifierBarycentre(this);
	  outilModifierObjetGraphique =new OutilModifierObjetGraphique(this);
	  outilModifierFormule = new OutilModifierFormule(this);
	  // Outil servant  faire apparatre le dialogue d'historique
	  outilHistorique = new OutilHistorique(this);
	  // Outil servnt  nommer un point ou une droite
	  outilEntreeNom = new OutilEntrerNom(this);
	  // Outil servant  cliquer sur les objets qu'on a demand de construire
	  outilPropSol = new OutilPropSol(this);
		
	}
}
