/*
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import javax.swing.*;
import objets.*;
import types.*;
import Applet.*;

/**
 *
 * @author Yves
 * Classe ancetre de tous les diteurs insrs dans une boite de dialogue et chargs
 * de modifier la valeur d'une valeur dynamique
 */
@SuppressWarnings("serial")
public class EditeurValeur extends JTextField {
  protected AppletMathGraph32 appletMere;
  private CValeurAssocieeAVariable val; // L'objet CValeurDynamique que ce JTextField doit modifier.
  private CElementBase objet; // L'objet qui contient val et qui doit tre modifi
  
  /** Creates a new instance of EditeurValeur */
  public EditeurValeur(AppletMathGraph32 appletMereInit, 
      CValeurAssocieeAVariable valInit, CElementBase objetInit) {
    super(30);
    appletMere = appletMereInit;
    val = valInit;
    objet = objetInit;
    setEditable(true); 
  }
  /* Vrification de la validit de la rponse contenue dans le champ d'dition
   * du JTextField
  */
  public final boolean verifieValidite() {
    double v;
    int indice;
    // Pour la version applet on ne doit pouvoir utiliser que des objets
    // crs par l'utilisateur
    //  CListeObjets liste = appletMere.listePr; Modifi version 4.6
    CListeObjets liste = appletMere.listePourConstruction(); // Spcial Applet : la liste pour vrification de syntaxe
    CListeObjets listePr = appletMere.listePr; // Spcial Applet la liste pour la cration du calcul
    
    String rep = (String) getText();
    if (rep.length() == 0) {
    	appletMere.dialogueAvertissement(AppletMathGraph32.getString("ch97"));
    	requestFocus();
    	return false;
    }
    // Dans le cas ou on cre un nouvel objet, celui-ci n'a pas encore t ajout
    //  la liste des objets de l'applet.
    int indobj = liste.indexOf(objet);
    if (indobj == -1) indice = liste.size() - 1;
    else indice = indobj - 1;

    Int inderr = new Int();
    // On vrifie d'abord la syntaxe de la rponse
    // La seule valeur qu'on a le droit d'utiliser est pi qui est toujours
    // le premier objet de la liste
    if (CCalculBase.verifieSyntaxe(liste, rep, inderr, indice, null)){
    
      CCalculBase calc = CCalculBase.creeCalculBase(rep ,listePr, 0, rep.length()-1, null);
      try {
        v = calc.resultat(false);
      }
      catch (ErreurCalculException e) {
        appletMere.dialogueAvertissement(AppletMathGraph32.getString("ch44"));
        requestFocus();
        return false;
      }

      // Si tout est correct, on affecte le rsultat du calcul propos
      //  la valeur dynamique que ce combo doit modifier.
      // Les deux lignes suivantes sont  revoir car initialement dans MathGraph32
      // on ne peut pas rentrer un calcul comme angle.
      // val.chaineCalcul = rep;
      // val.calcul = calc;
      if (valeurAcceptee(v))  {
        val.donneCalcul(calc);
        return true;
      }
      else return false;
    }
    else {
      // On place le curseur  l'endroit de l'erreur de syntaxe
      select(inderr.getValue(), inderr.getValue());
      appletMere.dialogueAvertissement(AppletMathGraph32.getString("ch5"));
      requestFocus();
      return false;
    }
  }
  
  /*
   * Fonction charge de vrifier si la rponse, quand elle est correcte, est valide
   * pour le type de valeur attendu. Par exemple, le rayon d'un cercle doit tre non nul.
   * Renvoie true par dfaut et doit tre redfinie pour tous les descendants.
   *
  */
  public boolean valeurAcceptee(double v) {
    return true;
  }
}
