/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.*;

import javax.swing.*;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import Applet.AppletMathGraph32;
import objets.CValeurDynamique;
import types.FonteNom;
import types.NatureCalcul;
import java.awt.event.*;

/**
 * Dialogue associ  un JTextfield, prsentant les valeurs relles jusqu' indiceMaxi.
 * Lorsqu'on quitte la valeur choisie est insre au point d'insertion du JTextField utilis
 */
@SuppressWarnings("serial")
public class ListeValeursOuFonctionsReellesTextFieldDlg extends MtgDlg implements ActionListener, MouseListener {
  private JTextField editeur; // Le textField dans lequel sera envoy le choix
  @SuppressWarnings("unused")
	private int indiceMaxi;         // L'indice maxi dans la liste d'objets du dernier objet qui puisse
                          // figurer dans l'diteur
  private boolean intermPermis; // True si les objets intermdiaires sont autoriss dans la liste
  private JLabel label;
  private ListeValeursOuFonctionsReelles listeVal;
  private JButton btnOK, btnCancel;  
  private JTextArea info;  
  
  /** Creates a new instance of ListeValeursComboDlg */
  public ListeValeursOuFonctionsReellesTextFieldDlg(AppletMathGraph32 appletInit, JTextField editeurInit,
      int indiceMaxiInit, boolean intermPermisInit) {
    super(AppletMathGraph32.getString("listeValeursDlg1"),true, appletInit);
    editeur = editeurInit;
    indiceMaxi = indiceMaxiInit;
    intermPermis = intermPermisInit;
    label = new JLabel(AppletMathGraph32.getString("listeValeursDlg3"));
    btnOK = new JButton(AppletMathGraph32.getString("btnOK"));
    btnCancel = new JButton(AppletMathGraph32.getString("btnCancel"));
    info = new JTextArea();
    info.setFont(FonteNom.getFonteNom(2, FonteNom.Normal));    
    JScrollPane scrollPane = new JScrollPane(info, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    info.setEditable(false);
    info.setColumns(20);
    info.setEditable(false);
    info.setLineWrap(false);
    info.setRows(4);
    scrollPane.setPreferredSize(new Dimension(350, 90));
    // Modification version 4.6
    // listeVal = new ListeValeursOuFonctionsReelles(applet.listeConstruction, indiceMaxiInit, intermPermis);
    listeVal = new ListeValeursOuFonctionsReelles(applet.listePourConstruction(), indiceMaxiInit, intermPermis);
    getContentPane().setLayout(new GridBagLayout());
    GridBagConstraints constraints = new GridBagConstraints();
    setConstraints(constraints, 0, 0, 1, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.top = 4;
    constraints.insets.bottom = 4;
    constraints.insets.left = 5;
    constraints.insets.right = 5;

    getContentPane().add(label, constraints);
    constraints.insets.top = 0;
    constraints.insets.bottom = 5;    
    setConstraints(constraints, 0, 1, 1, 1, 0, 70, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    JScrollPane scrollList = new JScrollPane(listeVal, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    
    getContentPane().add(scrollList, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 100, 30, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    getContentPane().add(scrollPane, constraints);
    
    constraints.insets.bottom = 0;
    
    JPanel paneDroit = new JPanel(); // Pour mettre les deux boutons en haut et  droite
    paneDroit.setLayout(new GridLayout(2,1,0,5));
    paneDroit.add(btnOK);
    paneDroit.add(btnCancel);
    
    setConstraints(constraints, 1, 1, 1, 1, 20, 0, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    getContentPane().add(paneDroit, constraints);

    setContentPane(getContentPane());
    btnOK.addActionListener(this);
    btnCancel.addActionListener(this);
    listeVal.addMouseListener(this);
    listeVal.addListSelectionListener(new ListSelectionListener() {
      public void valueChanged(ListSelectionEvent evt) {
      	listValSelectionChanged();
      }
    });
    
    getRootPane().setDefaultButton(btnOK);
    listValSelectionChanged();
    pack();  
  }

  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource()== btnOK) {
      actionButtonOK();
    }
    else {
      if (evt.getSource()== btnCancel) {
        dispose();
        editeur.requestFocus(); // Ajout version 4.5.2
      }
    }   
  }

  public void actionButtonOK() {
		int i = editeur.getCaretPosition();
    String ch = editeur.getText();
    CValeurDynamique val = listeVal.valeurSelectionnee();
    String nom = val.nom();
    int longNom = nom.length();
    String chdebut = nom;
    String chfin;
  	// Si on colle le nom d'une fonction on colle aussi la parenthse
  	// et les virgules ventuelles
    int decalage = 0;
    boolean fonction = true;
  	if (val.estDeNatureCalcul(NatureCalcul.NFonction | NatureCalcul.NDerivee | NatureCalcul.NSuiteRecurrenteReelle)) {
  		chdebut = chdebut + "(";
  		chfin = ")";
  		decalage++;
  	}
  	else {
    	if (val.getNatureCalcul() == NatureCalcul.NFonction2Var) {
    		chdebut = chdebut + "(";
    		chfin = ",)";
    		decalage++;
    	}
    	else {
      	if (val.getNatureCalcul() == NatureCalcul.NFonction3Var) {
      		chdebut = chdebut + "(";
      		chfin = ",,)";
      		decalage++;
      	}
      	else{
      		chfin = "";
      		fonction = true;
      	}
    	}
  	}
		String selection = editeur.getSelectedText();
		if (selection != null) {
			i = ch.indexOf(selection) + selection.length();
			if (fonction) decalage ++;
			editeur.setText(ch.substring(0,editeur.getSelectionStart()) + chdebut
					+ selection + chfin + ch.substring(editeur.getSelectionEnd()));
		}
		else {
	    String ch2 = ch.substring(0, i) + chdebut + chfin + ch.substring(i);
	    editeur.setText(ch2);
		}
		dispose(); // Modification version 4.5.2. Doit tre avant les deux lignes suivantes
		// pour que dans le cas o on dite un mtgFormulaEditor celui retrouve bien le focus
	  editeur.setCaretPosition(longNom + i + decalage);
	  editeur.requestFocus();
		// dispose(); 
  }

  public void listValSelectionChanged() {
  	int i = listeVal.getSelectedIndex();
  	if (i != -1) {
  		info.setText(listeVal.pointeurs.get(i).infoPourHistorique());
  		info.setCaretPosition(0);
  	}
  }
  
  public void mouseClicked(MouseEvent evt) {
    if (evt.getClickCount() == 2) {
      actionButtonOK();
    }
  }

  public void mouseEntered(MouseEvent evt) {
  }

  public void mouseExited(MouseEvent evt) {
  }

  public void mousePressed(MouseEvent evt) {
  }

  public void mouseReleased(MouseEvent evt) {
  } 
}