/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import glob.*;

import java.awt.*;


import types.*;

public class CArcDeCercleDirect extends CArcDeCercle {

  /** Creates new CArcDeCercleDirect */
  public CArcDeCercleDirect() {
  }

  // Constructeur pour clonage
  public CArcDeCercleDirect(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, CPointAncetre OInit, CPointAncetre AInit,
        CPointAncetre pointFinArcInit, CValeurAngle angleAuCentreInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit, OInit, AInit,
      pointFinArcInit, angleAuCentreInit);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(A);
    CPointAncetre ptFinArc = null;
    CValeurAngle angleAuCentreClone = null;
    if (pointFinArc == null)
      angleAuCentreClone = (CValeurAngle) angleAuCentre.getClone(listeSource, listeCible);
    else
    {
      int ind3 = listeSource.indexOf(pointFinArc);
      ptFinArc = (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class));
    }
    int ind4 = listeSource.indexOf(impProto);            
    CElementBase ptelb = new CArcDeCercleDirect(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), ptFinArc, angleAuCentreClone);
    return ptelb;
  }  
  
  // Modifi pour la version 1.9.5 car surArc ne doit plus utiliser de pointeurs
  // sur les objets sources
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleTrait;
    if (horsFenetre) return;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    // Rajout en java par rapport  la version C++
    if (Kernel.testToile(centre_x, centre_y) && Kernel.testToile(origine_x, origine_y)) {   
      pDC.setStroke(Pinceau.CreePinceau(styleTrait));
      pDC.setColor(couleur);
      Kernel.secteurDirect(pDC, origine_x, origine_y, centre_x, centre_y, rayon, valeurAngleAuCentre);
    }
  }

  public double abscisseCurviligne(double ang) {
    double angrot, difang;

    angrot = valeurAngleAuCentre;
    if (angrot == 0) 
    {
      return 0;
    }
    if (angrot < 0) angrot = angrot + 2*Math.PI;
    difang = ang - ang1;
    if (difang < 0) difang = difang + 2*Math.PI;
    return difang/angrot;
  }

  public boolean surArc(double xt, double yt) {
    double ang;

    VecteurDirecteur w= new VecteurDirecteur( centre_x, centre_y, xt, yt);
    ang = w.angleRad();
    if (ang1 <= ang2) {
      return ((ang >= ang1) && (ang <= ang2));
    }
    else  {
      return ((ang <= ang2) || (ang >= ang1));
    }
  }
  // Mthode dfinie pour la version 1.8
  public int natureArc() {
    return NatureArc.NatureArcArcDirect;
  }
  
}
