/*
 * CArcDeCercleImage.java
 *
 * Created on 20 fvrier 2005, 19:13
 */
/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import java.awt.*;
import java.awt.geom.*;

import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CArcDeCercleImage extends CArcDeCercleAncetre {
  // Pour le posisionnement
  private Point2D.Double pointim = new Point2D.Double();
  CPointBase ptCentre;
  CPointBase ptOrigine;
  CPointBase ptExtremite;
  CTransformation transformation;
  CArcDeCercleAncetre antecedent;
  // L'arc de cercle image maintient de faon interne un arc de cercle
  // qui fera tout le travail et dont la nature exate sera dtermine
  //  la cration
  CArcDeCercleAncetre arc;

  /** Creates new CArcDeCercleImage */
  public CArcDeCercleImage() {
    ptCentre = new CPointBase();
    ptOrigine = new CPointBase();
    ptExtremite = new CPointBase();
  }

  public CArcDeCercleImage(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
          Color couleurInit, boolean masqueInit, int styleInit, CArcDeCercleAncetre antecedentInit,
          CTransformation transformationInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    ptCentre = new CPointBase();
    ptOrigine = new CPointBase();
    ptExtremite = new CPointBase();
    antecedent = antecedentInit;
    transformation = transformationInit;
    creeArcInterne();
  // Ce constructeur n'est appel que par getClone. L'arc arc sera cr  l'intrieur de
  // getClone
  }

  /**
   * fonction n'existant pas en C++ et renvoyant un pointeur vers l'antcdent direct de l'objet.
   * Si l'objet est un objet qui ne ncessite pas lors de sa cration la cration
   * d'objets supplmentaires, renvoie this et sinon envoi un pointeur sur le premier
   * de ces objets supplmentaires crs.
   * Par exemple, pour un point li  un bipoint (intersection de deux cercles
   * ou d'un cercle et une droite) renverra un pointeur sur le bipoint
   * cr avant ce CPointLieBipoint. Pour une objet image, renverra un pointeur
   * vers la transformation utilise etc ...
   */
  public CElementBase antecedentDirect() {
    return transformation;
  }
  
  /** Fonction devant tre redfinie pour tous les objets images par une transformation
   *  Sera utilise lorsqu'on modifie une image par une transformation pour que cette image
   *  soit associe  une autre transformation lrsque d'autres objets taient des images par
   *  cette mme transformation
   */
  public void associeATransformation(CTransformation trans) {
  	transformation = trans;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(transformation);
    int ind3 = listeSource.indexOf(impProto);
    CArcDeCercleImage ptelb = new CArcDeCercleImage(listeCible, (CImplementationProto) (listeCible.get(ind3, CImplementationProto.class)),
            estElementFinal, couleur, masque, style, (CArcDeCercleAncetre) (listeCible.get(ind1, CArcDeCercleAncetre.class)),
            (CTransformation) (listeCible.get(ind2, CTransformation.class)));
    // il faut crer l'arc maintenu de faon interne par l'arc que l'on clone et il
    // doit tre de mme nature que celui maintenu par l'objet clon
    switch (arc.natureArc()) {
      case NatureArc.NatureArcPetitArc:
        ptelb.arc = new CArcDeCercle(listeCible, null, false, couleur, masque, style, ptCentre,
                ptOrigine, ptExtremite, null);
        break;
      case NatureArc.NatureArcGrandArc:
        ptelb.arc = new CGrandArcDeCercle(listeCible, null, false, couleur, masque, style,
                ptCentre, ptOrigine, ptExtremite, null);
        break;
      case NatureArc.NatureArcArcDirect:
        ptelb.arc = new CArcDeCercleDirect(listeCible, null, false, couleur, masque, style,
                ptCentre, ptOrigine, ptExtremite, null);
        break;
      case NatureArc.NatureArcArcIndirect:
        ptelb.arc = new CArcDeCercleIndirect(listeCible, null, false, couleur, masque, style,
                ptCentre, ptOrigine, ptExtremite, null);
    }
    return ptelb;
  }

  @Override
  public String infoPourHistorique() {
    return textes.Chaines.chinfo37 + " " + transformation.designationTransf()
            + transformation.complementPourHist();
  }

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(antecedent);
    transformation.AjouteElementsVisuelsPourHist(liste);
  } 
  
  @Override
  public void setClone(CElementBase ptel) {
    arc.setClone(((CArcDeCercleImage) ptel).arc);
  }

  @Override
  public void donneCouleur(Color coul) {
   	super.donneCouleur(coul);
   	arc.donneCouleur(coul);
  }
  
  @Override
  public void donneStyle(int nouveauStyle) {
  	super.donneStyle(nouveauStyle);
  	arc.donneStyle(nouveauStyle);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || antecedent.dependDe(p) || transformation.dependDe(p);
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || antecedent.dependDePourBoucle(p) || transformation.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = transformation.existe && antecedent.existe;
    if (!existe) {
      return;
    }
    transformation.transformePoint(antecedent.centre_x, antecedent.centre_y, pointim);
    double x = pointim.x;
    double y = pointim.y;
    ptCentre.placeEn(x, y);
    transformation.transformePoint(antecedent.origine_x, antecedent.origine_y, pointim);
    ptOrigine.placeEn(pointim.x, pointim.y);
    transformation.transformeAngle(antecedent.valeurAngleAuCentre, pointim);
    double ang = pointim.x;
    VecteurDirecteur u = new VecteurDirecteur(ptCentre, ptOrigine);
    VecteurDirecteur v = new VecteurDirecteur();
    u.tourne(ang, v);
    ptExtremite.placeEn(x + v.x, y + v.y);
    arc.positionne(infoRandom, dimfen);
    // L'arc doit reflter les membres de l'arc interne qu'il maintient
    centre_x = arc.centre_x;
    centre_y = arc.centre_y;
    origine_x = arc.origine_x;
    origine_y = arc.origine_y;
    rayon = arc.rayon;
    // L'arc de cercle image tant dfini par trois points, quand Arc->Positionne() est appel,
    // l'angle est calcul et s'il est plat, l'arc est considr comme direct.
    // mais dans le cas d'un arc image, si l'angle tait pi ou -pi il faut tenir compte du
    // signe de l'angle et donc le rtablir ici.    
    valeurAngleAuCentre = ang;
    arc.valeurAngleAuCentre = ang;
  }
  @Override
  public boolean confonduAvec(CElementBase p) {
    CArcDeCercleImage pt;
    if (p.getClass() == this.getClass()) {
      pt = (CArcDeCercleImage) p;
      return (antecedent == pt.antecedent) && (transformation.confonduAvec(pt.transformation));
    }
    else return false;
  }  

  @Override
  public double abscisseMaximale() {
    return arc.abscisseMaximale();
  }

  @Override
  public double abscisseMinimale() {
    return arc.abscisseMinimale();
  }

  @Override
  public double abscisseCurviligne(double ang) {
    return arc.abscisseCurviligne(ang);
  }

  @Override
  public boolean surArc(double xt, double yt) {
    return arc.surArc(xt, yt);
  }

  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    return arc.distancePoint(xp, yp, masquage);
  }
  
  @Override
  public double distancePointPourSurface(double xp, double yp, boolean masquage) {
    return arc.distancePointPourSurface(xp, yp, masquage);
  }
  

  @Override
  public int natureArc() {
    return arc.natureArc();
  }

  @Override
  public final void cache() {
    super.cache();
    arc.cache();
  }

  @Override
  public final void montre() {
    super.montre();
    arc.montre();
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    if (!masque) {
      arc.trace(pDC, couleurFond, transparence);
    }
  }

  private void creeArcInterne() {
    if (transformation.getClass() == CSymetrieAxiale.class) {
      switch (antecedent.natureArc()) {
        case NatureArc.NatureArcPetitArc:
          arc = new CArcDeCercle(listeProprietaire, null, false,
                  couleur, masque, style, ptCentre, ptOrigine, ptExtremite, null);
          break;
        case NatureArc.NatureArcGrandArc:
          arc = new CGrandArcDeCercle(listeProprietaire, null, false,
                  couleur, masque, style, ptCentre, ptOrigine, ptExtremite, null);
          break;
        case NatureArc.NatureArcArcDirect:
          arc = new CArcDeCercleIndirect(listeProprietaire, null, false, couleur,
                  masque, style, ptCentre, ptOrigine, ptExtremite, null);
          break;
        case NatureArc.NatureArcArcIndirect:
          arc = new CArcDeCercleDirect(listeProprietaire, null, false,
                  couleur, masque, style, ptCentre, ptOrigine, ptExtremite, null);
      }
    } else // Conservation des angles orients
    {
      switch (antecedent.natureArc()) {
        case NatureArc.NatureArcPetitArc:
          arc = new CArcDeCercle(listeProprietaire, null, false, couleur,
                  masque, style, ptCentre, ptOrigine, ptExtremite, null);
          break;
        case NatureArc.NatureArcGrandArc:
          arc = new CGrandArcDeCercle(listeProprietaire, null, false, couleur,
                  masque, style, ptCentre, ptOrigine, ptExtremite, null);
          break;
        case NatureArc.NatureArcArcDirect:
          arc = new CArcDeCercleDirect(listeProprietaire, null, false, couleur,
                  masque, style, ptCentre, ptOrigine, ptExtremite, null);
          break;
        case NatureArc.NatureArcArcIndirect:
          arc = new CArcDeCercleIndirect(listeProprietaire, null, false, couleur,
                  masque, style, ptCentre, ptOrigine, ptExtremite, null);
      }
    }    
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    ptCentre = new CPointBase();
    ptOrigine = new CPointBase();
    ptExtremite = new CPointBase();
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    transformation = (CTransformation) (list.get(ind1, CTransformation.class));
    antecedent = (CArcDeCercleAncetre) (list.get(ind2, CArcDeCercleAncetre.class));
    creeArcInterne();
  }
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    ptCentre = new CPointBase();
    ptOrigine = new CPointBase();
    ptExtremite = new CPointBase();
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    transformation = (CTransformation) (list.get(ind1, CTransformation.class));
    antecedent = (CArcDeCercleAncetre) (list.get(ind2, CArcDeCercleAncetre.class));
    creeArcInterne();
  }  
  /* Enregistrement dans un flux java */
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(transformation);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(antecedent);
    oups.writeInt(ind2);
  }
    
  @Override
  public boolean contientParDefinition(CPointAncetre po) {
  	if (po.estPointImage()) {
  		CPointImage ptim = (CPointImage) po;
  		return (transformation == ptim.transformation) && (antecedent.contientParDefinition(ptim.antecedent));
  	}
  	else return true;
  }
  
}
