/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;


import glob.*;

import java.awt.*;


import types.*;

// Enumerationjaja, l'arc de cercle indirect a t remain par rapport au C++
public class CArcDeCercleIndirect extends CArcDeCercle {

  /** Creates new CArcDeCercleIndirect */
  public CArcDeCercleIndirect() {
  }
  
  // Constructeur pour clonage
  public CArcDeCercleIndirect(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
        Color couleurInit, boolean masqueInit, int styleInit,
        CPointAncetre OInit, CPointAncetre AInit,
        CPointAncetre pointFinArcInit, CValeurAngle angleAuCentreInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit,
      OInit, AInit, pointFinArcInit, angleAuCentreInit);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(A);
    CPointAncetre ptFinArc = null;
    CValeurAngle angleAuCentreClone = null;
    if (pointFinArc == null)
      angleAuCentreClone = (CValeurAngle) angleAuCentre.getClone(listeSource, listeCible);
    else
    {
      int ind3 = listeSource.indexOf(pointFinArc);
      ptFinArc = (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class));
    }
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CArcDeCercleIndirect(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style,
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), ptFinArc, angleAuCentreClone);
    return ptelb;
  }  
  
  // Modifi pour la version 1.9.5 car surArc ne doit plus utiliser de pointeurs
  // sur les objets sources
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    int styleTrait;
    if (horsFenetre) return;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    // Rajout en java par rapport  la version C++
    if (Kernel.testToile(centre_x, centre_y) && Kernel.testToile(origine_x, origine_y)) {   
      pDC.setStroke(Pinceau.CreePinceau(styleTrait));
      pDC.setColor(couleur);
      VecteurDirecteur u = new VecteurDirecteur(centre_x, centre_y, origine_x, origine_y);
      VecteurDirecteur v = new VecteurDirecteur();
      u.tourne(valeurAngleAuCentre, v);
      double x1 = centre_x + v.x;
      double y1 = centre_y + v.y;
  		// double difang = ang1 - ang2;
      Kernel.secteurDirect(pDC, x1, y1, centre_x, centre_y, rayon, - valeurAngleAuCentre);
    }
  }
  
  public double abscisseCurviligne(double ang) {
    double angrot,difang;

    angrot = valeurAngleAuCentre;
    if (angrot == 0)
    {
      return 0;
    }
    angrot = - angrot;
    if (angrot < 0) angrot = angrot + 2*Math.PI;
    difang = ang - ang2;
    if (difang < 0) difang = difang + 2*Math.PI;
    return difang/angrot;
  }

  // POur cette procdure, on permute les roles de ang1 et ang2 par rapport  l'arc direct
  public boolean surArc(double xt, double yt) {
    double ang;

    VecteurDirecteur w= new VecteurDirecteur(centre_x, centre_y, xt, yt);
    ang = w.angleRad();
    if (ang2 <= ang1) {
      return ((ang >= ang2) && (ang <= ang1));
    }
    else  {
      return ((ang <= ang1) || (ang >= ang2));
    }
  } 
  // Mthode dfinie pour la version 1.8
  public int natureArc() {
    return NatureArc.NatureArcArcIndirect;
  }
  
}
