/*
 * CCalcul.java
 *
 * Created on 1 juillet 2004, 18:56
 */

/**
 *
 * @author  Yves
 * @version
 */
package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CCalcul extends CCalculAncetre {
  public String chaineCalcul;
  public CCalculBase calcul;
  public double resultat;
  
  /** Creates new CCalcul */
  public CCalcul() {
  }
  
  /* Constructeur pour outil de cration */
  public CCalcul(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit) {
    super(liste, impProtoInit, estElementFinalInit);
  }
  
  // Constructeur pour clonage
  public CCalcul(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomCalculInit, String chaineCalculInit, CCalculBase calculInit) {
    super(liste, impProtoInit, estElementFinalInit, nomCalculInit);
    chaineCalcul = new String(chaineCalculInit);
    calcul = calculInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    int ind1 = listeSource.indexOf(impProto);
    return new CCalcul(listeCible, (CImplementationProto)(listeCible.get(ind1, CImplementationProto.class)),
      estElementFinal, nomCalcul, chaineCalcul, calculClone);
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NCalculReel;
  }
  
  /***
   * Fonction reconstruisant la chaine reprsentant le calcul (par exemple lorsqu'un autre calcul a t renomm
   * A redfinir pour les fonctions
   */
  public void reconstruitChaineCalcul() {
  	chaineCalcul = calcul.chaineCalculSansPar("");	
  }
  
  @Override
  public String infoPourHistorique() {
    String ch = textes.Chaines.ch17 + " " + nom() + "\n" + nom() + " = " + chaineCalcul;
    if (existe) ch = ch + "\n" + textes.Chaines.chinfo97 + " " + glob.Kernel.chaineNombre(resultat, 16);
    else ch = ch + "\n" + textes.Chaines.ch18;
    return ch;
  }
  
  double rendValeur(boolean infoRandom) {
    return resultat;
  }
  //  revoir pour la version java
  double rendValeur(boolean infoRandom, double parametre) throws ErreurCalculException {
    return rendValeur(infoRandom);
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = calcul.existe();
    if (!existe) return;
    // A revoir en java pour la gestion des erreurs de calcul
    // AnnuleErreurDeCalcul();	// Pour dtecter une ventuelle erreur de
    //                         // calcul lors de l'appel
    try {
      resultat = calcul.resultat(infoRandom);
    } catch (ErreurCalculException e) {
      existe = false;
      resultat = 0;
    }
    if (Double.isNaN(resultat) || Double.isInfinite(resultat)) {
      existe = false;
    }
    // }
  }
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || calcul.dependDe(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || calcul.dependDePourBoucle(p);
  }
  
  // Rajout pour la version 1.8 : nom() qui rend le nom du calcul.
  // Ne sera redfini que pour les calculs et les variables
  public String nom() {
    return nomCalcul;
  }

   // Inutile de redfinir informationModification() pour les objets de type calcul
  
  /**
   * cration de l'arbre binaire reprsentant la chane de calcul
   */
  public void creeCalcul() {
    calcul = CCalculBase.creeCalculBase(chaineCalcul, listeProprietaire, 0,
            chaineCalcul.length()-1, null);  
  }
  
  
  /**
   * Ajout pour la version 3.0 : Renvoie true seulement si le calcul associ
   * est constant et ne fait pas appel  la fonction Rand()
   * @return
   */
  public boolean estConstant() {
  	return calcul.estConstant();
  }
 
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps,list);
    chaineCalcul = inps.readCString();
    calcul = (CCalculBase) (inps.readCObject(list));
  }

  // Version 1.9.5 : Suite  l'introduction de CDerivee
  // il a fallu dfinir cette produre car CDerivee n'enregistre
  // pas le calcul dans le flux.
  public void chargeDepuisFluxSansCalcul(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps,list);
  }

  /* Lecture depuis un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    chaineCalcul = inps.readUTF();
    calcul = (CCalculBase) (inps.readObject(list));
  }
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeUTF(chaineCalcul);
    oups.writeObject(calcul);
  }
  /* Lecture depuis un flux java pour lecture de drive */  
  public void readSansCalcul(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
  }  
  /* Enregistrement dans un flux java */
  public void writeSansCalcul(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
  }
}
