/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;

import java.awt.Color;
import java.awt.Graphics2D;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.Kernel;

public class CCercleOA extends CCercle {
  CPointAncetre O;
  CPointAncetre A;
  /** Creates new CCercleOA */
  public CCercleOA() {
  }
  
  // Constructeur pour clonage
  public CCercleOA(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit, int styleInit,
      CPointAncetre OInit, CPointAncetre AInit) {
      super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit); 
    O = OInit;
    A = AInit;
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {

    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    if ((Kernel.testToile(xCarre1,yCarre1)) && (Kernel.testToile(xCarre2,yCarre2)))
    {
      pDC.setStroke(Pinceau.CreePinceau(styleTrait));
      pDC.setColor(couleur);
      pDC.drawOval((int)Math.round(xCarre1),(int)Math.round(yCarre1),
          (int)Math.round(xCarre2 - xCarre1),(int)Math.round(yCarre2 - yCarre1));
    }
  }   

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(O);
    int ind2 = listeSource.indexOf(A);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CCercleOA(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
    return ptelb;
  }  

  @Override
  public String infoPourHistorique() {
    return getString("chinfo11");
  }

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(O);
    liste.add(A); // Ajout version 4.7.6
  } 
  
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
  	return memoriseDependance(super.dependDe(p) || O.dependDe(p) || A.dependDe(p));
  }

  public boolean dependDePourBoucle(CElementBase p) {
  	return ((p == this) || O.dependDePourBoucle(p) || A.dependDePourBoucle(p));
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
		existe = (O.existe) && (A.existe);
		if (!existe) return;
    VecteurDirecteur u = new VecteurDirecteur();
		u.x = A.x - O.x;
		u.y = A.y - O.y;
		rayon = u.norme();
		centre_x = O.x;
		centre_y = O.y;
		super.positionne(infoRandom, dimfen);
  }
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CCercleOA pt;
    if (p.getClass() == this.getClass()) {
      pt = (CCercleOA) p;
      return ((O == pt.O) && (A == pt.A));
    }
    else return false;
  }
  
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (O == ancienPoint) O = nouveauPoint;
    if (A == ancienPoint) A = nouveauPoint;
  }

  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  /* Enregistrement dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    O = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    A = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(O);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(A);
    oups.writeInt(ind2);
  }   
  /**
   * Fonction rajoute pour la version 2.1 et servant  savoir si un point
   * est par dfinition sur une droite. Renverra true par exemple
   * si la droite est dfinie par deux points et le point est un de ces deux points
   * ou si le point est li  la droite
   * A t rajout pour amliorer l'outil d'intersection
   * A redfinir pour les descendants quand ncessaire
   * */
  /*
   * public boolean contientParDefinition(CPointAncetre po)
  {
    return (A == po) || po.appartientParDefinition(this);
  }
  */
  
  @Override
  public boolean contientParDefinition(CPointAncetre po) {
  	if (A == po) return true;
  	if (po.estPointImage()) {
  		CPointImage ptim = (CPointImage) po;
  		if (ptim.transformation.natureTransformation() == CTransformation.rotation) {
  			CRotation rot = (CRotation) ptim.transformation;
  			return ptim.antecedent.appartientParDefinition(this) && (rot.centre == O);
  		}
  		else return false;
  	}
  	else return false;
  }  
}
