/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import types.NatureCalcul;
import types.NatureObjet;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

/**
 * 
 * @author Yves
 */
public class CImplementationProto extends CElementBase {
	public String nomProto; // Le nom du prototype qui l'a gnr
	public int nbIntermediaires;
	public int nbFinaux;
	public CSousListeObjets listeSources;

	/** Creates a new instance of CImplementationProto */
	public CImplementationProto() {
		listeSources = new CSousListeObjets();
	}

	/**
	 * Constructeur pour clonage
	 */
	public CImplementationProto(CListeObjets liste, String nomProtoInit, int nbIntermediairesInit,
			int nbFinauxInit, CSousListeObjets listeSourcesInit) {
		super(liste, null, false);
		nomProto = nomProtoInit;
		nbIntermediaires = nbIntermediairesInit;
		nbFinaux = nbFinauxInit;
		listeSources = listeSourcesInit;
	}

	/**
	 * Constructeur pour implmentation  partir d'un protoype Avanat appel les
	 * lments gnriques du prototype ont t mis  pointer sur les objets
	 * sources dans liste
	 * 
	 * @param liste
	 * @param nomProtoInit
	 * @param nbIntermediairesInit
	 * @param nbFinauxInit
	 */
	public CImplementationProto(CListeObjets liste, CPrototype proto) {
		super(liste, null, false);
		nomProto = proto.nom;
		listeSources = new CSousListeObjets();
		int nbSources = proto.nbObjSources;
		for (int i = 0; i < nbSources; i++) {
			listeSources.add(((CElementGenerique) proto.get(i)).elementAssocie);
		}
		nbFinaux = proto.nbObjFinaux;
		nbIntermediaires = proto.size() - nbSources - nbFinaux;
	}

  @Override
	public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
		CSousListeObjets listeSourcesClone = new CSousListeObjets();
		listeSourcesClone.setImage(listeSources, listeSource, listeCible);
		CElementBase ptelb = new CImplementationProto(listeCible, nomProto, nbIntermediaires, nbFinaux,
				listeSourcesClone);
		return ptelb;
	}

	public final CElementBase dernierObjetFinal() {
		int ind = listeProprietaire.indexOf(this);
		return listeProprietaire.get(ind + nbIntermediaires + nbFinaux);
	}

	@Override
	public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
		return memoriseDependance(super.dependDe(p) || listeSources.dependDe(p));
	}


	@Override
	public long getNature() {
		return NatureObjet.NImplementationProto;
	}

	public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
		listeSources.remplacePoint(ancienPoint, nouveauPoint);
	}



	/**
	 * Fonction renvoyant le nombre d'objets finaux de type calcul ayant un nom
	 * Est utilis lors de la fin de l'implmentation d'une construction pour
	 * proposer de renommer ces calculs
	 * 
	 * @return
	 */
	public int nombreObjetsFinauxTypeCalculNommes() {
		int res = 0;
		int ind = listeProprietaire.indexOf(this);
		for (int i = ind + 1; i <= ind + nbFinaux + nbIntermediaires; i++) {
			CElementBase elb = listeProprietaire.get(i);
			if (elb.estElementFinal && ((elb.getNatureCalcul() & NatureCalcul.NToutCalculNommeSaufConst) != 0))
				res++;
		}
		return res;
	}

	/**
	 * Fonction rajout version 3.6.1 pour corriger des bugs lors du reclassement d'objets
	 * @return
	 */
	public void  determineNombreObjetsIntermediaires() {
		int indimp = listeProprietaire.indexOf(this);
		int nb = 0;
		for (int i = indimp+1; i < listeProprietaire.size(); i++) {
			CElementBase elb = listeProprietaire.get(i);
			if ((elb.impProto == this) && !elb.estElementFinal) nb++;
			else break;
		}
		nbIntermediaires = nb;
	}	
	
	// chargeDepuisFlux a t ajout  la version 2.5 pour cause d'implmentation
	// de prototype
	public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.chargeDepuisFlux(inps, list);
		nomProto = inps.readCString();
		nbIntermediaires = inps.readCInt();
		nbFinaux = inps.readCInt();
		listeSources.chargeDepuisFlux(inps, list);
	}

	/* Enregistrement dans un flux java */
	public void read(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
		super.read(inps, list);
		nomProto = inps.readUTF();
		nbIntermediaires = inps.readInt();
		nbFinaux = inps.readInt();
		listeSources.read(inps, list);
	}

	/* Enregistrement dans un flux java */
	public void write(MathGraphDataOutputStream oups, CListeObjets list) throws IOException {
		super.write(oups, list);
		oups.writeUTF(nomProto);
		oups.writeInt(nbIntermediaires);
		oups.writeInt(nbFinaux);
		listeSources.write(oups, listeProprietaire);
	}
}
