/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.*;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CMarqueAngleOriente extends CMarqueAngleGeometrique {
	double xdeb,ydeb;  // les coordones du point dbut de l'arc
	VecteurDirecteur v0; // Vecteur de longueur rayon colinaire  OB
	VecteurDirecteur vecteurDirection; // vecteur dfinissant la direction
														// inverse de l'arc
	public int motifFleche;

  /** Creates new CMarqueAngleOriente */
  public CMarqueAngleOriente() {
  	v0 = new VecteurDirecteur();
    vecteurDirection = new VecteurDirecteur();
  }
  
  // Constructeur pour clonage
  public CMarqueAngleOriente(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleInit, int styleMarqueInit, double rayonInit, int motifFlecheInit,
    	CPointAncetre AInit, CPointAncetre OInit, CPointAncetre BInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit,styleMarqueInit, rayonInit,
    		 AInit, OInit, BInit);
    motifFleche = motifFlecheInit;
  	v0 = new VecteurDirecteur();
    vecteurDirection = new VecteurDirecteur();
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(A);
    int ind2 = listeSource.indexOf(O);
    int ind3 = listeSource.indexOf(B);
    int ind4 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CMarqueAngleOriente(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, styleMarque, rayon, motifFleche, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
    (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;
  }

  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    ((CMarqueAngleOriente)ptel).vecteurDirection.setCopy(vecteurDirection);
    xdeb = ((CMarqueAngleOriente)ptel).xdeb;
    ydeb = ((CMarqueAngleOriente)ptel).ydeb;
  }
  
  /**
   * Appel pour les marques d'angle de sens direct et indirect
   * @param infoRandom
   * @param dimfen
   */
  public void positionneAncetre(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  }
  
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	double petitAngle; // Reprsente l'angle de correction pour la direction de la flche
  	// due  la longueur de la flche
    
    super.positionne(infoRandom, dimfen);
    if (!existe) return;
    // Optimisation pour la version 1.9.5
    double longueurFleche =StyleFleche.longeurFlechePourArc(motifFleche, listeProprietaire.basseDefinition);
    v.vecteurColineaire(rayon,v0);
    double angle = u.mesureAngleVecteurs(v);
    if (Kernel.zeroAngle(Math.abs(angle) - Math.PI/2)) petitAngle = 0;
    else petitAngle = longueurFleche/rayon;

    if (Kernel.zero(angle))
    {
      existe = false;
      return;
    }
    else
    {
      if (Kernel.zeroAngle(Math.abs(angle)-Math.PI)) v0.tourne(-Math.PI/2,vecteurDirection);
      else
      {
        if (angle <= 0) v0.tourne(Math.PI/2+petitAngle, vecteurDirection);
        else v0.tourne(-Math.PI/2-petitAngle, vecteurDirection);
      }
    }
    xdeb = O.x + v0.x;
    ydeb = O.y + v0.y;
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {
    boolean basseDefinition = listeProprietaire.basseDefinition;
    if (A.horsEcran || O.horsEcran || B.horsEcran) return;
    super.trace(pDC, couleurFond, transparence);
    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    pDC.setStroke(Pinceau.CreePinceau(styleTrait));
    pDC.setColor(couleur);
    StyleFleche.traceFlechePourArc(pDC,xdeb,ydeb,vecteurDirection,motifFleche, basseDefinition);
  }
 
  @Override
  public boolean estOriente() {
  	return true;
  }
  
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    motifFleche = inps.readByte();
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    motifFleche = inps.readByte();
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(motifFleche);
  }
  
  public void donneStyleFleche(int styleFleche) {
  	motifFleche = styleFleche;
  }
  
  
}
