/*
 * CMediatrice.java
 *
 * Created on 4 juillet 2004, 17:16
 */

/**
 *
 * @author  Yves
 * @version
 */
package objets;

import java.io.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CMediatrice extends CDroite {
  CPointAncetre point1;
  CPointAncetre point2;
  
  /** Creates new CMediatrice */
  public CMediatrice() {
    super(); // Pour que le vecteur soit cr
  }
  
  // Constructeur pour clonage
  public CMediatrice(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
    boolean masqueInit, String nomInit, int indiceTailleNomInit,
    int styleInit, double abscisseNomInit, CPointAncetre point1Init, CPointAncetre point2Init) {
      super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit,
        masqueInit, nomInit, indiceTailleNomInit, styleInit, abscisseNomInit);
      point1 = point1Init;
      point2 = point2Init;
  }
  
  /** Constructeur plus court pour objets visuels */
  public CMediatrice(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, int styleInit, CPointAncetre point1Init, CPointAncetre point2Init, int indiceTailleNomInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, false, styleInit, indiceTailleNomInit);
    point1 = point1Init;
    point2 = point2Init;
  }
  
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(point1);
    int ind2 = listeSource.indexOf(point2);
    int ind3 = listeSource.indexOf(impProto);
    return new CMediatrice(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y,
      masque, nom, indiceTailleNom, style, abscisseNom,
      (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),(CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)));
  }

  @Override
  public String infoPourHistorique() {
    return textes.Chaines.chinfo10;
  }
  
  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(point1);
    liste.add(point2);
  } 
  
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || point1.dependDe(p) || point2.dependDe(p);
  }
 
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || point1.dependDePourBoucle(p) || point2.dependDePourBoucle(p));
  }
  
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    VecteurDirecteur v = new VecteurDirecteur();
    existe = (point1.existe) && (point2.existe);
    if (!(existe)) return;
    point_x = (point1.x + point2.x)/2;
    point_y = (point1.y + point2.y)/2;
    v.setVecteur(point1, point2);
    if ((v.x == 0) && (v.y == 0)) existe = false;
    else {
      v.getOrthogonal(vecteur);
      super.positionne(infoRandom, dimfen);
    }
  }
  
   
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
   **/
  public boolean confonduAvec(CElementBase p) {
    CMediatrice pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMediatrice) p;
      return (((point1 == pt.point1) && (point2 == pt.point2))
      ||
        ((point2 == pt.point1) && (point1 == pt.point2)));
    } else return false;
  }
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (point1 == ancienPoint) point1 = nouveauPoint;
    if (point2 == ancienPoint) point2 = nouveauPoint;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    point1 = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    point2 = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(point1);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(point2);
    oups.writeInt(ind2);
  }
  
  @Override
  public boolean estPerpendiculaireParDefinition(CDroiteAncetre dte) {
  	return listeProprietaire.sontParDefinitionSurParallele(point1, point2, dte);  
  }
  
}
