/*
 * CMesureAbscisse.java
 *
 * Created on 7 juillet 2004, 21:44
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;

public class CMesureAbscisse extends CValeurDynamique {
	double abscisse;
	CPointAncetre origine;
	CPointAncetre extremite;
	CPointAncetre pointMesure;

  /** Creates new CMesureAbscisse */
  public CMesureAbscisse() {
  }
  
  // Constructeur pour clonage
  public CMesureAbscisse(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      CPointAncetre origineInit, CPointAncetre extremiteInit, CPointAncetre pointMesureInit) {
    super(liste, impProtoInit, estElementFinalInit);
    origine = origineInit;
    extremite = extremiteInit;
    pointMesure = pointMesureInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(origine);
    int ind2 = listeSource.indexOf(extremite);   
    int ind3 = listeSource.indexOf(pointMesure);   
    int ind4 = listeSource.indexOf(impProto);    
    CElementBase ptelb = new CMesureAbscisse(listeCible, (CImplementationProto)(listeCible.get(ind4, CImplementationProto.class)),
      estElementFinal, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
      (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)), (CPointAncetre)(listeCible.get(ind3, CPointAncetre.class)));
    return ptelb;    
  }

  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NMesureAbscisseSurDroite;
  }
  
  @Override
  public String infoPourHistorique() {
    String ch =  textes.Chaines.chinfo31 + " " + pointMesure.rendNom() + "\n" + textes.Chaines.chinfo32 
            + " (" + origine.rendNom() + "," + extremite.rendNom() + ")\n"
            + textes.Chaines.chinfo33 + " " + nom();
    if (existe) ch = ch + "\n" + textes.Chaines.chinfo97 + Kernel.chaineNombre(abscisse, 16);
    else ch = ch + "\n" + textes.Chaines.ch18;
    return ch;
  }
  
  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(origine);
    liste.add(extremite);
    liste.add(pointMesure);
  } 
  
  
  @Override
  public void setClone(CElementBase ptel) {
    abscisse = ((CMesureAbscisse)ptel).abscisse;
  }  
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || origine.dependDe(p) || extremite.dependDe(p) || pointMesure.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || origine.dependDePourBoucle(p) || extremite.dependDePourBoucle(p)
      || pointMesure.dependDePourBoucle(p);
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = origine.existe && extremite.existe
			&& pointMesure.existe && !(origine.egal(extremite));
    existe = existe && Kernel.aligne(origine.x,origine.y,extremite.x,
			extremite.y,pointMesure.x,pointMesure.y);
    if (!existe) return;
    // L'orientation sur l'cran est l'inverse de celle du repre
		// choisi.Il faut donc permuter les rles de u et v}
    VecteurDirecteur v = new VecteurDirecteur(origine,extremite);
    VecteurDirecteur u = new VecteurDirecteur(origine,pointMesure);
		// Nouvelle modif
    if (Math.abs(v.x)>Math.abs(v.y)) abscisse = u.x/v.x;
    else abscisse = u.y/v.y;
  }

  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return ((el == origine) || (el == extremite) || (el == pointMesure));
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return abscisse;
  }

  @Override
  public String nom() {
    return pointMesure.nom + "(" + origine.nom + "," + extremite.nom + ")";
  }
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  public boolean confonduAvec(CElementBase p) {
    CMesureAbscisse pt;
    if (p.getClass() == this.getClass()) {
      pt = (CMesureAbscisse) p;
      return ((origine == pt.origine)
			&& (extremite == pt.extremite
			&& (pointMesure == pt.pointMesure)));
    }
    else return false;
  }    

  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (origine == ancienPoint) origine = nouveauPoint;
    if (extremite == ancienPoint) extremite = nouveauPoint;
    if (pointMesure == ancienPoint) pointMesure = nouveauPoint;
  }
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
     super.chargeDepuisFlux(inps, list);
     int ind1 = inps.readCInt();
     int ind2 = inps.readCInt();
     int ind3 = inps.readCInt();
     origine = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     extremite = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
     pointMesure = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
     int ind1 = inps.readInt();
     int ind2 = inps.readInt();
     int ind3 = inps.readInt();
     origine = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
     extremite = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
     pointMesure = (CPointAncetre) (list.get(ind3,CPointAncetre.class));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(origine);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(extremite);
    oups.writeInt(ind2);
    int ind3 = list.indexOf(pointMesure);
    oups.writeInt(ind3);
  }  
}
