/*
 * COperation.java
 *
 * Created on 1 juillet 2004, 20:31
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class COperation extends CCalculBase {
	public static final String parouv ="(";
	public static final String parfer = ")";
	public static final String zero = "0";
	public static final String zeroComp = "0+0*i";
	
	
  CCalculBase operande1;
  CCalculBase operande2;
  int operateur;
  // Attributs pour optimisation
  Complexe c1 = new Complexe();
  Complexe c2 = new Complexe();

  /** Creates new COperation */
  public COperation() {
  }
  public COperation(CListeObjets liste, CCalculBase operande1Init, 
      CCalculBase operande2Init, int operateurInit) {
    super(liste);
    operande1 = operande1Init;
    operande2 = operande2Init;
    operateur = operateurInit;
  }
  
  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande1 = operande1.getClone(listeSource, listeCible); 
    CCalculBase cloneOperande2 = operande2.getClone(listeSource, listeCible); 
    CCalculBase ptcalc = new COperation(listeCible, cloneOperande1, cloneOperande2, operateur);
    return ptcalc;
  }     
  
  public boolean dependDe(CElementBase p) {
  	return operande1.dependDe(p) || operande2.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
  	return operande1.dependDePourBoucle(p) || operande2.dependDePourBoucle(p);
  }

  boolean existe() {
  	return operande1.existe() && operande2.existe();
  }

  private double resultatBase(boolean infoRandom, double op1, double op2) throws ErreurCalculException {
    double denom;
    switch (operateur)
    {
      case Operateur.Plus:
        return op1 + op2;
      case Operateur.Moin:
        return op1 - op2;
      case Operateur.Mult: 
        return op1 * op2;
      //  revoir pour gestion des exceptions
      case Operateur.Divi:
         denom = op2;
         if (denom == 0)
         {
           throw new ErreurCalculException();
         }
         else return op1 / denom;
      case Operateur.Inf:
        if (op1 < op2)
                return 1;
        else return 0;
      case Operateur.Sup:
        if (op1 > op2)
                return 1;
        else return 0;
      case Operateur.InfOuEgal:
        if (op1 <= op2)
                return 1;
        else return 0;
      case Operateur.SupOuEgal:
        if (op1 >= op2)
                return 1;
        else return 0;
      case Operateur.Egalite:
        if (op1 == op2)
                return 1;
        else return 0;
      case Operateur.Diff:
        if (op1 != op2)
                return 1;
        else return 0;
      default: return 0;
    } // switch Operateur
  }
  public double resultat(boolean infoRandom) throws ErreurCalculException {
     double o1 = operande1.resultat(infoRandom);
     double o2 = operande2.resultat(infoRandom);
     return resultatBase(infoRandom, o1, o2);
  }

  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
      double o1 = operande1.resultatFonction(infoRandom, valeurParametre);
     double o2 = operande2.resultatFonction(infoRandom, valeurParametre);
     return resultatBase(infoRandom, o1, o2);

  }
  // La fonction suivnte est utilise par les deux fonctions suivantes
  private void resultatComplexeBase(boolean infoRandom, Complexe op1, Complexe op2, Complexe zRes)
                              throws ErreurCalculException {
    switch (operateur)
    {
      case Operateur.Plus:
              zRes.x = op1.x + op2.x;
              zRes.y = op1.y + op2.y;
              return;
       case Operateur.Moin:
              zRes.x = op1.x - op2.x;
              zRes.y = op1.y - op2.y;
              return;
      case Operateur.Mult: 
              zRes.x = op1.x*op2.x - op1.y*op2.y;
              zRes.y = op1.x*op2.y + op1.y*op2.x;
              return;
      case Operateur.Divi:
        {
          double mod2 = op2.x*op2.x + op2.y*op2.y;
          if (mod2 == 0)
          {
            throw new ErreurCalculException();
          }
          else
          {
            zRes.x = (op1.x*op2.x + op1.y*op2.y)/mod2;
            zRes.y = (op1.y*op2.x - op1.x*op2.y)/mod2;
            return;
          }
        }
      case Operateur.Inf:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            double mod1 = op1.x*op1.x + op1.y*op1.y;
            double mod2 = op2.x*op2.x + op2.y*op2.y;
            if (mod1 < mod2) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.Sup:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            if (op1.x > op2.x) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.InfOuEgal:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            if (op1.x <= op2.x) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.SupOuEgal:
        // Si on est dans C, on n'accepte de comparer deux complexes
        // que s'ils sont rels
        {
          zRes.x = 0;
          zRes.y = 0;
          if ((op1.y != 0) || (op2.y != 0))
          {
            throw new ErreurCalculException();
          }
          else
          {
            if (op1.x >= op2.x) zRes.x = 1;
            else zRes.x = 0;
            return;
          }
        }
      case Operateur.Egalite:
        zRes.y = 0;
        if ((op1.x == op2.x) && (op1.y == op2.y))
                zRes.x = 1;
        else zRes.x = 0;
        return;
      case Operateur.Diff:
        zRes.y = 0;
        if ((op1.x != op2.x) || (op1.y != op2.y))
                zRes.x = 1;
        else zRes.x = 0;
        return;
    } // switch Operateur   
  }

  // Ajout pour la version 2.0 calculs sur les complexes.
  // Le rsultat est renvoy dans z
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
    operande1.resultatComplexe(infoRandom, c1);
    operande2.resultatComplexe(infoRandom, c2);
    resultatComplexeBase(infoRandom, c1, c2, zRes);
  }
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
  throws ErreurCalculException
  {
    operande1.resultatFonctionComplexe(infoRandom, valeurParametre, c1);
    operande2.resultatFonctionComplexe(infoRandom, valeurParametre, c2);
    resultatComplexeBase(infoRandom, c1, c2, zRes);
  }

  public boolean  dependDeVariable() {
    return operande1.dependDeVariable() || operande2.dependDeVariable();
  }    
  public CCalculBase getCopie() {
   	return new COperation(listeProprietaire, operande1.getCopie(), operande2.getCopie(),
      operateur);
  }
  
  @Override
	public String chaineCalcul(String varFor) {
  	String ch1 ="", ch2 = "", ch = "";
  	switch (operateur)
  	{
  		case Operateur.Plus:
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Plus) || (op2 == Operateur.Moin) || (op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "+" + ch2 + parfer;
  			break;
  		case Operateur.Moin:
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "-" + ch2 + parfer;
  			break;
  		case Operateur.Mult: 
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "*" + ch2 + parfer;
  			break;
  		case Operateur.Divi:
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			ch2 = operande2.chaineCalcul(varFor); 
  			ch = parouv + ch1 + "/" + ch2 + parfer;
  			break;
  		case Operateur.Inf:
  			ch = parouv + operande1.chaineCalcul(varFor) + "<" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.Sup:
  			ch = parouv + operande1.chaineCalcul(varFor) + ">" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.InfOuEgal:
  			ch = parouv + operande1.chaineCalcul(varFor) + "<=" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.SupOuEgal:
  			ch = parouv + operande1.chaineCalcul(varFor) + ">=" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.Egalite:
  			ch = parouv + operande1.chaineCalcul(varFor) + "=" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.Diff:
  			ch = parouv + operande1.chaineCalcul(varFor) + "<>" + operande2.chaineCalcul(varFor) + parfer;
  	} // switch Operateur}
  	// Au cas o le calcul commenait par un moins unaire, la chaine commence par "0-"
  	if (ch.substring(0, 3).compareTo("(0-") == 0) ch = parouv + "-" + ch.substring(3);
  	return ch;
  }
  
  @Override
	public String chaineCalculSansPar(String varFor) {
  	String ch1 = "", ch2 = "", ch = "";
  	switch (operateur) {
  		case Operateur.Plus:
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Plus) || (op2 == Operateur.Moin) || (op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "+" + ch2;
  			break;
  		case Operateur.Moin:
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Plus) || (op1 == Operateur.Moin) || (op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "-" + ch2;
  			break;
  		case Operateur.Mult: 
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			if (operande2.getClass() == COperation.class) {
  				int op2 = ((COperation) operande2).operateur;
  				if ((op2 == Operateur.Mult) || (op2 == Operateur.Divi))
  					ch2 = operande2.chaineCalculSansPar(varFor);
  				else ch2 = operande2.chaineCalcul(varFor); 
  			}
  			else ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "*" + ch2;
  			break;
  		case Operateur.Divi:
  			if (operande1.getClass() == COperation.class) {
  				int op1 = ((COperation) operande1).operateur;
  				if ((op1 == Operateur.Mult) || (op1 == Operateur.Divi))
  					ch1 = operande1.chaineCalculSansPar(varFor);
  				else ch1 = operande1.chaineCalcul(varFor); 
  			}
  			else ch1 = operande1.chaineCalcul(varFor); 
  			ch2 = operande2.chaineCalcul(varFor); 
  			ch = ch1 + "/" + ch2;
  			break;
  		case Operateur.Inf:
  			ch = parouv + operande1.chaineCalcul(varFor) + "<" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.Sup:
  			ch = parouv + operande1.chaineCalcul(varFor) + ">" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.InfOuEgal:
  			ch = parouv + operande1.chaineCalcul(varFor) + "<=" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.SupOuEgal:
  			ch = parouv + operande1.chaineCalcul(varFor) + ">=" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.Egalite:
  			ch = parouv + operande1.chaineCalcul(varFor) + "=" + operande2.chaineCalcul(varFor) + parfer;
  			break;
  		case Operateur.Diff:
  			ch = parouv + operande1.chaineCalcul(varFor) + "<>" + operande2.chaineCalcul(varFor) + parfer;
  	} // switch Operateur}
  	// Pour un calcul complexe nul la chane renvoye est "0+0*i" on renvoie alors "0"
  	if (ch.compareTo(zeroComp) == 0) return zero;
  	// Au cas o le calcul commenait par un moins unaire, la chaine commence par "0-"
  	if (ch.substring(0, 2).compareTo("0-") == 0) ch = ch.substring(1);
  	
  	return ch;		
	}
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    operateur = inps.readByte();
    operande1 = (CCalculBase) (inps.readCObject(list));
		operande2 = (CCalculBase) (inps.readCObject(list));
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operateur = inps.readByte();
    operande1 = (CCalculBase) (inps.readObject(list));
		operande2 = (CCalculBase) (inps.readObject(list));
  }  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(operateur);
    oups.writeObject(operande1);
    oups.writeObject(operande2);
  }
  
  @Override
	public boolean estConstant() {
		return operande1.estConstant() && operande2.estConstant();
	}
  
}
