/*
 * CPointImage.java
 *
 * Created on 10 juillet 2004, 14:43
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CPointImage extends CPointAncetre {
	public CPointAncetre antecedent;
	public CTransformation transformation;
  // Pour le positionnement
  private Point2D.Double pointim = new Point2D.Double();
  
  /** Creates new CPointImage */
  public CPointImage() {
  }
  
  /** Fonction devant tre redfinie pour tous les objets images par une transformation
   *  Sera utilise lorsqu'on modifie une image par une transformation pour que cette image
   *  soit associe  une autre transformation lrsque d'autres objets taient des images par
   *  cette mme transformation
   */
  public void associeATransformation(CTransformation trans) {
  	transformation = trans;
  }
  
  // Conctructeur plus court pour les crations visuelles d'objets
  public CPointImage(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, int motifInit, int indiceTailleNomInit, CPointAncetre antecedentInit, CTransformation transformationInit) {
      super(liste, impProtoInit, estElementFinalInit, couleurInit, false, 0d, 0d, false, "", indiceTailleNomInit, motifInit);
      antecedent = antecedentInit;
      transformation = transformationInit;  
  }
  
  /** Constructeur pour clonage */
  public CPointImage(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit, boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CPointAncetre antecedentInit, CTransformation transformationInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
          dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit);
        antecedent = antecedentInit;
        transformation = transformationInit;
  }  

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(antecedent);
    int ind2 = listeSource.indexOf(transformation);  
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointImage(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
        indiceTailleNom, motif, (CPointAncetre)(listeCible.get(ind1, CPointAncetre.class)),
        (CTransformation)(listeCible.get(ind2)));
    return ptelb;
  }
  
  @Override
  public String infoPourHistorique() {
    return textes.Chaines.chinfo18 + " " + transformation.designationTransf()
            + transformation.complementPourHist();
  }
  
  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(antecedent);
    transformation.AjouteElementsVisuelsPourHist(liste);
  } 
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || antecedent.dependDe(p) || transformation.dependDe(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || antecedent.dependDePourBoucle(p) || transformation.dependDePourBoucle(p));
  }

  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
	
		existe = transformation.existe && antecedent.existe;
		if (!existe) return;
		if (!transformation.pointAUneImage(antecedent))
		{
			existe = false;
			return;
		}
		transformation.transformePoint(antecedent.x,antecedent.y, pointim);
		super.placeEn(pointim.x, pointim.y);
	}
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
   **/
  public boolean confonduAvec(CElementBase p) {
    CPointImage pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointImage) p;
      return (transformation.confonduAvec(pt.transformation) &&
              antecedent.confonduAvec(pt.antecedent));
    } else return false;
  }
  
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (antecedent == ancienPoint) antecedent = nouveauPoint;
  }
  
  /**
   * fonction n'existant pas en C++ et renvoyant un pointeur vers l'antcdent direct de l'objet.
   * Si l'objet est un objet qui ne ncessite pas lors de sa cration la cration
   * d'objets supplmentaires, renvoie this et sinon envoi un pointeur sur le premier
   * de ces objets supplmentaires crs.
   * Par exemple, pour un point li  un bipoint (intersection de deux cercles
   * ou d'un cercle et une droite) renverra un pointeur sur le bipoint
   * cr avant ce CPointLieBipoint. Pour une objet image, renverra un pointeur
   * vers la transformation utilise etc ...
   */
  public CElementBase antecedentDirect() {
    return transformation;
  }

  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    antecedent = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));
  }  

  /** Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    antecedent = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    transformation = (CTransformation) (list.get(ind2,CTransformation.class));
  }
  
  /** Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(antecedent);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(transformation);
    oups.writeInt(ind2);
  }
  
  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
  	if(super.appartientParDefinition(droite)) return true;
   	int natTrans = transformation.natureTransformation();
		switch (natTrans) {
		case CTransformation.homothetie :
  		CHomothetie hom = (CHomothetie) transformation;
  		return (hom.centre.appartientParDefinition(droite)) && (antecedent.appartientParDefinition(droite));
		case CTransformation.symetrieCentrale :
  		CSymetrieCentrale sym = (CSymetrieCentrale) transformation;
  		return sym.centre.appartientParDefinition(droite) && antecedent.appartientParDefinition(droite);
  	case CTransformation.translation :
  			CTranslation trans = (CTranslation) transformation;
  			return (antecedent.appartientParDefinition(droite) && listeProprietaire.sontParDefinitionSurParallele(trans.origineVecteur, trans.extremiteVecteur, droite))
  				|| (trans.extremiteVecteur.appartientParDefinition(droite) && listeProprietaire.sontParDefinitionSurParallele(antecedent, trans.origineVecteur, droite));
  	
  	default : return false;
  	}
	}
  
  @Override
  public boolean estPointImage() {
  	return true;
  }
  
  
}
