/*
 * CPointLieBipoint.java
 *
 * Created on 8 juillet 2004, 08:54
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;


// Type de point d'intersection de deux objets
import java.io.*;
import java.awt.Color;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CPointLieBipoint extends CPointAncetre {
	CBipoint ptBipoint;
	int indiceDansBipoint; // Indice du point associe dans le bipoint

  /** Creates new CPointLieBipoint */
  public CPointLieBipoint() {
  }
  
  // Constructeur pour clonage
  public CPointLieBipoint(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit, CBipoint ptBipointInit, int indiceDansBipointInit) {
     super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
       dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit);
     ptBipoint = ptBipointInit;
     indiceDansBipoint = indiceDansBipointInit;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(ptBipoint);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointLieBipoint(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif,
        (CBipoint)(listeCible.get(ind1, CBipoint.class)), indiceDansBipoint);
    return ptelb; 
  }

  @Override
  public String infoPourHistorique() {
    return ptBipoint.infoPourHistorique();
  }

  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    ptBipoint.ajouteAntecedents(liste);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || ptBipoint.dependDe(p); 
  }

  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || ptBipoint.dependDePourBoucle(p)); 
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    CPointDansBipoint ptp;

    switch (indiceDansBipoint)
    {
      case 1:
        ptp = ptBipoint.point1;
        break;
      case 2:
        ptp = ptBipoint.point2;
        break;
      // Inutile mais exig par le compilateur java
      default : ptp = ptBipoint.point1;
    }
    existe = ptp.existe;
    if (!existe) return;
    super.placeEn(ptp.x, ptp.y);
  }
      
  /**
   * fonction n'existant pas en C++ et renvoyant un pointeur vers l'antcdent direct de l'objet.
   * Si l'objet est un objet qui ne ncessite pas lors de sa cration la cration
   * d'objets supplmentaires, renvoie this et sinon envoi un pointeur sur le premier
   * de ces objets supplmentaires crs.
   * Par exemple, pour un point li  un bipoint (intersection de deux cercles
   * ou d'un cercle et une droite) renverra un pointeur sur le bipoint
   * cr avant ce CPointLieBipoint. Pour une objet image, renverra un pointeur
   * vers la transformation utilise etc ...
   */
  @Override
  public CElementBase antecedentDirect() {
    return ptBipoint;
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    indiceDansBipoint = inps.readByte();
    int ind1 = inps.readCInt();
    ptBipoint = (CBipoint) (list.get(ind1,CBipoint.class));   
  }
  
  /* Chargement depuis un flux java */
  @Override
  public void read( MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    indiceDansBipoint = inps.readByte();
    int ind1 = inps.readInt();
    ptBipoint = (CBipoint) (list.get(ind1,CBipoint.class));   
  }
  
  /* Enregistrement dans un flux java */
  @Override
  public void write( MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeByte(indiceDansBipoint);
    int ind1 = list.indexOf(ptBipoint);
    oups.writeInt(ind1);
  }  
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return super.appartientParDefinition(droite) || ptBipoint.estDefiniPar(droite);
  }
  /** Fonction ajoute pour la version 2.1 en vue d'amliorer l'outil de cration d'intersection
   * Renvoie true si le point est sur la droite par dfinition
   * A redfinir pour les descendants concerns
   * */
  @Override
  public boolean appartientParDefinition(CCercle cercle)
  {
    return super.appartientParDefinition(cercle) || ptBipoint.estDefiniPar(cercle);
  } 
}
