/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import java.awt.geom.*;
import java.awt.Color;


import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;
import glob.*;


public class CPointLieCercle extends CPointLie {
	private CCercle cercleLie;

  /** Creates new CPointLieCercle */
  public CPointLieCercle() {
  }
  public CPointLieCercle(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit, boolean pointLibreInit, double abscisseInit, CCercle cercleLieInit, boolean marquePourTraceInit) {
    super (liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit,
      dec_xInit, dec_yInit, masqueInit, nomInit, indiceTailleNomInit, motifInit, pointLibreInit, abscisseInit, marquePourTraceInit);
     cercleLie = cercleLieInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(cercleLie);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CPointLieCercle(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom,
      indiceTailleNom, motif, pointLibre, abscisse, (CCercle)(listeCible.get(ind1, CCercle.class)), marquePourTrace);
    return ptelb;
  }
  
  /**
   * Fonction renvoyant la chane  afficher dans la bote de dialogue d'historique
   * pour dcrire l'objet.
   * @return
   */
  @Override
  public String infoPourHistorique() {
    return getString("chinfo2") + " " + cercleLie.complementInfo();
  }

  /**
   * Fonction ajoutant  list les lments qui ont gnr l'objet.
   * @param liste
   */
  @Override
  public void ajouteAntecedents(CListeObjets liste) {
    liste.add(cercleLie);
  }
   
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || cercleLie.dependDe(p));
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || cercleLie.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double xn,yn;
    CArcDeCercleAncetre ptArc;
    double angrot;
    VecteurDirecteur u = new VecteurDirecteur();
    VecteurDirecteur v = new VecteurDirecteur();

		existe = cercleLie.existe;
		if (!existe) return;
    switch (cercleLie.natureArc()) {
      case NatureArc.NatureArcPetitArc :
        ptArc = (CArcDeCercle)(cercleLie);
        u.setVecteur(ptArc.centre_x, ptArc.centre_y, ptArc.origine_x, ptArc.origine_y);
        angrot = ptArc.valeurAngleAuCentre;
        u.tourne(abscisse*angrot,v);
        break;
      case NatureArc.NatureArcGrandArc :
				ptArc = (CArcDeCercleAncetre)cercleLie;
        u.setVecteur(ptArc.centre_x, ptArc.centre_y, ptArc.origine_x, ptArc.origine_y);
 				angrot = ptArc.valeurAngleAuCentre;
				if (Kernel.zeroAngle(Math.abs(angrot)-Math.PI) || (angrot >= 0))
					u.tourne(-abscisse*(2*Math.PI-Math.abs(angrot)),v);
				else u.tourne(abscisse*(2*Math.PI-Math.abs(angrot)),v);
        break;
      case NatureArc.NatureArcArcDirect :
        ptArc = (CArcDeCercleAncetre)cercleLie;
        u.setVecteur(ptArc.centre_x, ptArc.centre_y, ptArc.origine_x, ptArc.origine_y);
        angrot = ptArc.valeurAngleAuCentre;
        if (angrot < 0) angrot = angrot + 2*Math.PI;
        u.tourne(abscisse*angrot,v);
        break;
      case NatureArc.NatureArcArcIndirect :
        ptArc = (CArcDeCercleAncetre)cercleLie;
        u.setVecteur(ptArc.centre_x, ptArc.centre_y, ptArc.origine_x, ptArc.origine_y);
        angrot = ptArc.valeurAngleAuCentre;
        VecteurDirecteur w = new VecteurDirecteur();
        u.tourne(angrot,w);
        angrot = - angrot;
        if (angrot < 0) angrot = angrot + 2*Math.PI;
        w.tourne(abscisse*angrot,v);
        break;
      case NatureArc.NatureArcCercle :
      default :
          u.x = cercleLie.rayon;
          u.y = 0;
          u.tourne(abscisse,v);
		}
		xn = cercleLie.centre_x + v.x;
		yn = cercleLie.centre_y + v.y;
		if (cercleLie.surArc(xn,yn))
		{
			existe = true;
			super.placeEn(xn,yn);
			super.positionne(infoRandom, dimfen); // Ajout version 4.6.4
		}
		else existe = false;
	}

  // La procdure suivante test si lengthdplacement du point correspondant  la position
  // (xtest, ytest)  de la souris est possible et si oui renvoyant dans pointr les
  // nouvelles coordonnes et dans pointAbs.x l'abscisse pour un point li

  public boolean testDeplacement(DimensionFenetre dimfen, double xtest, double ytest,  
         Point2D.Double pointr, Doub abscr) {
    boolean valable;

    VecteurDirecteur u = new VecteurDirecteur(cercleLie.centre_x, cercleLie.centre_y,
                                xtest, ytest);
    VecteurDirecteur v = new VecteurDirecteur();
    u.vecteurColineaire(cercleLie.rayon,v);
    if (u.nonNul() && cercleLie.surArc(xtest,ytest))
    {
      pointr.x = cercleLie.centre_x + v.x;
      pointr.y = cercleLie.centre_y + v.y;
      valable = dimfen.dansFenetre(pointr.x,pointr.y);
      // Si c'est un arc de cercle, il s'agira d'une abscisse curviligne
      // comprise entre 0 et 1
      double ang = v.angleRad();
      abscr.setValue(cercleLie.abscisseCurviligne(ang));
    }
    else valable = false;
    return valable;
  }
 
  public double abscisseMaximale() {
  	return cercleLie.abscisseMaximale();
  }

  public double abscisseMinimale() {
  	return cercleLie.abscisseMinimale();
  }

  public CElementBase lieA() {
    return cercleLie;
  }
  
  /** Rajout pour la version 2.0 : permet de connatre si l'lment point par pt
   * est confondu avec l'lment en question ou trs proche.
   * Pour cette version applet, ne sera redfini que pour les objets que l'utilisateur
   * pourra effectivement crer en ligne.
  **/
  @Override
  public boolean confonduAvec(CElementBase p) {
  	boolean res = super.confonduAvec(p);
   	if (!res) return false;
  	CPointLieCercle pt = (CPointLieCercle) p;
  	return (cercleLie == pt.cercleLie);
  }
  
  
  public void donneAbscisse(double abs) {
    abscisse = abs;
  }
  // Rajout pour la version 1.9.5
  @Override
	public boolean lieALigneFermee() {
    return (cercleLie.abscisseMinimale() == 0)
      && (cercleLie.abscisseMaximale() == 2*Math.PI);
  }
	
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    cercleLie = (CCercle) (list.get(ind1,CCercle.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    cercleLie = (CCercle) (list.get(ind1,CCercle.class));
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(cercleLie);
    oups.writeInt(ind1);
  }

  @Override
  public boolean appartientParDefinition(CCercle cercle)
  {
    return super.appartientParDefinition(cercle) || (cercleLie == cercle);
  }
}
