/*
 * CPuissance.java
 *
 * Created on 6 juillet 2004, 20:21
 */

/**
 *
 * @author  Yves
 * @version 
 */
package objets;

import java.io.*;

import types.Complexe;
import types.ErreurCalculException;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CPuissance extends CCalculBase {
  CCalculBase operande;
  CCalculBase exposant;
  // Attribut rajout pour optimisation
  Complexe z1 = new Complexe();
  Complexe ope1 = new Complexe();
  Complexe exp1 = new Complexe();

  /** Creates new CPuissance */
  public CPuissance() {
  }
  
  // Constructeur pour clonage
  public CPuissance(CListeObjets liste, CCalculBase operandeInit, CCalculBase exposantInit) {
    super(liste);
    operande = operandeInit;
    exposant = exposantInit;
  }

  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase cloneOperande = operande.getClone(listeSource, listeCible); 
    CCalculBase cloneExposant = exposant.getClone(listeSource, listeCible); 
    //  revoir car la fonction a dj t cre
    CCalculBase ptcalc = new CPuissance(listeCible, cloneOperande, cloneExposant);
    return ptcalc;
  }
  
  public boolean dependDe(CElementBase p) {
  	return operande.dependDe(p) || exposant.dependDe(p);
  }

  public boolean dependDePourBoucle(CElementBase p) {
  	return operande.dependDePourBoucle(p) || exposant.dependDePourBoucle(p);
  }

  boolean existe() {
    return operande.existe() && exposant.existe();
  }

  // La procdure suivante a t optimise pour la version 2.0
  public double resultatBase(boolean infoRandom, double x, double n) throws ErreurCalculException {
    if((x == 0) && (n != 0))
    {
      if (n>0) return 0;
      else 
      {
        throw new ErreurCalculException();
      }
    }
    if (n == 0)
    {
      if (x == 0)
      {
        throw new ErreurCalculException();
      }
      else return 1;
    }
    else
    {
      // Optimisation pour la version 2.0
      // Pour un exposant entier compris entre 1 et 255
      // on ne fait que des produits
      if ((n == Math.floor(n)) && (n > 0) && (n < 256)) return CCalculBase.PuisExpEnt(x, (int)n);
      else
      {
        if (x < 0)
        {
          if (n == Math.floor(n))
          {
            if (n == 2*Math.floor(n/2))
            return Math.exp(n*Math.log(Math.abs(x)));
            else return -Math.exp(n*Math.log(Math.abs(x)));
          }
          else
          {
            throw new ErreurCalculException();
          }
        }
        else return Math.exp(n*Math.log(x));
      }
    }
  }
  public double resultat(boolean infoRandom) throws ErreurCalculException {
    double x1 = operande.resultat(infoRandom);
    double n1 = exposant.resultat(infoRandom);
    return resultatBase(infoRandom, x1, n1);   
  }
  public double resultatFonction(boolean infoRandom, double valeurParametre) throws ErreurCalculException {
    double x1 = operande.resultatFonction(infoRandom, valeurParametre);
    double n1 = exposant.resultatFonction(infoRandom, valeurParametre);
    return resultatBase(infoRandom, x1, n1);
  }
  void resultatComplexeBase(boolean infoRandom, Complexe ope, Complexe exp, Complexe zRes)  throws ErreurCalculException {
    boolean openull = ope.moduleCarre() == 0;
    boolean expnull = exp.moduleCarre() == 0;
    if (expnull)
    {
      if (openull)
      {
        throw new ErreurCalculException();
      }
      else
      {
        zRes.x = 1;
        zRes.y = 0;
        return;
      }
    }
    else
    {
      if (openull)
      {
        zRes.x = 0;
        zRes.y = 0;
        // Si l'oprande est nul, on regarde si l'exposant est rel ou pas.
        // S'il ne l'est pas, le rsultat n'existe pas
        if (exp.y != 0)
        {
          throw new ErreurCalculException();
        }
        else
        {
          if (exp.x > 0)
          {
            return;
          }
          else
          {
            throw new ErreurCalculException();
          }
        }
      }
      else // Si l'oprande n'est pas nul ni l'exposant
      {
        // On regarde si l'exposant est rel et strictemnt infrieur  256.
        // Si oui on utilise un algorithme n'utilisant que des produits
        if ((exp.y == 0) && (exp.x == Math.floor(exp.x))) {
          if ((exp.x>0) && (exp.x < 256))
          {
            int expent = (int)exp.x;
            ope.puisCompExpEnt(expent, zRes);
          }
          else
          {
            if ((exp.x<0) && (exp.x > -256))
            {
              int expent = (int)exp.x;
              ope.puisCompExpEnt(-expent, z1);
              z1.InverseComp(zRes);
            }
            else ope.puisComp(exp, zRes);
          }
        }
        else
        {
          ope.puisComp(exp, zRes);
        }
      }
    }    
  }
  public void resultatComplexe(boolean infoRandom, Complexe zRes)  throws ErreurCalculException {
    operande.resultatComplexe(infoRandom, ope1);
    exposant.resultatComplexe(infoRandom, exp1);
    resultatComplexeBase(infoRandom, ope1, exp1, zRes);
  }  
  void  resultatFonctionComplexe(boolean infoRandom, Complexe valeurParametre, Complexe zRes)
                            throws ErreurCalculException {
    operande.resultatFonctionComplexe(infoRandom, valeurParametre, ope1);
    exposant.resultatFonctionComplexe(infoRandom, valeurParametre, exp1);
    resultatComplexeBase(infoRandom, ope1, exp1, zRes);
  }
  public boolean  dependDeVariable() {
    return operande.dependDeVariable() || exposant.dependDeVariable();
  }
  public CCalculBase getCopie() {
  	return new CPuissance(listeProprietaire, operande.getCopie(), exposant.getCopie());
  }
  
	public String chaineCalcul(String varFor) {
		return operande.chaineCalcul(varFor) + "^" + exposant.chaineCalcul(varFor);
	}
  
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    operande = (CCalculBase) (inps.readCObject(list));
    exposant = (CCalculBase) (inps.readCObject(list));
  }
  
  /* Chargement depuis un flux java */
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    operande = (CCalculBase) (inps.readObject(list));
    exposant = (CCalculBase) (inps.readObject(list));
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    oups.writeObject(operande);
    oups.writeObject(exposant);
  }
  
  @Override
	public boolean estConstant() {
		return operande.estConstant() && exposant.estConstant();
	}
  
}
