/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CResultatValeur extends CCalculBase {
  // La valeur associe
	public CValeurDynamique valeurAssociee;

  /** Creates new CResultatValeur */
  public CResultatValeur() {
  }
  
  // Constructeur avec une valeur initiale
  public CResultatValeur(CListeObjets liste, CValeurDynamique valeurInit)
  {
    super(liste);
    valeurAssociee = valeurInit;
  }
  
  @Override
  protected long nature() {
  	return natResultatValeur;
  }  

  public CCalculBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(valeurAssociee);    
    CCalculBase ptcalc = new CResultatValeur(listeCible, (CValeurDynamique)(listeCible.get(ind1, CValeurDynamique.class)));
    return ptcalc;  
  }
  
  @Override
  public boolean existe(boolean infoRandom) {
  	return valeurAssociee.existe;
  }
  
  public double resultat(boolean infoRandom) {
    return valeurAssociee.rendValeur(infoRandom);
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	return valeurAssociee.dependDe(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
  	return valeurAssociee.dependDePourBoucle(p);
  }
  
  @Override
  public boolean  dependDeVariable(int ind) {
    return false;
  }
  
  @Override
  public CCalculBase getCopie() {
  	return new CResultatValeur(listeProprietaire, valeurAssociee);
  } 
  
  @Override
	public String chaineCalcul(String[] varFor) {
		return valeurAssociee.nom();
	}
  
  @Override
  public String chaineLatex(String[] varFor) {
  	if (valeurAssociee.nom().equals("pi")) return "\\pi "; // Espace  la fin ncesssaire
  	else return valeurAssociee.nom();
  }
  
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    valeurAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  }
  
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    valeurAssociee = (CValeurDynamique) (list.get(ind1,CValeurDynamique.class));
  }
  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(valeurAssociee);
    oups.writeInt(ind1);
  }
  
  // L'appel  une valeur dynamique est forcment non constant
  @Override
	public boolean estConstant() {
  	return valeurAssociee.getClass() == CCalcConst.class;
	}
  
  // Ajout version 4.6
  /**
   * Pour un rsultat de valeur on remplace par une constante ayant la mme valeur
   */
  @Override
  public CCalculBase calculAvecValeursRemplacees() {
  	double d = valeurAssociee.rendValeur(false);
  	if (d < 0) return new CMoinsUnaire(listeProprietaire, new CConstante(listeProprietaire, -d));
  	else return new CConstante(listeProprietaire, d);
  }
  
  @Override
	public CCalculBase membreGauche() {
  	return valeurAssociee.membreGauche();
	}
	
	// Ajout version 4.6
	public CCalculBase membreDroit() {
  	return valeurAssociee.membreDroit();
	}
  
}
