/*
 * CSolutionEquation.java
 *
 * Created on 21 juillet 2004, 12:57
 */

/**
 *
 * @author  Yves
 * @version
 */
package objets;

import java.io.*;
import types.*;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CSolutionEquation extends CCalculAncetre {
  public CFonc fonctionAssociee;
  // Les bornes de l'intervalle de recherche
  public CValeurAssocieeAVariable a;
  public CValeurAssocieeAVariable b;
  // L'incertitude recherche
  public CValeurAssocieeAVariable incertitude;
  // La valeur de la solution de l'quation
  private double solution;
  
  /** Creates new CSolutionEquation */
  public CSolutionEquation() {
  }
  
  // Constructeur pour clonage
  public CSolutionEquation(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    String nomInit, CFonc fonctionAssocieeInit,
    CValeurAssocieeAVariable aInit, CValeurAssocieeAVariable bInit, CValeurAssocieeAVariable incertitudeInit) {
    super(liste, impProtoInit, estElementFinalInit, nomInit);
    fonctionAssociee = fonctionAssocieeInit;
    a = aInit;
    b = bInit;
    incertitude = incertitudeInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(fonctionAssociee);
    int ind2 = listeSource.indexOf(impProto);
    CValeurAssocieeAVariable aClone = (CValeurAssocieeAVariable) a.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable bClone = (CValeurAssocieeAVariable) b.getClone(listeSource, listeCible);
    CValeurAssocieeAVariable incertitudeClone = (CValeurAssocieeAVariable) incertitude.getClone(listeSource, listeCible);
    CElementBase ptelb = new CSolutionEquation(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, nomCalcul, (CFonc)(listeCible.get(ind1, CFonc.class)), aClone,
      bClone, incertitudeClone);
    return ptelb;
  }
  
  @Override
  public long getNatureCalcul() {
  	return NatureCalcul.NSolutionEquation;
  }
  
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || a.dependDe(p) || b.dependDe(p)
    || incertitude.dependDe(p) || fonctionAssociee.dependDe(p);
  }
  
  public boolean dependDePourBoucle(CElementBase p) {
    return (p == this) || a.dependDePourBoucle(p) || b.dependDePourBoucle(p)
    || incertitude.dependDePourBoucle(p) || fonctionAssociee.dependDePourBoucle(p);
  }
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    double xm,xa,xb,ya,yb,ym,e;
    a.positionne(infoRandom, dimfen);
    b.positionne(infoRandom, dimfen);
    incertitude.positionne(infoRandom, dimfen);
    existe = fonctionAssociee.existe && a.existe() && b.existe() && incertitude.existe() ;
    if (!existe) return;
    xa = a.rendValeur(infoRandom);
    xb = b.rendValeur(infoRandom);
    e = incertitude.rendValeur(infoRandom);
    if (xa > xb) {
      double temp = xa;
      xa = xb;
      xb = temp;
    }
    try {
      ya = fonctionAssociee.rendValeurFonction(infoRandom, xa);
      yb = fonctionAssociee.rendValeurFonction(infoRandom, xb);
      if ((ya*yb > 0) || (e <= 0)) {
        existe = false;
        return;
      }
      
    } catch (ErreurCalculException err) {
      existe = false;
      return;
    }
    double nBoucles = Math.floor(Math.log(Math.abs(xb - xa)/e)/Math.log(2)) + 1;
    if (nBoucles > 1000) {
      existe = false;
      return;
    }
    xm = (xa + xb) /2;
    for (int i = 1; i <= (int) nBoucles; i ++) {
      xm = (xa + xb) /2;
      try {
        ym = fonctionAssociee.rendValeurFonction(infoRandom,xm);
      } catch (ErreurCalculException err) {
        existe = false;
        return;
      }
      if (ym*ya > 0) xa = xm;
      else xb = xm;
    }
    existe = true;
    solution = xm;
  }
  
  @Override
  public double rendValeur(boolean infoRandom) {
    return solution;
  }
    
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    a = new CValeurAssocieeAVariable();
    a.chargeDepuisFlux(inps, list);
    b = new CValeurAssocieeAVariable();
    b.chargeDepuisFlux(inps, list);
    incertitude = new CValeurAssocieeAVariable();
    incertitude.chargeDepuisFlux(inps, list);
  }
  /* Lecture depuis dans un flux java */  
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    fonctionAssociee = (CFonc) (list.get(ind1,CFonc.class));
    a = new CValeurAssocieeAVariable();
    a.read(inps, list);
    b = new CValeurAssocieeAVariable();
    b.read(inps, list);
    incertitude = new CValeurAssocieeAVariable();
    incertitude.read(inps, list);
  }
  
  /* Enregistrement dans un flux java */
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(fonctionAssociee);
    oups.writeInt(ind1);
    a.write(oups, list);
    b.write(oups, list);
    incertitude.write(oups, list);
  }
  
  @Override
	public boolean estConstant() {
		return a.estConstant() && b.estConstant() && incertitude.estConstant() && fonctionAssociee.estConstant();
	}

}
