/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import Applet.*;
import java.awt.*;

import objets.*;
import types.*;

/**
 *
 * @author Yves
 * Outil servant  crer l'intersection de deux objets.
 */
public class OutilInt extends Outil {
  CElementGraphique elt1, elt2;
  
  /** Creates a new instance of OutilInt */
  public OutilInt(AppletMathGraph32 appletMereInit) {
    super(appletMereInit, true, AppletMathGraph32.iconCreerIntersection);
  }
  
  public int idMenu() {
    return 32805;
  }
  
  public String toolTip() {
    return getString("toolTip11");
  }
  
  /** Appel lorsque l'outil est slectionn */
  public void select() {
    super.select();
    elt1 = null;
    elt2 = null;
    appletMere.outilPointageActif = appletMere.outilPointageIntersection;
    designationDe(NatureObjet.NTouteDroite | NatureObjet.NToutCercle);
    appletMere.outilPointageActif.reset();
    appletMere.indication(getString("intersection1"));
  }
 
  /* Appel lorsqu'un objet a t cliqu pour tre trait par l'outil
   * par exemple le premier point pour crer une droite passant par deux points */
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    if (elt1 == null) {
      appletMere.indication(getString("intersection2")); 
      elt1 = elg;
      excluDeDesignation(elg);
      ajouteClignotementDe(elg);
      resetClignotement();
    }
    else {
      if (elt2 == null) {
        elt2 = elg;
        creeObjet();
        annuleClignotement();
      }
    }
  }
  
  public void traiteObjetsDesignes(CElementGraphique elg1, CElementGraphique elg2, Point pointClique) {
    elt1 = elg1;
    elt2 = elg2;
    creeObjet();
  }  
  
  /* Appel pour crer finalement l'objet une fois que tous les objets dont il
   * dpend ont t dsigns */
  public void creeObjet() {
    CListeObjets liste = appletMere.listePr;
    long nat1 = elt1.getNature();
    long nat2 = elt2.getNature();
    boolean existe = false;
    boolean pasDejaCree = false;
    int NbObjetsCrees = 0;
    Bool intersectionInutile = new Bool(false);
    CElementBase inter;
    CPointAncetre pointDejaCree = null;
    
    Color coul = appletMere.couleurActive();
    if (((nat1 & NatureObjet.NTouteDroiteSaufVecteur) != 0) 
      & ((nat2 & NatureObjet.NTouteDroiteSaufVecteur) != 0)) {
      // Cas de l'intersection de deux objets de type droite
      // Technique d'intersection "intelligente"
      // On regarde d'abord s'il existe un point dj cr qui soit sur un des deux objets
      // et trs proche d'un point d'intersection.
      // Si oui on cre un objet de type AutrePointIntersectionCercles
      // A t amlior par rapport  la version C++ qui ne rgardait pas le cas de deux droites
      boolean pointIntDejaCree = liste.pointIntersectionDejaCree((CDroiteAncetre)elt1, (CDroiteAncetre)elt2);
      if (pointIntDejaCree) {
        // les deux points d'intersection sont dj crs et sur les deux objets
        appletMere.dialogueAvertissement(getString("ch6"));
        select();
        return;
      }
      else {
        inter = new CIntDroiteDroite(liste, null, false, coul, false, 0d, 0d, false, "",
           2, MotifPoint.petitRond, (CDroiteAncetre) elt1, (CDroiteAncetre) elt2, false);
        pasDejaCree = appletMere.ajouteElementAvecVerification(inter);

        if (pasDejaCree) {
          annuleClignotement();
          existe = inter.existe;
          NbObjetsCrees = 1;
        }
      }
    }
    else {
      if (((nat1 & NatureObjet.NToutCercle) != 0) 
        && ((nat2 & NatureObjet.NToutCercle) != 0)) {
        // Cas de l'intersection de deux objets de type cercle
        // Technique d'intersection "intelligente"
				// On regarde d'abord s'il existe un point dj cr qui soit sur un des deux objets
				// et trs proche d'un point d'intersection.
				// Si oui on cre un objet de type AutrePointIntersectionCercles
				pointDejaCree = liste.pointIntersectionDejaCree
					((CCercle)elt1, (CCercle)elt2, intersectionInutile);
				if (intersectionInutile.getValue()) {
          // les deux points d'intersection sont dj crs et sur les deux objets
          appletMere.dialogueAvertissement(getString("ch6"));
          select();
					return;
				}
				else
				{
					if (pointDejaCree != null)
					{
						CAutrePointIntersectionCercles intpoint = new CAutrePointIntersectionCercles
              (liste, null, false, coul, false, 0d, 0d, false, "", 2, 
              		MotifPoint.petitRond, (CCercle) elt1, (CCercle) elt2, pointDejaCree);
            pasDejaCree = appletMere.ajouteElementAvecVerification(intpoint);
            if (pasDejaCree) {
              NbObjetsCrees = 1;
              appletMere.positionne();
              existe = intpoint.existe;
            }
					}
					else
					{        
            inter = new CIntCercleCercle(liste, null, false, (CCercle) elt1, (CCercle) elt2);
            pasDejaCree = appletMere.ajouteElementAvecVerification(inter);
            if (pasDejaCree) {
              CPointLieBipoint ptpl1 = new CPointLieBipoint(liste, null, false,
                coul, false, 0d, 0d, false, "", 2, MotifPoint.petitRond, (CIntCercleCercle) inter, 1, false);
              appletMere.ajouteElement(ptpl1);    
              CPointLieBipoint ptpl2 = new CPointLieBipoint(liste, null, false,
                coul, false, 0d, 0d, false, "", 2, MotifPoint.petitRond, (CIntCercleCercle) inter, 2, false);
              appletMere.ajouteElement(ptpl2);
              appletMere.positionne();
              annuleClignotement();
              existe = inter.existe;
              NbObjetsCrees = 3;
            }
          }
        }
      }
      else { // cas d'une droite et un cercle
        CDroiteAncetre ptd;
        CCercle ptc;
        if ((nat1 & NatureObjet.NTouteDroiteSaufVecteur) != 0) {
          ptd = (CDroiteAncetre) elt1;
          ptc = (CCercle) elt2;
        }
        else {
          ptd = (CDroiteAncetre) elt2;
          ptc = (CCercle) elt1;
        }
        // Technique d'intersection "intelligente"
				// On regarde d'abord s'il existe un point dj cr qui soit sur un des deux objets
				// et trs proche d'un point d'intersection.
				// Si oui on cre un objet de type AutrePointIntersectionCercles
				intersectionInutile = new Bool(false);
				pointDejaCree = liste.pointIntersectionDejaCree(ptd, ptc, intersectionInutile);
				if (intersectionInutile.getValue()) {
          // les deux points d'intersection sont dj crs et sur les deux objets
          appletMere.dialogueAvertissement(getString("ch6"));
					return;
				}
				else
				{
					if (pointDejaCree != null)
					{
						CAutrePointIntersectionDroiteCercle intpoint = new CAutrePointIntersectionDroiteCercle
              (liste, null, false, coul, false, 0d, 0d, false,
              "", 2, MotifPoint.petitRond, ptc, ptd, pointDejaCree);
            pasDejaCree = appletMere.ajouteElementAvecVerification(intpoint);
            NbObjetsCrees = 1;
            appletMere.positionne();
            existe = intpoint.existe;
            
					}
          else {
            inter = new CIntDroiteCercle(liste, null, false, (CDroiteAncetre) ptd, (CCercle) ptc);
            pasDejaCree = appletMere.ajouteElementAvecVerification(inter);
            if (pasDejaCree) {
            CPointLieBipoint ptpl1 = new CPointLieBipoint(liste, null, false,
              coul, false, 0d, 0d, false, "", 2, MotifPoint.petitRond, (CIntDroiteCercle) inter, 1, false);
            appletMere.ajouteElement(ptpl1);    
            CPointLieBipoint ptpl2 = new CPointLieBipoint(liste, null, false,
              coul, false, 0d, 0d, false, "", 2, MotifPoint.petitRond, (CIntDroiteCercle) inter, 2, false);
            appletMere.ajouteElement(ptpl2);    
            appletMere.positionne();
            existe = inter.existe;
            NbObjetsCrees = 3;
            }
          }
        }
      }
    }
    if (pasDejaCree) {
      if (existe) {
        appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil10"));        
      }
      else { // Si les objets ne sont pas scants ou  alors le seul point  crer existe dj
        if (pointDejaCree == null) appletMere.dialogueAvertissement(getString("ch109")); // Objets non scants affich
        else appletMere.dialogueAvertissement(getString("ch6")); // Affich : le ou les points d'intersection existent dj
        appletMere.detruitNDerniersElements(NbObjetsCrees);
      }
    }
    select();
  }

}
