/**
  	Applet MathGraph32 Java. Software for animating dynamic mathematics figures on web pages
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


package outils;

import java.awt.Point;
import java.util.Vector;
import Applet.AppletMathGraph32;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CNoeudPointeurSurPoint;
import objets.CPointAncetre;
import objets.CPolygone;
import objets.CSegment;
import types.NatureObjet;

public class OutilPolygone extends Outil {
  CPointAncetre premierPoint;
  Vector<CNoeudPointeurSurPoint> listePoints;
  
	public OutilPolygone(AppletMathGraph32 appletMereInit) {
		super(appletMereInit, true, AppletMathGraph32.iconCreerPolygone);
	}
	
  public int idMenu() {
    return 32864;
  }
  
  public String toolTip() {
    return getString("toolTip34");
  }
	
  @Override
  public void select() {
    super.select();
    designationDe(NatureObjet.NToutPoint);
    appletMere.outilPointageActif = appletMere.outilPointageCre;
    appletMere.indication(getString("polygone1"));
    listePoints = new Vector<CNoeudPointeurSurPoint> ();
  }
  
  @Override
  public void deselect() {
    listePoints = null;
    System.gc();
    super.deselect();
  }  
    
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    CPointAncetre point = (CPointAncetre) elg;
    ajouteClignotementDe(elg);
    if (listePoints.size() == 0) {
    	premierPoint = point;
  		CNoeudPointeurSurPoint n = new CNoeudPointeurSurPoint(point.listeProprietaire, point);
  		listePoints.add(n);
  		ajouteObjetsVisuels();
  		ajouteClignotementDe(point);
  		resetClignotement();
      appletMere.indication(getString("polygone2"));  		
    }
    else {
    	if (point == premierPoint) {
    		if (listePoints.size() <= 2) {
    			appletMere.dialogueAvertissement(getString("ch41"));
    			select();
    		}
    		else {
      		appletMere.listeExclusionDesignation.retireTout();
    			CNoeudPointeurSurPoint n = new CNoeudPointeurSurPoint(point.listeProprietaire, point);
    			listePoints.add(n);
     			creeObjetApresClicDroit();
    		}
    	}
    	else {
    		appletMere.listeExclusionDesignation.retireTout();
    		excluDeDesignation(point);
  			CNoeudPointeurSurPoint n = new CNoeudPointeurSurPoint(point.listeProprietaire, point);
  			listePoints.add(n);
  			ajouteObjetsVisuels();
    		ajouteClignotementDe(point);
    		resetClignotement();
    	}
    }
  }
 
  @Override
  public void creeObjetApresClicDroit() {
		if (listePoints.size() <= 2) {
			appletMere.dialogueAvertissement(getString("ch41"));
			select();
		}
		else {
			// Si le polygone n'a pas t ferm on le ferme
			CNoeudPointeurSurPoint n1 = listePoints.firstElement();
			CNoeudPointeurSurPoint n2 = listePoints.lastElement();
			if (n1.pointeurSurPoint != n2.pointeurSurPoint ) {
				CPointAncetre pointDebut = n1.pointeurSurPoint;
				CNoeudPointeurSurPoint n = new CNoeudPointeurSurPoint(pointDebut.listeProprietaire, pointDebut);
				listePoints.add(n);
			}
			CPolygone poly = new CPolygone(appletMere.listePr, listePoints, appletMere.couleurActive(), appletMere.styleTraitActif());
			if (appletMere.ajouteElementAvecVerification(poly)) {
			  appletMere.gestionnaire.enregistreFigureEnCours(getString("chaineOutil36"));
			  appletMere.repaintFigureAvecCalcul();
			}
	    select();		
		}
  }
  
  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = appletMere.listeObjetsVisuels;
  	// Si ce n'est pas le premier point, on transforme le dernier segment en modifiant
  	// son extrmit;
  	CPointAncetre dernierPoint = listePoints.lastElement().pointeurSurPoint;
  	if (listePoints.size() >=2)
  	{
  		CSegment pts2 = (CSegment) liste.get(liste.size()-1);
  		pts2.point2 = dernierPoint;
  	}
  	// Si on en est au deuxime point cliqu, on rajoute un segment pour fermer la boucle
  	if (listePoints.size() == 2)
  	{
  		CPointAncetre ptp = listePoints.firstElement().pointeurSurPoint;
  		CSegment pts3 = new CSegment(liste, null, false, appletMere.couleurActive(), false, appletMere.styleTraitActif(), appletMere.pointSouris, ptp);
  		liste.add(pts3);
  	}
  	// On cre un segment joignant le dernier point entr au point souris
  	CSegment pts1 = new CSegment(liste, null, false,appletMere.couleurActive(), false, appletMere.styleTraitActif(),
  			dernierPoint, appletMere.pointSouris);
  	liste.add(pts1);
  }
}