/*
 * Complexe.java
 *
 * Created on 27 janvier 2007, 09:49
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 * Type complexe cr pour la version 2.0
 */

package types;
import glob.*;
import objets.CCalculBase;
import objets.CValeurAngle;

/**
 * Classe reprsentant les nombres complexes.
 * x est la partie relle
 * y est la partie imaginaire
 * @author Yves
 */
public  class Complexe {
  public double x;
  public double y;
  // Les membres suivants sont crs pour ne pas crer de complexes sur la piles lors des
  // calculs
  
  /** Creates a new instance of Complexe */
  public Complexe() {
  }
  /** Cration  partir de deux valeurs donnes*/
  public Complexe (double xinit, double yinit) {
    x = xinit;
    y = yinit;
  }
  /* Cration  partir d'un complexe dj existant */
  public Complexe (Complexe complexeInit) {
    x = complexeInit.x;
    y = complexeInit.y;
  }
  public void set(double xInit, double yInit)
  {
    x = xInit;
    y = yInit;
  }
  public void set(Complexe zInit)
  {
    x = zInit.x;
    y = zInit.y;
  }
  
  public boolean isNanOrInfinite() {
     return Double.isNaN(x) || Double.isInfinite(x)
     || Double.isNaN(y) || Double.isInfinite(y);
  }
  /////////////////////////////////////////////////////////////////////
  // Dfinition des fonctions de calcul sur les nombres complexes
  /////////////////////////////////////////////////////////////////////
  /*  Fonction renvoyant l'argument principal ( condition que le complexe
  // soit non nul. Tient compte de l'unit d'angle */
  public double argument(int uniteAngle)
  {
    double resul;
    if (x == 0) {
      if (y > 0)  resul = Math.PI/2;
      else resul = - Math.PI/2;
    }
    else
    {
      double tangente = y/x;
      if (x > 0) resul = Math.atan(tangente);
      else
      {
        if (y >= 0) resul = Math.atan(tangente) + Math.PI;
        else resul = Math.atan(tangente) - Math.PI;
      }
    }
    if (uniteAngle == UniteAngle.Radian) return resul;
    else return resul*Kernel.ConvRadDeg;
  }  
  /* Fonction renvoyant le module */
  public double module()
  {
    return Math.sqrt(x*x + y*y);
  }
  /* Renvoie la somme de this avec z1 */
  public void sommeComp(Complexe z1, Complexe res) {
    res.x = x + z1.x;
    res.y = y + z1.y;
  }
  /* Renvoie la diffrence de this avec z1 */
  public void differenceComp(Complexe z1, Complexe res) {
    res.x = x - z1.x;
    res.y = y - z1.y;
  }
  /* Renvoie le produit de this avec z1 */
  public void produitComp(Complexe z1, Complexe res) {
    res.x = x*z1.x - y*z1.y;
    res.y = x*z1.y + z1.x*y;
  }
  public double moduleCarre()
  {
    return x*x + y*y;
  }
  /* Renvoie le quotient de this par z1 */
  public void quotientComp(Complexe z1, Complexe res)
  {
    double mod2 = z1.moduleCarre();
    res.x = (x*z1.x + y*z1.y)/mod2;
    res.y = (z1.x*y - x*z1.y)/mod2;
  }
  public void InverseComp(Complexe res)
  {
    double mod2 = moduleCarre();
    res.x = x/mod2;
    res.y = -y/mod2;
  }
  public void expComp(int uniteAngle, Complexe res)
  {
    if (uniteAngle == UniteAngle.Radian) {
      res.x = Math.exp(x)*CCalculBase.cosinus(y);
      res.y = Math.exp(x)*CCalculBase.sinus(y);
    }
    else
    {
      res.x = Math.exp(x)*CCalculBase.cosinus(y*CValeurAngle.convDegRad);
      res.y = Math.exp(x)*CCalculBase.sinus(y*CValeurAngle.convDegRad);
    }
  }
  public void sinComp(int uniteAngle, Complexe res)
  // On utilise la formule sinz = 1/(2i)(exp(iz)-exp(-iz))
  {
    Complexe c1 = new Complexe(-y, x);
    // c1 contient iz
    Complexe c2 = new Complexe(y, -x);
    // c2 contient -iz
    Complexe c3 = new Complexe();
    c1.expComp(uniteAngle, c3); // C3 contient exp(iz)
    Complexe c4 = new Complexe();
    c2.expComp(uniteAngle, c4); // C4 contient exp(-iz)
    Complexe c5 = new Complexe();
    c3.differenceComp(c4, c5);
    // On divise c5 par -2i, donc on le multiplie par 0.5*i
    res.x = -0.5*c5.y;
    res.y = 0.5*c5.x;
    // Pour soulager le garbage Collector
    c1 = null;
    c2 = null;
    c3 = null;
    c4 = null;
    c5 = null;
  }
  public void cosComp(int uniteAngle, Complexe res)
  // On utilise la formule sinz = 1/2(exp(iz)+exp(-iz))
  {
    Complexe c1 = new Complexe(-y, x);
    // c1 contient iz
    Complexe c2 = new Complexe(y,-x);
    // c2 contient -iz
    Complexe c3 = new Complexe();
    c1.expComp(uniteAngle, c3); // C3 contient exp(iz)
    Complexe c4 = new Complexe();
    c2.expComp(uniteAngle, c4); // C4 contient exp(-iz)
    Complexe c5 = new Complexe();
    c3.sommeComp(c4, c5);
    res.x = 0.5*c5.x;
    res.y = 0.5*c5.y;
    // Pour soulager le garbage Collector
    c1 = null;
    c2 = null;
    c3 = null;
    c4 = null;
    c5 = null;
  }
  public void tanComp(int uniteAngle, Complexe res)
  {
    // On utilise la formule 1/i*(exp(2iz-1)/(exp(2iz+1))
    Complexe c1 = new Complexe(-2*y, 2*x); // c1 contient 2ic
    // c1 contient 2ic;
    Complexe c2 = new Complexe();
    c1.expComp(uniteAngle, c2); // c2 contient exp(2ic)
    Complexe c3 = new Complexe(c2.x + 1, c2.y); // c3 contient exp(2c) + 1
    c2.x = c2.x - 1; // z1 contient exp(2c) - 1
    Complexe c4 = new Complexe();
    c2.quotientComp(c3, c4);
    res.set(c4.y, -c4.x);
    // Pour soulager le garbage Collector
    c1 = null;
    c2 = null;
    c3 = null;
    c4 = null;
  }
  public void shComp(int uniteAngle, Complexe res)
  {
    Complexe c1 = new Complexe(-x, -y);
    // c1 contient -this;
    Complexe c2 = new Complexe();
    expComp(uniteAngle, c2); // c2 contient expComp(this)
    Complexe c3 = new Complexe();
    c1.expComp(uniteAngle, c3); // C3 contient  expComp(-this)
    c2.differenceComp(c3, res);
    res.x /= 2;
    res.y /= 2;
    // Pour soulager le garbage Collector
    c1 = null;
    c2 = null;
    c3 = null;
  }
  public void chComp(int uniteAngle, Complexe res)
  {
    Complexe c1 = new Complexe(-x, -y);
    // c1 contient -this;
    Complexe c2 = new Complexe();
    expComp(uniteAngle, c2); // c2 contient expComp(this)
    Complexe c3 = new Complexe();
    c1.expComp(uniteAngle, c3); // c3 contient expComp(-this)
    c2.sommeComp(c3, res);
    res.x /= 2;
    res.y /= 2;
     // Pour soulager le garbage Collector
    c1 = null;
    c2 = null;
    c3 = null;
 }
  public void thComp(int uniteAngle, Complexe res)
  {
    Complexe c1 = new Complexe(2*x, 2*y); // c1 contient 2*this
    // c1 contient 2c;
    Complexe c2 = new Complexe();
    c1.expComp(uniteAngle, c2); // c2 contient exp(2this)
    Complexe c3 = new Complexe(c2.x + 1, c2.y); // c3 contient exp(2c) + 1
    c2.x = c2.x - 1; // z1 contient exp(2c) - 1
    c2.quotientComp(c3, res);
     // Pour soulager le garbage Collector
    c1 = null;
    c2 = null;
    c3 = null;
  }
  /* LnComp renvoie le ln du module +i*l'argument principal.
  // Le complexe doit tre non nul
  // Tient compte de l'unit d'angle*/
  public void lnComp(int uniteAngle, Complexe res)
  {
    double r = module();
    res.x = Math.log(r);
    res.y = argument(uniteAngle);
  }
  // Elevation du complexe this (non nul)  la puissance le complexe b
  // On utilise la formule z1^z2 = exp(z2*ln(z1))
  public void puisComp(Complexe b, Complexe res)
  {
    Complexe c1 = new Complexe();
    lnComp(UniteAngle.Radian, c1); // c1 contient ln(this)
    Complexe c2 = new Complexe();
    b.produitComp(c1, c2); // c2 contient b*ln(this)
    c2.expComp(UniteAngle.Radian, res);
     // Pour soulager le garbage Collector
    c1 = null;
    c2 = null;
  }
  // Elvation d'un complexe  une puissance entire d'exposant
  // strictement positif, infrieur ou gal  255 et non nul
  // On utilise l'algorithme optimis n'utilisant que des produits.
  public void puisCompExpEnt(int exposant, Complexe res)
  {
    res.x = 1;
    res.y = 0;
    Complexe tampon = new Complexe(x, y);
    // Version n'utilisant que des dcalages de bits pour optimisation
    int b = 128;
    int indfin = 7;
    while ((b & exposant) == 0)
    {
      indfin--;
      b = b >> 1;
    }
    for (int i = 0; i <=indfin; i++)
    {
      if ((exposant % 2) != 0)
      {
        // On n'appele pas produitComp qui crerait un nouvel objet Complexe
        double x = res.x*tampon.x - res.y*tampon.y;
        double y = res.x*tampon.y + res.y * tampon.x;
        res.x = x;
        res.y = y;
      }
      exposant >>= 1;
      // On lve tampon au carr
      double x= tampon.x*tampon.x - tampon.y*tampon.y;
      double y = 2*tampon.x*tampon.y;
      tampon.x = x;
      tampon.y = y;
    }
  }

}
