/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
 *
 * @author  Yves
 * @version 
 */

package types;

// Nature des objets crs
// Contrairement  la version C++, il est inutile de dfinir getNature pour chaque objet
// car cela ne sert que pour les objets dsignables, nommables (points et droites)
// les macros et les variables
// La nature NPoint qui est la nature du point construit sera la nature des points dfinie
// dans CPointAncetre. Il ne sera redfini que pour les points mobiles (pojnts libres et points lis).
// Contrairement au C++, je dfinis une nature AutreObjetGraphique rserv aux autres objets
public class NatureObjet {
  static final public long NAucunObjet                = 0;
  static final public long NPointBase                 = 1;
  static final public long NPoint                     = 1 << 1; //2 puissance1=2
  static final public long NPointLie                  = 1 << 2;
  static final public long NBarycentre								= 1 << 3;
  static final public long NPointDansRepere           = 1 << 4;
  static final public long NPointBaseEnt              = 1 << 5; // N'existait pas ds la version C++
  static final public long NPointParAffixe						= 1 << 6;
  static final public long NPointLiePoint             = 1 << 7;
  static final public long NMacro                     = 1 << 8;
  static final public int indiceMacro = 8;
  static final public long NDroite                    = 1 << 9;
  static final public long NVecteur										= 1 << 10;
  static final public long NSegment                   = 1 << 11;
  static final public long NDemiDroite                = 1 << 12;
  static final public long NCercle                    = 1 << 13;
  static final public long NMarqueAngle               = 1 << 14;
  static final public long NArc                       = 1 << 15;
  static final public long NPolygone                  = 1 << 16;
  static final public long NSurface										= 1 << 17;
  static final public long NLieu											= 1 << 18;
  static final public long NLieuDiscret								= 1 << 19;
  static final public long NLigneBrisee               = 1 << 20;  
  static final public long NLieuObjet                 = 1 << 21;
  static final public long NImage                     = 1 << 22;
  static final public long NGrapheSuiteRec            = 1 << 23;
  static final public long NGrapheSuiteRecComplexe		= 1 << 24;
  static final public long NCommentaire								= 1 << 25;
  static final public long NValeurAffichee					 	= 1 << 26;
  static final public long NMarqueSegment							= 1 << 27;
  static final public long NObjetDuplique             = 1 << 28;
  static final public long NDemiPlan								  = 1 << 29;
  // Attention : Dans la version java, le repre est non graphique 
  // Il est considr comme non graphique car dans les constructions il est choisi en premier dans
  // les objets non graphiques
  static public final long NRepere							      = 1L << 30;	 // Le repre est le seul objet considr comme  la fois
  																																// graphique et non graphqiqe
  static final public long NTransformation            = 1L << 31;  //Le L ncessaire sinon considr comme un int
  static public final long NBipoint                   = 1L << 32;
  static public final long NImplementationProto	      = 1L << 33;
  static public final long NPointInterieurPolygone		= 1L << 34;
  static public final long NLatex											= 1L << 35;
  static public final int nombreTypesObjetsGraphiques = 35;
  
  static final public long NCommentaireOuLatex = NCommentaire | NLatex;
  
  static final public long NToutPoint = NPointBase | NPoint |  NPointLie | NBarycentre |
  	NPointDansRepere | NPointBaseEnt | NPointParAffixe | NPointLiePoint | NPointInterieurPolygone;
  static final public long NToutPointSaufPointLie = NPointBase | NPoint | NBarycentre |
  	NPointDansRepere | NPointBaseEnt | NPointParAffixe | NPointLiePoint | NPointInterieurPolygone;
  
  static final public long NToutObjet	= NToutPoint | NMacro | NDroite | NVecteur | NSegment| NDemiDroite  | NCercle
    | NMarqueAngle | NArc| NPolygone | NSurface | NLieu | NLieuDiscret | NLigneBrisee| NLieuObjet | NImage  
    | NGrapheSuiteRec | NGrapheSuiteRecComplexe |NCommentaire | NLatex | NValeurAffichee | NMarqueSegment
    | NObjetDuplique | NDemiPlan | NRepere;
  static final public long NToutObjetSaufRepere	= NToutPoint | NMacro | NDroite | NVecteur | NSegment| NDemiDroite  | NCercle
    | NMarqueAngle | NArc| NPolygone | NSurface | NLieu | NLieuDiscret | NLigneBrisee| NLieuObjet | NImage  
    | NGrapheSuiteRec | NGrapheSuiteRecComplexe |NCommentaire | NLatex | NValeurAffichee | NMarqueSegment
    | NObjetDuplique | NDemiPlan;
  static final public long NTouteDroite = NDroite | NSegment | NDemiDroite | NVecteur;
  static final public long NTouteDroiteSaufVecteur = NDroite | NSegment | NDemiDroite;
  
  static final public long NObjetLigne = NTouteDroite | NCercle | NMarqueAngle | NArc | NPolygone | NLieu |
  	NLigneBrisee | NGrapheSuiteRec | NGrapheSuiteRecComplexe;
  static final public long NToutCercle = NCercle | NArc; //  revoir
  static final public long NObjetNommable = NToutPoint | NDroite;
  static final public long NPointMobile = NPointBase | NPointLie | NPointBaseEnt | NPointInterieurPolygone;
  static final public long NAffichageLieAPoint = NCommentaire | NLatex | NValeurAffichee | NMacro | NImage;
  static final public long NToutObjetPourLieu = NTouteDroite | NCercle | NArc | NLieu | NLieuDiscret
		|NPolygone | NLigneBrisee | NValeurAffichee | NCommentaire | NLatex | (NToutPoint - NPointBase) | NLieuObjet;
  static final public long NObjetGraphiqueSourcePourConst= NToutPoint | NSegment | NDroite
		| NDemiDroite | NCercle | NArc | NPointDansRepere | NPointLiePoint
		| NPolygone | NLigneBrisee | NPointParAffixe;

  // Aprs le rajout des macros qui sont des objets graphiques, il a 
  // fallu crer un tableau de conversion qui,  la nature d'un objet
  // associe son numro d'objet graphique, compris entre 0 et 
  // NombreTypesDesignables - 1
  
  public static final int nombreTypesDesignables = 32;

  public static final long conversionNumeroNature[] =
    {	NPointBase,NPoint,NPointLie,NBarycentre,NPointDansRepere,NPointBaseEnt,NPointParAffixe,NPointLiePoint,
  		NMacro,NDroite,NVecteur,NSegment,NDemiDroite,NCercle,NMarqueAngle,NArc,NPolygone,NSurface,NLieu,NLieuDiscret,NLigneBrisee,
  		NLieuObjet,NImage,NGrapheSuiteRec,NGrapheSuiteRecComplexe,NCommentaire,NValeurAffichee,
  		NMarqueSegment,NObjetDuplique,NDemiPlan, NPointInterieurPolygone, NLatex};
        

  /** Fonction pas utilise dans la version C++ renvoyant l'indice de la
      nature de l'objet dont la nature est nat
   *  Pour fonctionner correctement, nat ne doit pas tre nul avant appel.
  **/
  public static final int indiceDeNature(long nat) {
    int indnat = 0;
    boolean typeTrouve = false;
    while (!typeTrouve) 
    {
       if ((nat % 2) == 0)
       {
          indnat++;
          nat >>= 1;
       }
       else typeTrouve = true;
    } //while 
    return indiceNatureGraphique(indnat);
  }
  
  /**
   * Du au fait que NPointInterieurPolygone est dfini aprs des objets non graphiques
   * @param indice
   * @return
   */
  public static final int indiceNatureGraphique(int indice) {
  	if (indice < 30)  return indice;
  	else return indice - 4;
  }
  /** Tableau renvoyant une chaine de caractres dcrivant le type d'objet
   *  dont l'indice de nature est indnat
  **/
  public final static String descriptionNature[] = {
    textes.Chaines.natPointLibre, textes.Chaines.natPointConstruit, textes.Chaines.natPointLie, 
    "", "", "",
    "", "", "",
    textes.Chaines.natDroite, textes.Chaines.natVecteur, textes.Chaines.natSegment, textes.Chaines.natDemiDroite,
    textes.Chaines.natCercle, "", textes.Chaines.natArcDeCercle,
    textes.Chaines.natPolygone, "", "",
    "", textes.Chaines.natLigneBrisee,
    "", "", "",
    "", "", "",
    "", "", "",
    ""
  };
}
