/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package types;
/*

 *
 * Created on 17 juillet 2004, 21:28
 */

/**
 *
 * @author  Yves
 * @version 
 */
// Classe dfinissant les diffrents types de remplissage possibles pour les surfaces
// et fournissant des objets TexturePaint servant aux remplissages
import java.awt.image.*;
import java.awt.geom.*;
import java.awt.*;
public abstract class StyleRemplissage {

  
  public static final int QuadrillageVertical = 0;
  public static final int QuadrillageHorizontal = 1;
  public static final int Quadrillage45Haut = 2;
  public static final int Quadrillage45Bas = 3;
  public static final int QuadrillagePlein = 4;
  public static final int QuadrillagePleinTransp = 5;
  public static final int QuadrillageCroise = 6;
  
  public static final TexturePaint getTexturePaint(Color couleur, int styleRemplissage, boolean basseDefinition,
  		Color couleurFond) {
  	if (basseDefinition) {
      BufferedImage bi = new BufferedImage(4, 4, BufferedImage.TYPE_4BYTE_ABGR);
      WritableRaster r = bi.getRaster();
      // On spcifie les 3 composantes pour le trait et le coefficient de transparence en dernier
      int[] couleurTrait = new int[4];
      couleurTrait[0] = couleur.getRed();
      couleurTrait[1] = couleur.getGreen();
      couleurTrait[2] = couleur.getBlue();
      couleurTrait[3] = 255; // Le trait remplace ce qu'il recouvre
      int[] coulFond = new int[4];
      coulFond[0] = couleurFond.getRed();
      coulFond[1] = couleurFond.getGreen();
      coulFond[2] = couleurFond.getBlue();
      Rectangle2D.Double rect = new Rectangle2D.Double(0,0,4,4);
      
	    switch (styleRemplissage) {
	      case QuadrillageVertical :
	
	        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures     
	        for (int i = 0; i < 4; i++) {
	          for (int j = 0; j < 4; j++) {
	            r.setPixel(i, j, coulFond);
	          }
	        }
	        r.setPixel(0,0,couleurTrait);
	        r.setPixel(0,1,couleurTrait);
	        r.setPixel(0,2,couleurTrait);
	        r.setPixel(0,3,couleurTrait);
	        return new TexturePaint(bi, rect);
	      case QuadrillageHorizontal :
	
	        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
	        for (int i = 0; i < 4; i++) {
	          for (int j = 0; j < 4; j++) {
	            r.setPixel(i, j, coulFond);
	          }
	        }
	        r.setPixel(0,0,couleurTrait);
	        r.setPixel(1,0,couleurTrait);
	        r.setPixel(2,0,couleurTrait);
	        r.setPixel(3,0,couleurTrait);
	        return new TexturePaint(bi, rect);
	      case Quadrillage45Bas :
	
	        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
	        for (int i = 0; i < 4; i++) {
	          for (int j = 0; j < 4; j++) {
	            r.setPixel(i, j, coulFond);
	          }
	        }
	        r.setPixel(0,0,couleurTrait);
	        r.setPixel(1,1,couleurTrait);
	        r.setPixel(2,2,couleurTrait);
	        r.setPixel(3,3,couleurTrait);
	        return new TexturePaint(bi, rect);
	      case Quadrillage45Haut :
	
	        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
	        for (int i = 0; i < 4; i++) {
	          for (int j = 0; j < 4; j++) {
	            r.setPixel(i, j, coulFond);
	          }
	        }
	        r.setPixel(3,0,couleurTrait);
	        r.setPixel(2,1,couleurTrait);
	        r.setPixel(1,2,couleurTrait);
	        r.setPixel(0,3,couleurTrait);
	        return new TexturePaint(bi, rect);
	      case QuadrillageCroise :
	
	        coulFond[3] = 0; // La couleur de fond est transparente pour les hachures
	        for (int i = 0; i < 4; i++) {
	          for (int j = 0; j < 4; j++) {
	            r.setPixel(i, j, coulFond);
	          }
	        }
	        r.setPixel(0,0,couleurTrait);
	        r.setPixel(1,1,couleurTrait);
	        r.setPixel(2,2,couleurTrait);
	        r.setPixel(3,3,couleurTrait);
	        r.setPixel(3,0,couleurTrait);
	        r.setPixel(2,1,couleurTrait);
	        r.setPixel(1,2,couleurTrait);
	        r.setPixel(0,3,couleurTrait);
	        return new TexturePaint(bi, rect);        
	        
	      default : return null; // Ne doit pas arriver
	    }
	  }
  	else {
  		// Compltement remani pour version 4.6.2
      BufferedImage bi = null;
    	Graphics2D g2D = null;
    	Rectangle2D.Double rect = null;
    	
	    switch (styleRemplissage) {
	      case QuadrillageVertical :
	        bi = new BufferedImage(8, 8, BufferedImage.TYPE_4BYTE_ABGR);
	        rect = new Rectangle2D.Double(0,0,8,8);
	      	g2D = bi.createGraphics();	
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0f));
	      	g2D.setColor(couleurFond);
	      	g2D.fillRect(0, 0, 8, 8);
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));
	      	g2D.setBackground(couleurFond);
	      	g2D.setColor(couleur);
	      	g2D.drawLine(0, 0, 0, 8);
	        return new TexturePaint(bi, rect);
	        
	      case QuadrillageHorizontal :
	        bi = new BufferedImage(8, 8, BufferedImage.TYPE_4BYTE_ABGR);
	        rect = new Rectangle2D.Double(0,0,8,8);
	      	g2D = bi.createGraphics();		
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0f));
	      	g2D.setColor(couleurFond);
	      	g2D.fillRect(0, 0, 8, 8);
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));
	      	g2D.setBackground(couleurFond);
	      	g2D.setColor(couleur);
	      	g2D.drawLine(0, 0, 8, 0);
	      	// g2D.drawLine(0, 8, 11, 8);
	        return new TexturePaint(bi, rect);
	        
	      case Quadrillage45Bas :
	      	
	        bi = new BufferedImage(12, 12, BufferedImage.TYPE_4BYTE_ABGR);
	        rect = new Rectangle2D.Double(0,0,12,12);
	      	g2D = bi.createGraphics();			
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0f));
	      	g2D.setColor(couleurFond);
	      	g2D.fillRect(0, 0, 12, 12);
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));
	      	g2D.setBackground(couleurFond);
	      	g2D.setColor(couleur);
	      	g2D.drawLine(0, 8, 3, 11);
	      	g2D.drawLine(4, 0, 11, 7);
	        return new TexturePaint(bi, rect);
	        
	      case Quadrillage45Haut :
	        bi = new BufferedImage(12, 12, BufferedImage.TYPE_4BYTE_ABGR);
	        rect = new Rectangle2D.Double(0,0,12,12);
	      	g2D = bi.createGraphics();			
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0f));
	      	g2D.setColor(couleurFond);
	      	g2D.fillRect(0, 0, 12, 12);
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));
	      	g2D.setBackground(couleurFond);
	      	g2D.setColor(couleur);
	      	g2D.drawLine(0, 2, 2, 0);
	      	g2D.drawLine(3, 11, 11, 3);
	        return new TexturePaint(bi, rect);
	        
	      case QuadrillageCroise :
	
	        bi = new BufferedImage(12, 12, BufferedImage.TYPE_4BYTE_ABGR);
	        rect = new Rectangle2D.Double(0,0,12,12);
	      	g2D = bi.createGraphics();			
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 0f));
	      	g2D.setColor(couleurFond);
	      	g2D.fillRect(0, 0, 12, 12);
	      	g2D.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, 1f));
	      	g2D.setBackground(couleurFond);
	      	g2D.setColor(couleur);
	      	g2D.drawLine(0, 2, 2, 0);
	      	g2D.drawLine(3, 11, 11, 3);
	      	g2D.drawLine(0, 8, 3, 11);
	      	g2D.drawLine(4, 0, 11, 7);
	        return new TexturePaint(bi, rect);
	        
	      default : return null; // Ne doit pas arriver
	    } 		
  	}
  }
}